program SoilGen;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

uses
{$IFNDEF FPC}
{$ELSE}
  Interfaces,
{$ENDIF}
  Forms,
  untGUI {frmMAIN},
  untProcesses,
  untBioturbation in 'untBioturbation.pas' {frmBIOTURBATION},
  untClimate in 'untClimate.pas' {frmClimate},
  untImportClimateData in 'untImportClimateData.pas' {frmImportClimateData},
  untNonFatal in 'untNonFatal.pas' {frmNonFatal},
  untEditSoilData in 'untEditSoilData.pas' {frmEditSoilData},
  untImportFertilizerData in 'untImportFertilizerData.pas' {frmImportFertilizerData},
  untAbout in 'untAbout.pas' {frmAbout},
  untSoftConstants in 'untSoftConstants.pas' {frmSoftConstants},
  untEvents in 'untEvents.pas' {frmEvents},
  untMemDump in 'untMemDump.pas' {frmMemDump},
  untDecayProtection in 'untDecayProtection.pas' {frmDecayProtection},
  untEditChemConstants in 'untEditChemConstants.pas' {frmEditChemConstants},
  untEditCinputs in 'untEditCinputs.pas' {frmEditCinputs},
  untEditBedrock in 'untEditBedRock.pas' {frmEditBedRock};
{$R *.res}

begin
  RequireDerivedFormResource := True;
  Application.Initialize;
  untGUI.ClayMigrationAllowed:=false;
  Application.CreateForm(TfrmMAIN, frmMAIN);
  Application.CreateForm(TfrmBIOTURBATION, frmBIOTURBATION);
  Application.CreateForm(TfrmClimate, frmClimate);
  Application.CreateForm(TfrmImportClimateData, frmImportClimateData);
  Application.CreateForm(TfrmNonFatal, frmNonFatal);
  Application.CreateForm(TfrmEditSoilData, frmEditSoilData);
  Application.CreateForm(TfrmImportFertilizerData, frmImportFertilizerData);
  Application.CreateForm(TfrmAbout, frmAbout);
  Application.CreateForm(TfrmSoftConstants, frmSoftConstants);
  Application.CreateForm(TfrmEvents, frmEvents);
  Application.CreateForm(TfrmMemDump, frmMemDump);
  Application.CreateForm(TfrmDecayProtection,  frmDecayProtection);  
  Application.CreateForm(TfrmEditChemConstants,  frmEditChemConstants);
  Application.CreateForm(TfrmEditCinputs,  frmEditCinputs);
  Application.CreateForm(TfrmEditBedRock,  frmEditBedRock);
  Application.Run;
end.
