unit untAbout;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$MODE Delphi}

{$DEBUGINFO ON}

interface

uses
  {$IFDEF UNIX}{$IFDEF UseCThreads}
  cthreads,
  {$ENDIF}{$ENDIF}
  Classes,sysutils, lazfileutils
  { FPC trunk fileinfo reads exe resources as long as you register the appropriate units }
  , fileinfo
  , winpeimagereader {need this for reading exe info}
  , elfreader {needed for reading ELF executables}
  , machoreader {needed for reading MACH-O executables}
  ,
  LCLIntf, LCLType, LMessages, Messages, {SysUtils,} Variants, {Classes,} Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Buttons, fileutil,
  resource, versiontypes, versionresource;

type
  TfrmAbout = class(TForm)
    Image1: TImage;
    mmAbout: TMemo;
    Label1: TLabel;
    btbttnOK: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure btbttnOKClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmAbout: TfrmAbout;

implementation

{$R *.lfm}
uses untGUI;

procedure TfrmAbout.FormCreate(Sender: TObject);
const
  InfoNum = 10;
Var
  FileVerInfo: TFileVersionInfo;
begin
  frmAbout.Caption:='About '+untGUI.VersionID;

  FileVerInfo:=TFileVersionInfo.Create(nil);
  try
    FileVerInfo.ReadFileInfo;
    FileVerInfo.filename :=AppendPathDelim(getcurrentdir)+untGUI.VersionID+'.exe';
    txt:=FileVerInfo.VersionStrings.Values['FileVersion'];
  finally
    FileVerInfo.Free;
  end;

  if length(txt)>0 then txt:=' version: '+txt;

  label1.Caption:=untGUI.VersionID+txt {untLEACHcShell.VersionID};
end;

procedure TfrmAbout.btbttnOKClick(Sender: TObject);
begin
   close;
end;

procedure TfrmAbout.FormActivate(Sender: TObject);
begin
   btbttnOK.SetFocus;
end;

end.
