unit untBioturbation;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$MODE Delphi}

{$DEBUGINFO ON} 
interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin, ExtCtrls, Buttons, FileUtil,Math, lazfileutils,untSoilGenConst;

type
  biorec=record
          year,biodtop,biodmax,biodbot,bioperctop,biopercmax,biopercbot:integer;
         end;
  TfrmBIOTURBATION = class(TForm)
    btbttnSaveClose: TBitBtn;
    GroupBox1: TGroupBox;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    imgdmax: TImage;
    imgdtop: TImage;
    Label15: TLabel;
    imgfractop: TImage;
    lblBIOYEAR1: TLabel;
    lblBIOYEAR2: TLabel;
    imgfracmax: TImage;
    Label16: TLabel;
    imgdbot: TImage;
    Label17: TLabel;
    imgfracbot: TImage;
    lblBIOYEAR3: TLabel;
    GroupBox2: TGroupBox;
    imgBIO: TImage;
    Label1: TLabel;
    Label2: TLabel;
    spndtDBOT: TSpinEdit;
    spndtDMAX: TSpinEdit;
    spndtDTOP: TSpinEdit;
    Label9: TLabel;
    Label11: TLabel;
    spndtBIOYEAR: TSpinEdit;
    Label8: TLabel;
    Label7: TLabel;
    Label6: TLabel;
    Label10: TLabel;
    spndtFRACTOP: TSpinEdit;
    spndtFRACMAX: TSpinEdit;
    spndtFRACBOT: TSpinEdit;
    bttnBIOENTER: TBitBtn;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label18: TLabel;
    lblTonHaYear: TLabel;
    btbttnClose: TBitBtn;
    pndlgOPEN: TOpenDialog;
    btbttnBIOIMPORT: TBitBtn;
    procedure FormActivate(Sender: TObject);
    procedure btbttnSaveCloseClick(Sender: TObject);
    procedure bttnBIOENTERClick(Sender: TObject);
    procedure spndtDTOPChange(Sender: TObject);
    procedure spndtDMAXChange(Sender: TObject);
    procedure spndtDBOTChange(Sender: TObject);
    procedure btbttnCloseClick(Sender: TObject);
    procedure btbttnBIOIMPORTClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Procedure ResetBIO;
    procedure sorteer;
    procedure show1setting;
    procedure LeesData(fn:string);
  end;

var
  frmBIOTURBATION: TfrmBIOTURBATION;
  biodata:array[1..MaxNoYears] of biorec;
  periodeteller:integer;

implementation

{$R *.lfm}
uses untGUI;

Procedure TfrmBIOTURBATION.ResetBIO;
var txt:string;
begin
  str(untGUI.frmMAIN.spndtMultipleYears.Value,txt);
  lblBIOYEAR1.Caption:=txt+' .. 1 years BP' ;
  lblBIOYEAR2.Caption:=txt+' .. 1 years BP' ;
  lblBIOYEAR3.Caption:=txt+' .. 1 years BP' ;
  spndtBIOYEAR.MaxValue:=untGUI.frmMAIN.spndtMultipleYears.Value;
  spndtDTOP.MaxValue:=round(untGUI.DataInit.PDEPTH);
  spndtDMAX.MaxValue:=round(untGUI.DataInit.PDEPTH);
  spndtDBOT.MaxValue:=round(untGUI.DataInit.PDEPTH);
  spndtDTOP.Increment:=round(untGUI.DataInit.DLZ);
  spndtDMAX.Increment:=round(untGUI.DataInit.DLZ);
  spndtDBOT.Increment:=round(untGUI.DataInit.DLZ);
  with imgdtop.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end;   
  with imgdmax.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end;
  with imgdbot.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end;  
  with imgfractop.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end; 
  with imgfracmax.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end; 
  with imgfracbot.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end;
end;

procedure TfrmBIOTURBATION.FormActivate(Sender: TObject);
begin
   pndlgOPEN.InitialDir:=startdir;
   periodeteller:=0; spndtbioyear.Value:=1;
   spndtdtop.Value:=0; spndtdmax.Value:=round(untGUI.DataInit.DLZ);
   spndtdbot.Value:=2*round(untGUI.DataInit.DLZ);
   spndtfractop.Value:=3; spndtfracmax.Value:=3; spndtfracbot.Value:=0;
   ResetBIO;
end;

procedure TfrmBIOTURBATION.btbttnSaveCloseClick(Sender: TObject);
var fbio2,fbio:textfile;
    yeara,yearb,dtopa,dtopb,dmaxa,dmaxb,dbota,dbotb,
    fractopa,fractopb,fracmaxa,fracmaxb,fracbota,fracbotb,
    year,dtop,dmax,dbot,fractop,fracmax,fracbot,dy:integer;
begin
  {}
   if FileExistsUTF8('bioturbSET.txt') { *Converted from FileExists*  } then
   begin
     system.Assign(fbio2,'bioturbINT.txt'); system.Rewrite(fbio2);
     system.Assign(fbio,'bioturbSET.txt');  system.Reset(fbio);
     system.Readln(fbio,yearb,dtopb,dmaxb,dbotb,fractopb,fracmaxb,fracbotb);
     {indien eerste ingevoerde jaar niet het startjaar (1) van de simulatie is:
      aanvullen (alles constant)}
     if yearb>1 then
     begin
       for year:=1 to min(untGUI.frmMAIN.spndtMultipleYears.Value,yearb-1) do
       system.writeln(fbio2,year,' ',dtopb,' ',dmaxb,' ',dbotb,' ',fractopb/1000:5:3,
                            ' ',fracmaxb/1000:5:3,' ',fracbotb/1000:5:3,' extrapolated data');
     end;
     if not (eof(fbio)) then
     repeat
       {stel de *a variabelen gelijk aan de reeds ingelezen *b variabelen}
       yeara:=yearb; dtopa:=dtopb; dmaxa:=dmaxb; dbota:=dbotb;
       fractopa:=fractopb; fracmaxa:=fracmaxb; fracbota:=fracbotb;
       {schrijf de *a veriabelen alvast weg, percentages omzettend in fracties}
       if yearb<=untGUI.frmMAIN.spndtMultipleYears.Value
       then system.writeln(fbio2,yeara,' ',dtopa,' ',dmaxa,' ',dbota,' ',fractopa/1000:5:3,
                            ' ',fracmaxa/1000:5:3,' ',fracbota/1000:5:3,' SET data');
       {lees nieuwe set *.b variabelen}
       system.Readln(fbio,yearb,dtopb,dmaxb,dbotb,fractopb,fracmaxb,fracbotb);
       {indien er meer dan 1 jaar tussen *a en *b zit: interpoleren per jaar}
       if yearb-yeara>1 then
       begin
         dy:=yearb-yeara;
         for year:=min(untGUI.frmMAIN.spndtMultipleYears.Value,yeara)+1
             to    min(untGUI.frmMAIN.spndtMultipleYears.Value,yearb-1)
             do
         begin
           dtop:=round(dtopa+(year-yeara)*(dtopb-dtopa)/dy);
           dmax:=round(dmaxa+(year-yeara)*(dmaxb-dmaxa)/dy);
           dbot:=round(dbota+(year-yeara)*(dbotb-dbota)/dy);
           fractop:=round(fractopa+(year-yeara)*(fractopb-fractopa)/dy);
           fracmax:=round(fracmaxa+(year-yeara)*(fracmaxb-fracmaxa)/dy);
           fracbot:=round(fracbota+(year-yeara)*(fracbotb-fracbota)/dy);
           system.writeln(fbio2,year,' ',dtop,' ',dmax,' ',dbot,' ',
                                fractop/1000:5:3,' ',fracmax/1000:5:3,' ',
                                fracbot/1000:5:3,' interpolated data');
         end;
       end;
     until (eof(fbio));
     {laatste jaar wegschrijven}
     if yearb<=untGUI.frmMAIN.spndtMultipleYears.Value
     then system.writeln(fbio2,yearb,' ',dtopb,' ',dmaxb,' ',dbotb,' ',fractopb/1000:5:3,
                          ' ',fracmaxb/1000:5:3,' ',fracbotb/1000:5:3,' SET data');
     {indien laatste ingevoerde jaar niet het eindjaar van de simulatie is:
      aanvullen (alles constant)}
     if yearb<untGUI.frmMAIN.spndtMultipleYears.Value then
     begin
       for year:=yearb+1 to untGUI.frmMAIN.spndtMultipleYears.Value do
       system.writeln(fbio2,year,' ',dtopb,' ',dmaxb,' ',dbotb,' ',fractopb/1000:5:3,
                            ' ',fracmaxb/1000:5:3,' ',fracbotb/1000:5:3,' extrapolated data');
     end;
     closefile(fbio);
     closefile(fbio2);
     untGUI.frmMAIN.chckbxBioturbation.Checked:=true;
     untGUI.frmMAIN.chckbxBioturbation.enabled:=true;
     untGUI.frmMAIN.btbttnBioturbation.Enabled:=false;
   end;
   close;
end;

procedure TfrmBIOTURBATION.bttnBIOENTERClick(Sender: TObject);
var i,j,xco1,yco1,xco2,yco2:integer;
    fbio:textfile;
begin
   resetBIO;
   {zet settings weg in datastructuur}
   periodeteller:=periodeteller+1;
   biodata[periodeteller].biodtop:=spndtDTOP.Value;
   biodata[periodeteller].biodmax:=spndtDMAX.Value;
   biodata[periodeteller].biodbot:=spndtDBOT.Value;
   biodata[periodeteller].bioperctop:=spndtFRACTOP.Value;
   biodata[periodeteller].biopercmax:=spndtFRACMAX.Value;
   biodata[periodeteller].biopercbot:=spndtFRACBOT.Value;
   biodata[periodeteller].year:=spndtBIOYEAR.maxvalue+1-spndtBIOYEAR.Value;
   {check of een eerdere invoer "overruled" is.
    Indien JA dan het eerder ingevoerde jaar op -1 zetten.
    Dit jaar wordt tijdens sorteren uit de reeks verwijderd.}
   for i:=periodeteller downto 2 do
     for j:=1 to i-1 do
     if (biodata[i].year=biodata[j].year) then biodata[j].year:=-1;
   {bewaar datastructuur na sorteren}
   if periodeteller>1 then sorteer;
   {periodeteller is nu aangepast als er dubbele jaren waren! Alleen het
    laatst ingevoerde jaar is bewaard}
   system.assign(fbio,'bioturbSET.txt'); system.rewrite(fbio);
   for i:=1 to periodeteller do
   with biodata[i] do system.writeln(fbio,year,' ',biodtop,' ',biodmax,' ',biodbot,
                                     ' ',bioperctop,' ',biopercmax,' ',biopercbot);
   closefile(fbio);
   {zet in grafiek}
   for i:=1 to periodeteller-1 do
   begin
     {algemeen}
     xco1:=round(imgdtop.width*biodata[i].year/spndtBIOYEAR.MaxValue);
     xco2:=round(imgdtop.width*biodata[i+1].year/spndtBIOYEAR.MaxValue);
     {top diepte}
     yco1:=round(imgdtop.height*biodata[i].biodtop/untGUI.DataInit.PDEPTH);
     yco2:=round(imgdtop.height*biodata[i+1].biodtop/untGUI.DataInit.PDEPTH);
     imgdtop.Canvas.MoveTo(xco1,yco1);
     imgdtop.Canvas.LineTo(xco2,yco2);
     {max diepte}
     yco1:=round(imgdtop.height*biodata[i].biodmax/untGUI.DataInit.PDEPTH);
     yco2:=round(imgdtop.height*biodata[i+1].biodmax/untGUI.DataInit.PDEPTH);
     imgdmax.Canvas.MoveTo(xco1,yco1);
     imgdmax.Canvas.LineTo(xco2,yco2);
     {bot diepte}
     yco1:=round(imgdtop.height*biodata[i].biodbot/untGUI.DataInit.PDEPTH);
     yco2:=round(imgdtop.height*biodata[i+1].biodbot/untGUI.DataInit.PDEPTH);
     imgdbot.Canvas.MoveTo(xco1,yco1);
     imgdbot.Canvas.LineTo(xco2,yco2); 
     {top %}
     yco1:=imgdtop.height*-round(imgdtop.height*biodata[i].bioperctop/spndtFRACTOP.MaxValue);
     yco2:=imgdtop.height*-round(imgdtop.height*biodata[i+1].bioperctop/spndtFRACTOP.MaxValue);
     imgfractop.Canvas.MoveTo(xco1,yco1);
     imgfractop.Canvas.LineTo(xco2,yco2);
     {max %}
     yco1:=imgdtop.height*-round(imgdtop.height*biodata[i].biopercmax/spndtFRACMAX.MaxValue);
     yco2:=imgdtop.height*-round(imgdtop.height*biodata[i+1].biopercmax/spndtFRACMAX.MaxValue);
     imgfracmax.Canvas.MoveTo(xco1,yco1);
     imgfracmax.Canvas.LineTo(xco2,yco2);
     {bot %}
     yco1:=imgdtop.height*-round(imgdtop.height*biodata[i].biopercbot/spndtFRACBOT.MaxValue);
     yco2:=imgdtop.height*-round(imgdtop.height*biodata[i+1].biopercbot/spndtFRACBOT.MaxValue);
     imgfracbot.Canvas.MoveTo(xco1,yco1);
     imgfracbot.Canvas.LineTo(xco2,yco2);
   end;
end;

procedure TfrmBIOTURBATION.sorteer;
var temp:biorec; i,j,minus:integer; klaar:boolean;
{bubble sort om ingestelde bioturbatiejaren op volgorde te zetten.
 verwijder tevens de dubbele jaren (in feite overruled data)}
begin
  repeat
    klaar:=true;
    for i:=1 to periodeteller-1 do
    begin
      if biodata[i].year>biodata[i+1].year then
      begin
        temp:=biodata[i];
        biodata[i]:=biodata[i+1];
        biodata[i+1]:=temp;
        klaar:=false;
      end;
    end;
  until klaar;
  {dubbele inputs verwijderen door jaren=-1 uit de reeks te verwijderen}
  minus:=0;
  for i:=1 to periodeteller do
  begin
    if (biodata[i].year=-1) then
    begin
      for j:=i to periodeteller-1 do biodata[j]:=biodata[j+1];
      minus:=minus+1;
    end;
  end;
  periodeteller:=periodeteller-minus;
end;

procedure TfrmBIOTURBATION.show1setting;
var xco1,xco2,xco3,xco4,xco5,yco1,yco2,yco3,yco4,yco5,mm,node:integer;
  mass,dmass:real; txt:string;
begin
  with imgBIO.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
    Pen.Color:=clBlue;
    xco1:=0; yco1:=round(height*spndtDTOP.Value/untGUI.DataInit.PDEPTH);
    xco2:=round(width*spndtFRACTOP.Value/spndtFRACTOP.MaxValue);
    yco2:=yco1;
    xco3:=round(width*spndtFRACMAX.Value/spndtFRACMAX.MaxValue);
    yco3:=round(imgdtop.height*spndtDMAX.Value/untGUI.DataInit.PDEPTH);
    xco4:=round(width*spndtFRACBOT.Value/spndtFRACBOT.MaxValue);
    yco4:=round(imgdtop.height*spndtDBOT.Value/untGUI.DataInit.PDEPTH);
    xco5:=0; yco5:=yco4;
    MoveTo(xco1,yco1); LineTo(xco2,yco2); LineTo(xco3,yco3);
    LineTo(xco4,yco4); LineTo(xco5,yco5); LineTo(xco1,yco1);
    Brush.Color:=clYellow; FloodFill(99,79,clBlue,fsborder);
    Brush.Color:=clBlue; FloodFill(1,yco3,clBlue,fsborder);
  end;
  mass:=0;
  for mm:=spndtDTOP.Value to spndtDBOT.Value do
  begin
    if mm<=spndtDMAX.Value
    then
    begin
      if spndtDMAX.Value<>spndtDTOP.Value
      then dmass:=spndtFRACTOP.Value+(spndtFRACMAX.Value-spndtFRACTOP.Value)*
                                     (spndtDMAX.Value-mm)/
                                     (spndtDMAX.Value-spndtDTOP.Value)
      else dmass:=(spndtFRACMAX.Value+spndtFRACTOP.Value)/2
    end
    else
    begin
      if spndtDMAX.Value<>spndtDBOT.Value
      then dmass:=spndtFRACMAX.Value+(spndtFRACBOT.Value-spndtFRACMAX.Value)*
                                   (spndtDBOT.Value-mm)/
                                   (spndtDBOT.Value-spndtDMAX.Value)
      else dmass:=(spndtFRACBOT.Value+spndtFRACMAX.Value)/2
    end;
    node:=trunc(mm/DataInit.DLZ)+1;
    mass:=mass+dmass*untGUI.DataInit.RHO[node]*10/1000;
  end;
  str(mass:3:1,txt);
  lblTonHaYear.Caption:='='+txt+'Mg/ha,yr';
end;

procedure TfrmBIOTURBATION.spndtDTOPChange(Sender: TObject);
begin
  show1setting;
end;

procedure TfrmBIOTURBATION.spndtDMAXChange(Sender: TObject);
begin
  show1setting;
end;

procedure TfrmBIOTURBATION.spndtDBOTChange(Sender: TObject);
begin
  show1setting;
end;

procedure TfrmBIOTURBATION.btbttnCloseClick(Sender: TObject);
begin
   if FileExistsUTF8('bioturbSET.txt') { *Converted from FileExists*  } then DeleteFileUTF8('bioturbSET.txt'); { *Converted from DeleteFile*  }
   if FileExistsUTF8('bioturbINT.txt') { *Converted from FileExists*  } then DeleteFileUTF8('bioturbINT.txt'); { *Converted from DeleteFile*  }
   untGUI.frmMAIN.chckbxBioturbation.Checked:=false;
   untGUI.frmMAIN.chckbxBioturbation.Enabled:=false;
   untGUI.frmMAIN.btbttnBioturbation.Enabled:=true;
   close;
end;

procedure TfrmBIOTURBATION.LeesData(fn:string);
var fbio:textfile; i,xco1,yco1,xco2,yco2,xco3,xco4,xco5,yco3,yco4,yco5,mm,node:integer;
    mass,dmass:real; txt:string;
begin
  try
   periodeteller:=0;
   system.Assign(fbio,fn);  system.Reset(fbio);
   repeat
     periodeteller:=periodeteller+1;
     system.Readln(fbio,biodata[periodeteller].year,biodata[periodeteller].biodtop,
                        biodata[periodeteller].biodmax,biodata[periodeteller].biodbot,
                        biodata[periodeteller].bioperctop,
                        biodata[periodeteller].biopercmax,
                        biodata[periodeteller].biopercbot);
   until eof(fbio);
   {zet in grafiek}
   resetBio;
  {bovenste grafiek: jaar 1}
  with imgBIO.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
    Pen.Color:=clBlue;
    spndtDTOP.Value:=biodata[1].biodtop; spndtFRACTOP.Value:=biodata[1].bioperctop;
    spndtDMAX.Value:=biodata[1].biodmax; spndtFRACMAX.Value:=biodata[1].biopercmax;
    spndtDBOT.Value:=biodata[1].biodbot; spndtFRACBOT.Value:=biodata[1].biopercbot;
    spndtBIOYEAR.Value:=spndtBIOYEAR.MaxValue-biodata[1].year+1;
    xco1:=0; yco1:=round(height*spndtDTOP.Value/untGUI.DataInit.PDEPTH);
    xco2:=round(width*spndtFRACTOP.Value/spndtFRACTOP.MaxValue);
    yco2:=yco1;
    xco3:=round(width*spndtFRACMAX.Value/spndtFRACMAX.MaxValue);
    yco3:=round(height*spndtDMAX.Value/untGUI.DataInit.PDEPTH);
    xco4:=round(width*spndtFRACBOT.Value/spndtFRACBOT.MaxValue);
    yco4:=round(height*spndtDBOT.Value/untGUI.DataInit.PDEPTH);
    xco5:=0; yco5:=yco4;
    MoveTo(xco1,yco1); LineTo(xco2,yco2); LineTo(xco3,yco3);
    LineTo(xco4,yco4); LineTo(xco5,yco5); LineTo(xco1,yco1);
    Brush.Color:=clYellow; FloodFill(99,79,clBlue,fsborder);
    Brush.Color:=clBlue; FloodFill(1,yco3,clBlue,fsborder);
  end;
  mass:=0;
  for mm:=spndtDTOP.Value to spndtDBOT.Value do
  begin
    if mm<=spndtDMAX.Value
    then
    begin
      if spndtDMAX.Value<>spndtDTOP.Value
      then dmass:=spndtFRACTOP.Value+(spndtFRACMAX.Value-spndtFRACTOP.Value)*
                                     (spndtDMAX.Value-mm)/
                                     (spndtDMAX.Value-spndtDTOP.Value)
      else dmass:=(spndtFRACMAX.Value+spndtFRACTOP.Value)/2
    end
    else
    begin
      if spndtDMAX.Value<>spndtDBOT.Value
      then dmass:=spndtFRACMAX.Value+(spndtFRACBOT.Value-spndtFRACMAX.Value)*
                                   (spndtDBOT.Value-mm)/
                                   (spndtDBOT.Value-spndtDMAX.Value)
      else dmass:=(spndtFRACBOT.Value+spndtFRACMAX.Value)/2
    end;
    node:=trunc(mm/DataInit.DLZ)+1;
    mass:=mass+dmass*untGUI.DataInit.RHO[node]*10/1000;
  end;
  str(mass:3:1,txt);
  lblTonHaYear.Caption:='='+txt+'Mg/ha,yr';
  {nu individuele parameters: tijdsverloop}
   for i:=1 to periodeteller-1 do
   begin
     {algemeen}
     xco1:=round(width*biodata[i].year/spndtBIOYEAR.MaxValue);
     xco2:=round(width*biodata[i+1].year/spndtBIOYEAR.MaxValue);
     {top diepte}
     yco1:=round(height*biodata[i].biodtop/untGUI.DataInit.PDEPTH);
     yco2:=round(height*biodata[i+1].biodtop/untGUI.DataInit.PDEPTH);
     imgdtop.Canvas.MoveTo(xco1,yco1);
     imgdtop.Canvas.LineTo(xco2,yco2);
     {max diepte}
     yco1:=round(height*biodata[i].biodmax/untGUI.DataInit.PDEPTH);
     yco2:=round(height*biodata[i+1].biodmax/untGUI.DataInit.PDEPTH);
     imgdmax.Canvas.MoveTo(xco1,yco1);
     imgdmax.Canvas.LineTo(xco2,yco2);
     {bot diepte}
     yco1:=round(height*biodata[i].biodbot/untGUI.DataInit.PDEPTH);
     yco2:=round(height*biodata[i+1].biodbot/untGUI.DataInit.PDEPTH);
     imgdbot.Canvas.MoveTo(xco1,yco1);
     imgdbot.Canvas.LineTo(xco2,yco2);
     {top %}
     yco1:=height-round(height*biodata[i].bioperctop/spndtFRACTOP.MaxValue);
     yco2:=height-round(height*biodata[i+1].bioperctop/spndtFRACTOP.MaxValue);
     imgfractop.Canvas.MoveTo(xco1,yco1);
     imgfractop.Canvas.LineTo(xco2,yco2);
     {max %}
     yco1:=height-round(height*biodata[i].biopercmax/spndtFRACMAX.MaxValue);
     yco2:=height-round(height*biodata[i+1].biopercmax/spndtFRACMAX.MaxValue);
     imgfracmax.Canvas.MoveTo(xco1,yco1);
     imgfracmax.Canvas.LineTo(xco2,yco2);
     {bot %}
     yco1:=height-round(height*biodata[i].biopercbot/spndtFRACBOT.MaxValue);
     yco2:=height-round(height*biodata[i+1].biopercbot/spndtFRACBOT.MaxValue);
     imgfracbot.Canvas.MoveTo(xco1,yco1);
     imgfracbot.Canvas.LineTo(xco2,yco2);
   end;
   closefile(fbio);
   system.assign(fbio,'bioturbSET.txt'); system.rewrite(fbio);
   for i:=1 to periodeteller do
   with biodata[i] do system.writeln(fbio,year,' ',biodtop,' ',biodmax,' ',biodbot,
                                     ' ',bioperctop,' ',biopercmax,' ',biopercbot);
   closefile(fbio);

  except
     showmessage('Error reading '+fn+'. Check its format and import again.');
  end;
end;

procedure TfrmBIOTURBATION.btbttnBIOIMPORTClick(Sender: TObject);
begin
  if pndlgOPEN.Execute
  then LeesData(pndlgOPEN.FileName);
end;

end.
