unit untClimate;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$MODE Delphi}

{$DEBUGINFO ON} 
interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin, Buttons, ExtCtrls, FileUtil,untImportClimateData,lazfileutils, untSoilGenConst;

type
  ClimRec=record
    BPyear:integer;
    P,PE,JanT,JulT,PlantResSum,ManureSum:integer;
  end;
  ClimRec2=record
    VEG,Period:string;
  end;
  TfrmClimate = class(TForm)
    GroupBox1: TGroupBox;
    spndtP: TSpinEdit;
    spndtPE: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    spndtJanT: TSpinEdit;
    Label5: TLabel;
    spndtJulT: TSpinEdit;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    spndtPlantResSum: TSpinEdit;
    Label9: TLabel;
    spndtManureSum: TSpinEdit;
    Label10: TLabel;
    cmbbxVEG: TComboBox;
    edtPERIOD: TEdit;
    Label11: TLabel;
    Label12: TLabel;
    pndlgOPEN: TOpenDialog;
    btbttnBIOIMPORT: TBitBtn;
    spndtCLIYEAR: TSpinEdit;
    Label13: TLabel;
    GroupBox2: TGroupBox;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    imgJanT: TImage;
    imgP: TImage;
    Label17: TLabel;
    imgPE: TImage;
    lblBIOYEAR1: TLabel;
    lblBIOYEAR2: TLabel;
    imgJulT: TImage;
    Label18: TLabel;
    imgPlant: TImage;
    Label19: TLabel;
    imgManure: TImage;
    lblBIOYEAR3: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    bttnBIOENTER: TBitBtn;
    btbttnSaveClose: TBitBtn;
    btbttnClose: TBitBtn;
    lblavailable: TLabel;
    procedure btbttnBIOIMPORTClick(Sender: TObject);
    procedure bttnBIOENTERClick(Sender: TObject);
    procedure spndtCLIYEARChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure btbttnSaveCloseClick(Sender: TObject);
    procedure btbttnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure LeesData(fn:string);
    procedure ResetClim;
    procedure sorteer;
    procedure NotifyEConVertError;
    procedure ColorIt(color1,color2:TColor);
  end;

var
  frmClimate: TfrmClimate;
  climdata: array [1..MaxNoYears] of climrec;
  climdata2: array[1..MaxnoYears] of climrec2;
  periodeteller:integer;
  headertxt:string;

implementation

{$R *.lfm}
uses untGUI;

procedure TfrmClimate.btbttnBIOIMPORTClick(Sender: TObject);
begin
  if FileExistsUTF8('tempClimate.txt') { *Converted from FileExists*  } then DeleteFileUTF8('tempClimate.txt'); { *Converted from DeleteFile*  }
  untImportClimateData.frmImportClimateData.ShowModal;
  spndtCLIYEARChange(Sender);
  {LeesData('tempClimate.txt') called before exit rmImportClimateData}
  {if pndlgOPEN.Execute
  then
  begin
    LeesData(pndlgOPEN.FileName);
  end;}
end;

procedure TfrmClimate.LeesData(fn:string);
var fcl:textfile; i,xco1,yco1,xco2,yco2:integer;
    rP,rPE,rJanT,rJulT,rPlantResSum,rManureSum:real; txt:string;
begin
  try
   periodeteller:=0;
   for i:=1 to MaxNoYears do
   begin
     climdata[i].BPyear:=-1;
     climdata[i].P:=0;           climdata[i].PE:=0;
     climdata[i].JanT:=0;        climdata[i].JulT:=0;
     climdata[i].PlantResSum:=0; climdata[i].ManureSum:=0;
   end;
   system.Assign(fcl,fn);  system.Reset(fcl); system.Readln(fcl,headertxt);
   repeat
     periodeteller:=periodeteller+1; i:=periodeteller;
     system.Read(fcl,climdata[i].BPyear);
     system.Read(fcl,rP);  climdata[i].P:=round(rP);
     system.Read(fcl,rPE); climdata[i].PE:=round(rPE);
     system.Read(fcl,rJanT);
     system.Read(fcl,rJulT);
     system.Read(fcl,rPlantResSum);
     system.Read(fcl,rManureSum);
     climdata[i].JanT:=round(rJanT*10);
     climdata[i].JulT:=round(rJulT*10);
     climdata[i].PlantResSum:=round(rPlantResSum*10);
     climdata[i].ManureSum:=round(rManureSum*10);
     system.readln(fcl,txt);
     while pos('  ',txt)>0 do delete(txt,pos(' ',txt),1);
     if txt[1]=' ' then delete(txt,1,1);
     if pos(' ',txt)>0 then
     begin
       climdata2[i].VEG:=copy(txt,1,pos(' ',txt));
       delete(txt,1,pos(' ',txt));
       climdata2[i].Period:=txt;
     end
     else begin
            climdata2[i].VEG:=txt;
            climdata2[i].Period:='';
          end;
   until eof(fcl);
   {zet in grafiekjes}
   ResetClim;

  {nu individuele parameters: tijdsverloop}
   for i:=1 to periodeteller-1 do
   begin
     {algemeen: tijdsassen}
     xco1:=round(-imgP.width*(climdata[i].BPyear-spndtCLIYEAR.MaxValue)/spndtCLIYEAR.MaxValue);
     xco2:=round(-imgP.width*(climdata[i+1].BPyear-spndtCLIYEAR.MaxValue)/spndtCLIYEAR.MaxValue);
     {P }
     yco1:=round(imgP.height-imgP.height*climdata[i].P/spndtP.maxvalue);
     yco2:=round(imgP.height-imgP.height*climdata[i+1].P/spndtP.maxvalue);
     imgP.Canvas.MoveTo(xco1,yco1);
     imgP.Canvas.LineTo(xco2,yco2);
     {PE}
     yco1:=round(imgP.height-imgP.height*climdata[i].PE/spndtPE.maxvalue);
     yco2:=round(imgP.height-imgP.height*climdata[i+1].PE/spndtPE.maxvalue);
     imgPE.Canvas.MoveTo(xco1,yco1);
     imgPE.Canvas.LineTo(xco2,yco2);
     {JanT}
     yco1:=round(imgP.height-imgP.height*(climdata[i].JanT-spndtJanT.minvalue)/(spndtJanT.maxvalue-spndtJanT.minvalue));
     yco2:=round(imgP.height-imgP.height*(climdata[i+1].JanT-spndtJanT.minvalue)/(spndtJanT.maxvalue-spndtJanT.minvalue));
     imgJanT.Canvas.MoveTo(xco1,yco1);
     imgJanT.Canvas.LineTo(xco2,yco2);
     {JulT}
     yco1:=round(imgP.height-imgP.height*(climdata[i].JulT-spndtJulT.minvalue)/(spndtJulT.maxvalue-spndtJulT.minvalue));
     yco2:=round(imgP.height-imgP.height*(climdata[i+1].JulT-spndtJulT.minvalue)/(spndtJulT.maxvalue-spndtJulT.minvalue));
     imgJulT.Canvas.MoveTo(xco1,yco1);
     imgJulT.Canvas.LineTo(xco2,yco2);
     {Plant residue}
     yco1:=round(imgP.height-imgP.height*climdata[i].PlantResSum/spndtPlantResSum.maxvalue);
     yco2:=round(imgP.height-imgP.height*climdata[i+1].PlantResSum/spndtPlantResSum.maxvalue);
     imgPlant.Canvas.MoveTo(xco1,yco1);
     imgPlant.Canvas.LineTo(xco2,yco2);
     {Manure C}
     yco1:=round(imgP.height-imgP.height*climdata[i].ManureSum/spndtManureSum.maxvalue);
     yco2:=round(imgP.height-imgP.height*climdata[i+1].ManureSum/spndtManureSum.maxvalue);
     imgManure.Canvas.MoveTo(xco1,yco1);
     imgManure.Canvas.LineTo(xco2,yco2);
   end;
   closefile(fcl);
   system.assign(fcl,'ClimateSET.txt'); system.rewrite(fcl); system.writeln(fcl,headertxt);
   for i:=1 to periodeteller do
   with climdata[i] do
      system.writeln(fcl,BPyear,' ',P,' ',PE,' ',(JanT/10):5:1,
                          ' ',(JulT/10):5:1,' ',(PlantResSum/10):4:1,' ',(ManureSum/10):4:1,
                          ' ',climdata2[i].VEG,' ',climdata2[i].Period);
   closefile(fcl);

  except
     showmessage('Error reading '+fn+'. Check its format and import again.');
  end;
end;

procedure TfrmClimate.ResetClim;
var txt:string;
begin
  str(untGUI.frmMAIN.spndtMultipleYears.Value,txt);
  lblBIOYEAR1.Caption:=txt+'... 1 years BP';
  lblBIOYEAR2.Caption:=txt+'... 1 years BP';
  lblBIOYEAR3.Caption:=txt+'... 1 years BP';
  spndtCLIYEAR.MaxValue:=untGUI.frmMAIN.spndtMultipleYears.Value;
  with imgP.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end;
  with imgPE.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end;
  with imgJanT.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end;
  with imgJulT.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end;
  with imgPlant.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end;
  with imgmanure.Canvas do
  begin
    Pen.Color:=clBlack; Brush.Color:=clWhite;
    Rectangle(0,0,width,height);
  end;
end;

procedure TfrmClimate.bttnBIOENTERClick(Sender: TObject);
var i,j,xco1,yco1,xco2,yco2:integer;
    fcli:textfile;
begin
   resetclim;
   {zet settings weg in datastructuur}
   periodeteller:=periodeteller+1;
   climdata[periodeteller].P:=spndtP.Value;
   climdata[periodeteller].PE:=spndtPE.Value;
   climdata[periodeteller].JanT:=spndtJanT.Value;
   climdata[periodeteller].JulT:=spndtJulT.Value;
   climdata[periodeteller].PlantResSum:=spndtPlantResSum.Value;
   climdata[periodeteller].ManureSum:=spndtManureSum.Value;
   climdata[periodeteller].BPyear:=spndtCLIYEAR.Value;
   climdata2[periodeteller].Period:=edtPeriod.Text;
   climdata2[periodeteller].VEG:=cmbbxVEG.Text;
   {check of een eerdere invoer "overruled" is.
    Indien JA dan het eerder ingevoerde jaar op -1 zetten.
    Dit jaar wordt tijdens sorteren uit de reeks verwijderd.}
   for i:=periodeteller downto 2 do
     for j:=1 to i-1 do
     if (climdata[i].BPyear=climdata[j].BPyear) then climdata[j].BPyear:=-1;
   {bewaar datastructuur na sorteren}
   if periodeteller>1 then sorteer;
   {periodeteller is nu aangepast als er dubbele jaren waren! Alleen het
    laatst ingevoerde jaar is bewaard}
   system.assign(fcli,'climateSET.txt'); system.rewrite(fcli);
   if length(headertxt)<2
   then headertxt:='YearBP  P  EP  JanT  JulT  PlantsInC(t/ha,y)  ManureInC(t/ha,y)  Vegetation  Period';
   system.writeln(fcli,headertxt);
   for i:=1 to periodeteller do
   with climdata[i] do
      system.writeln(fcli,BPyear,' ',P,' ',PE,' ',(JanT/10):5:1,
                          ' ',(JulT/10):5:1,' ',(PlantResSum/10):4:1,' ',(ManureSum/10):4:1,
                          ' ',climdata2[i].VEG,' ',climdata2[i].Period);
   closefile(fcli);
   lblAvailable.Caption:='Data values available';  ColorIt(clgreen,clyellow);
   {zet in grafiek}
   for i:=1 to periodeteller-1 do
   begin
     {algemeen: tijdsassen}
     xco1:=round(-imgP.width*(climdata[i].BPyear-spndtCLIYEAR.MaxValue)/spndtCLIYEAR.MaxValue);
     xco2:=round(-imgP.width*(climdata[i+1].BPyear-spndtCLIYEAR.MaxValue)/spndtCLIYEAR.MaxValue);
     {P }
     yco1:=round(imgP.height-imgP.height*climdata[i].P/spndtP.maxvalue);
     yco2:=round(imgP.height-imgP.height*climdata[i+1].P/spndtP.maxvalue);
     imgP.Canvas.MoveTo(xco1,yco1);
     imgP.Canvas.LineTo(xco2,yco2);
     {PE}
     yco1:=round(imgP.height-imgP.height*climdata[i].PE/spndtPE.maxvalue);
     yco2:=round(imgP.height-imgP.height*climdata[i+1].PE/spndtPE.maxvalue);
     imgPE.Canvas.MoveTo(xco1,yco1);
     imgPE.Canvas.LineTo(xco2,yco2);
     {JanT}
     yco1:=round(imgP.height-imgP.height*(climdata[i].JanT-spndtJanT.minvalue)/(spndtJanT.maxvalue-spndtJanT.minvalue));
     yco2:=round(imgP.height-imgP.height*(climdata[i+1].JanT-spndtJanT.minvalue)/(spndtJanT.maxvalue-spndtJanT.minvalue));
     imgJanT.Canvas.MoveTo(xco1,yco1);
     imgJanT.Canvas.LineTo(xco2,yco2);
     {JulT}
     yco1:=round(imgP.height-imgP.height*(climdata[i].JulT-spndtJulT.minvalue)/(spndtJulT.maxvalue-spndtJulT.minvalue));
     yco2:=round(imgP.height-imgP.height*(climdata[i+1].JulT-spndtJulT.minvalue)/(spndtJulT.maxvalue-spndtJulT.minvalue));
     imgJulT.Canvas.MoveTo(xco1,yco1);
     imgJulT.Canvas.LineTo(xco2,yco2);
     {Plant residue}
     yco1:=round(imgP.height-imgP.height*climdata[i].PlantResSum/spndtPlantResSum.maxvalue);
     yco2:=round(imgP.height-imgP.height*climdata[i+1].PlantResSum/spndtPlantResSum.maxvalue);
     imgPlant.Canvas.MoveTo(xco1,yco1);
     imgPlant.Canvas.LineTo(xco2,yco2);
     {Manure C}
     yco1:=round(imgP.height-imgP.height*climdata[i].ManureSum/spndtManureSum.maxvalue);
     yco2:=round(imgP.height-imgP.height*climdata[i+1].ManureSum/spndtManureSum.maxvalue);
     imgManure.Canvas.MoveTo(xco1,yco1);
     imgManure.Canvas.LineTo(xco2,yco2);   end;
end;

procedure TfrmClimate.sorteer;
var temp:climrec; temp2:climrec2; i,j,minus:integer; klaar:boolean;
{bubble sort om ingestelde jaren op volgorde te zetten.
 verwijder tevens de dubbele jaren (in feite overruled data)}
begin
  repeat
    klaar:=true;
    for i:=1 to periodeteller-1 do
    begin
      if climdata[i].BPyear<climdata[i+1].BPyear then
      begin
        temp:=climdata[i];           temp2:=climdata2[i];
        climdata[i]:=climdata[i+1];  climdata2[i]:=climdata2[i+1];
        climdata[i+1]:=temp;         climdata2[i+1]:=temp2;
        klaar:=false;
      end;
    end;
  until klaar;
  {dubbele inputs verwijderen door jaren=-1 uit de reeks te verwijderen}
  minus:=0;
  for i:=1 to periodeteller do
  begin
    if (climdata[i].BPyear=-1) then
    begin
      for j:=i to periodeteller-1 do
      begin climdata[j]:=climdata[j+1]; climdata2[j]:=climdata2[j+1]; end;
      minus:=minus+1;
    end;
  end;
  periodeteller:=periodeteller-minus;
end;

procedure TfrmClimate.spndtCLIYEARChange(Sender: TObject);
var i,teller:integer;  found:boolean;
begin
 try
  begin
   teller:=spndtCLIYEAR.value;
   i:=1; found:=false;
   while (i<=MaxNoYears) and (not(found)) do
   begin
    if climdata[i].BPyear=teller then
    begin
     found:=true;
     spndtP.Value:=climdata[i].P;
     spndtPE.Value:=climdata[i].PE;
     spndtJanT.Value:=climdata[i].JanT;
     spndtJulT.Value:=climdata[i].JulT;
     spndtPlantResSum.Value:=climdata[i].PlantResSum;
     spndtManureSum.Value:=climdata[i].ManureSum;
     cmbbxVEG.Text:=climdata2[i].VEG;
     edtPeriod.Text:=climdata2[i].Period;
     lblAvailable.Caption:='Data values available';  ColorIt(clgreen,clyellow);
    end;
    i:=i+1;
   end;
   if not(found) then
   begin
     lblAvailable.Caption:='No data values available';  
     ColorIt(clyellow,clblack);
   end;
  end;
  except
    on EConvertError do NotifyEConVertError;
  end;
end;

procedure TfrmClimate.FormActivate(Sender: TObject);
var i:integer;
begin
  {for i:=1 to MaxNoYears do climdata[i].BPyear:=-1; }
 pndlgOPEN.InitialDir:=startdir;
 spndtCLIYEAR.MaxValue:=MaxNoyears;
end;

procedure TfrmClimate.NotifyEConVertError;
begin
  showmessage('While editing, always leave a numeric code in this field.');
end;

procedure TfrmClimate.ColorIt(color1,color2:TColor);
begin
  lblAvailable.Color:=color1;    lblAvailable.Font.Color:=color2;
  spndtP.Color:=color1;          spndtPE.Color:=color1;
  spndtJanT.Color:=color1;       spndtJulT.Color:=color1;
  spndtPlantResSum.Color:=color1;spndtManureSum.Color:=color1;
  cmbbxVEG.Color:=color1;        edtPeriod.Color:=color1;
  spndtP.Font.Color:=color2;          spndtPE.Font.Color:=color2;
  spndtJanT.Font.Color:=color2;       spndtJulT.Font.Color:=color2;
  spndtPlantResSum.Font.Color:=color2;spndtManureSum.Font.Color:=color2;
  cmbbxVEG.Font.Color:=color2;        edtPeriod.Font.Color:=color2;
  frmClimate.Refresh;
end;

procedure TfrmClimate.btbttnSaveCloseClick(Sender: TObject);
var fcli:textfile; i:integer;
begin
   system.assign(fcli,'Climate.txt'); system.rewrite(fcli);
   if length(headertxt)<2
   then headertxt:='YearBP  P  EP  JanT  JulT  PlantsInC(t/ha,y)  ManureInC(t/ha,y)  Vegetation  Period';
   system.writeln(fcli,headertxt);
   for i:=1 to periodeteller do
   with climdata[i] do
      system.writeln(fcli,BPyear,' ',P,' ',PE,' ',(JanT/10):5:1,
                          ' ',(JulT/10):5:1,' ',(PlantResSum/10):4:1,' ',(ManureSum/10):4:1,
                          ' ',climdata2[i].VEG,' ',climdata2[i].Period);
   closefile(fcli);
   untGUI.frmMAIN.chckbxClimate.Checked:=true;
   untGUI.frmMAIN.chckbxClimate.enabled:=true;
   untGUI.frmMAIN.btbttnClimateChange.Enabled:=false;
   Close;
end;

procedure TfrmClimate.btbttnCloseClick(Sender: TObject);
begin
  if FileExistsUTF8('Climate.txt') { *Converted from FileExists*  } then DeleteFileUTF8('Climate.txt'); { *Converted from DeleteFile*  }
  if FileExistsUTF8('ClimateINT.txt') { *Converted from FileExists*  } then DeleteFileUTF8('ClimateINT.txt'); { *Converted from DeleteFile*  }
  if FileExistsUTF8('ClimateSET.txt') { *Converted from FileExists*  } then DeleteFileUTF8('ClimateSET.txt'); { *Converted from DeleteFile*  }
  untGUI.frmMAIN.chckbxClimate.Checked:=false;
  untGUI.frmMAIN.chckbxClimate.Enabled:=false;
  untGUI.frmMAIN.btbttnClimateChange.Enabled:=true;
  Close;
end;

procedure TfrmClimate.FormCreate(Sender: TObject);
begin
   spndtCLIYEAR.MaxValue:=MaxNoyears;
end;

end.
