unit untEditSoilData;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$MODE Delphi}

{$DEBUGINFO ON}
interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, Spin, Grids, untProcesses, math, untSoilGenConst;

type
  {real=double;}
  Profile = array[1..MaxLayers] of real;

  { TfrmEditSoilData }

  TfrmEditSoilData = class(TForm)
    btbttnBOTTOM: TBitBtn;
    btbttnCopy: TBitBtn;
    btbttnDOWN: TBitBtn;
    btbttnTOP: TBitBtn;
    btbttnUP: TBitBtn;
    bttnAmorphite: TButton;
    bttnOtherite: TButton;
    bttnSaveAmorphite: TButton;
    bttnNoSaveAmorphite1: TButton;
    bttnSaveOtherite: TButton;
    bttnNoSaveOtherite: TButton;
    chckbxProceed: TCheckBox;
    edtACALCITE: TEdit;
    edtAGYPS: TEdit;
    edtAlks: TEdit;
    edtAls: TEdit;
    edtAlx: TEdit;
    edtAPCO2: TEdit;
    edtAT: TEdit;
    edtAW: TEdit;
    edtCas: TEdit;
    edtCax: TEdit;
    edtCEC: TEdit;
    edtCLAY: TEdit;
    edtCls: TEdit;
    edtDOCs: TEdit;
    edtfrAlbite: TEdit;
    edtfrAmorphite: TEdit;
    edtfrAnorthite: TEdit;
    edtfrAugite: TEdit;
    edtfrBiotite: TEdit;
    edtfrChlorite: TEdit;
    edtfrFayalite: TEdit;
    edtfrFosterite: TEdit;
    edtfrGibbsite: TEdit;
    edtfrHornblende: TEdit;
    edtfrIllite: TEdit;
    edtfrKaolinite: TEdit;
    edtfrKfeldspar: TEdit;
    edtfrMontmorillonite: TEdit;
    edtfrMuscovite: TEdit;
    edtfrOtherite: TEdit;
    edtfrQuartz: TEdit;
    edtStones: TEdit;
    edtH2Orgs: TEdit;
    edtHOrgs: TEdit;
    edtHx: TEdit;
    edtKs: TEdit;
    edtKx: TEdit;
    edtMineralsFileName: TEdit;
    edtMgs: TEdit;
    edtMgx: TEdit;
    edtNAs: TEdit;
    edtNax: TEdit;
    edtOC: TEdit;
    edtOrgs: TEdit;
    edtRHO: TEdit;
    edtSAND: TEdit;
    edtSC1: TEdit;
    edtSC2: TEdit;
    edtSC3: TEdit;
    edtSC7: TEdit;
    edtSC8: TEdit;
    edtSILT: TEdit;
    edtGravel: TEdit;
    edtSO4s: TEdit;
    GroupBox1: TGroupBox;
    btbttnQuitNoChange: TBitBtn;
    btbttnQuitWithChanges: TBitBtn;
    GroupBox2: TGroupBox;
    grpbxAmorphite: TGroupBox;
    grpbxOtherite: TGroupBox;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    Label28: TLabel;
    Label29: TLabel;
    Label30: TLabel;
    Label31: TLabel;
    Label32: TLabel;
    Label33: TLabel;
    Label34: TLabel;
    Label35: TLabel;
    Label36: TLabel;
    Label37: TLabel;
    Label38: TLabel;
    Label39: TLabel;
    Label40: TLabel;
    Label41: TLabel;
    Label42: TLabel;
    Label43: TLabel;
    Label44: TLabel;
    Label45: TLabel;
    Label46: TLabel;
    Label47: TLabel;
    Label48: TLabel;
    Label49: TLabel;
    Label50: TLabel;
    Label51: TLabel;
    Label52: TLabel;
    Label73: TLabel;
    Label74: TLabel;
    Label75: TLabel;
    Label76: TLabel;
    Label77: TLabel;
    Label78: TLabel;
    Label79: TLabel;
    Label80: TLabel;
    Label81: TLabel;
    Label82: TLabel;
    Label83: TLabel;
    lblAmorphite: TLabel;
    Label9: TLabel;
    lblOtherite: TLabel;
    lblCompartment: TLabel;
    lblDepths: TLabel;
    lblNoCompartments: TLabel;
    spndtPDEPTH: TSpinEdit;
    spndtDLZ: TSpinEdit;
    spndtMDIFF: TSpinEdit;
    spndtDISP: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    edtRHOS2: TEdit;
    Label3: TLabel;
    edtRHOS1: TEdit;
    Label4: TLabel;
    edtRHOS3: TEdit;
    Label7: TLabel;
    edtDELM: TEdit;
    Label8: TLabel;
    GroupBox3: TGroupBox;
    Label53: TLabel;
    edtLeachcFileName: TEdit;
    Label54: TLabel;
    edtDay: TEdit;
    Label55: TLabel;
    edtFRAC1: TEdit;
    Label56: TLabel;
    edtIRRIGA: TEdit;
    Label57: TLabel;
    edtIRATEA: TEdit;
    Label58: TLabel;
    edtARSE1: TEdit;
    Label59: TLabel;
    edtARSE2: TEdit;
    Label60: TLabel;
    edtARSE3: TEdit;
    Label61: TLabel;
    edtARSE4: TEdit;
    Label62: TLabel;
    edtARSE5: TEdit;
    Label63: TLabel;
    edtARSE6: TEdit;
    Label64: TLabel;
    edtARSE7: TEdit;
    Label65: TLabel;
    edtARSE12: TEdit;
    lblWEEKDAY: TLabel;
    btbttnFirst: TBitBtn;
    btbttnPREV: TBitBtn;
    btbttnNEXT: TBitBtn;
    btbttnLAST: TBitBtn;
    Label67: TLabel;
    chckbxWEEK: TCheckBox;
    Label66: TLabel;
    Label68: TLabel;
    Label69: TLabel;
    edtWEEK: TEdit;
    Label70: TLabel;
    edtPAN: TEdit;
    Label71: TLabel;
    edtWTDEP: TEdit;
    Label72: TLabel;
    edtAVETEM: TEdit;
    edtAMP: TEdit;
    strgrdAmorphite: TStringGrid;
    strgrdOtherite: TStringGrid;
    procedure btbttnQuitNoChangeClick(Sender: TObject);
    procedure btbttnQuitWithChangesClick(Sender: TObject);
    procedure bttnAmorphiteClick(Sender: TObject);
    procedure bttnNoSaveAmorphite1Click(Sender: TObject);
    procedure bttnNoSaveOtheriteClick(Sender: TObject);
    procedure bttnOtheriteClick(Sender: TObject);
    procedure bttnSaveAmorphiteClick(Sender: TObject);
    procedure bttnSaveOtheriteClick(Sender: TObject);
    procedure edtAlxChange(Sender: TObject);
    procedure edtCaxChange(Sender: TObject);
    procedure edtCLAYChange(Sender: TObject);
    procedure edtGravelChange(Sender: TObject);
    procedure edtHxChange(Sender: TObject);
    procedure edtKxChange(Sender: TObject);
    procedure edtMgxChange(Sender: TObject);
    procedure edtNaxChange(Sender: TObject);
    procedure edtOCChange(Sender: TObject);
    procedure edtSILTChange(Sender: TObject);
    procedure edtStonesChange(Sender: TObject);
    procedure spndtDISPChange(Sender: TObject);
    procedure spndtDLZChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure btbttnBOTTOMClick(Sender: TObject);
    procedure btbttnUPClick(Sender: TObject);
    procedure btbttnDOWNClick(Sender: TObject);
    procedure btbttnTOPClick(Sender: TObject);
    procedure spndtPDEPTHExit(Sender: TObject);
    procedure spndtDLZExit(Sender: TObject);
    procedure spndtPDEPTHChange(Sender: TObject);
    procedure spndtDISPExit(Sender: TObject);
    procedure spndtMDIFFExit(Sender: TObject);
    procedure edtDELMExit(Sender: TObject);
    procedure edtRHOS2Exit(Sender: TObject);
    procedure edtRHOS1Exit(Sender: TObject);
    procedure edtRHOS3Exit(Sender: TObject);
    procedure btbttnCopyClick(Sender: TObject);
    procedure chckbxWEEKClick(Sender: TObject);
    procedure btbttnFirstClick(Sender: TObject);
    procedure btbttnPREVClick(Sender: TObject);
    procedure btbttnNEXTClick(Sender: TObject);
    procedure btbttnLASTClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ShowLayer(layer:integer);
    function Val2Str(value:real):string;
    procedure Actief;
    function CheckVal(txt:string):real;
    procedure storelayer(layer:integer);
    procedure StoreTime(day,week:integer);
    procedure ShowTime(day,week:integer);
    procedure ActiefTime;
    procedure CheckData;
    procedure ReadMinFile;
  end;

var
  frmEditSoilData: TfrmEditSoilData;
  compartment,nocompartments,day,week:integer;
  SEA       : array[1..MaxChem] of profile;
  AXSE      : array[1..MaxChem] of profile;
  CLAY,SILT,SAND,OC,RHO,AW,ATEM,CALCITEA,GYPA,APCO2,
  SC1,SC2,SC3,SC7,SC8,LAMBDA,ACEC: profile;
  DELM,PDEPTH,DLZ,DIFO,DISP,RHOS1,RHOS2,RHOS3:real;
  FRAC1,IRRIGA,IRATEA
            : array[1..MaxNoIrrig] of real;
  ARSE      : array[1..MaxChem,1..MaxNoIrrig] of real;
  PAN,WTDEP,AVETEM,AMP
            : array[1..MaxNoWeeks] of real;
  TMPMinPools: array [1..nrminerals] of profile;
  TMPGibbsitepool: profile;

implementation

{$R *.lfm}
uses untGUI;

procedure TfrmEditSoilData.btbttnQuitNoChangeClick(Sender: TObject);
begin
  close;
end;

procedure TfrmEditSoilData.btbttnQuitWithChangesClick(Sender: TObject);
var layer,chem,dy,wk,i,j:integer; DUM:real;
  fWE:textfile;
begin
  storelayer(compartment);
  for layer:=1 to MaxLayers do
  begin;
    for chem:=1 to MaxChem do
    begin
      untGUI.DataInit.SEA[chem][layer]:=SEA[chem][layer];
      untGUI.DataInit.AXSE[chem][layer]:=AXSE[chem][layer];
    end;
    untGUI.DataInit.CLAY[layer]:=CLAY[layer];
    untGUI.DataInit.SILT[layer]:=SILT[layer] ;
    untGUI.DataInit.OC[layer]:=OC[layer];
    untGUI.DataInit.STONES[layer]:=STONES[layer];
    untGUI.DataInit.GRAVEL[layer]:=GRAVEL[layer];
    untGUI.DataInit.RHO[layer]:=RHO[layer];
    untGUI.DataInit.AW[layer]:=AW[layer];
    untGUI.DataInit.ATEM[layer]:=ATEM[layer];
    untGUI.DataInit.TEM[layer]:=ATEM[layer];
    untGUI.DataInit.lambda[layer]:=DISP; {assuming dispersivity constant!}
    {calcite and gypsum converted to kg/m2 soil: mass=RHO*DLZ*mfr*(1-stonefr-gravelfr), backconversion in untLeachCShell.frmMAIN.writedata}
    untGUI.DataInit.CALCITEA[layer]:=CALCITEA[layer]*RHO[layer]*dlz*(1-(STONES[layer]+GRAVEL[layer])/100); {PF23: all calcite mass in the fine earth, so correct for stones=gravel}
    untGUI.DataInit.GYPA[layer]:=GYPA[layer]*RHO[layer]*dlz*(1-(STONES[layer]+GRAVEL[layer])/100);         {PF23: all gypsum mass in the fine earth, so correct for stones=gravel}
    untGUI.DataInit.APCO2[layer]:=APCO2[layer];
    untGUI.DataInit.SC1[layer]:=SC1[layer];
    untGUI.DataInit.SC2[layer]:=SC2[layer];
    untGUI.DataInit.SC3[layer]:=SC3[layer];
    untGUI.DataInit.SC7[layer]:=SC7[layer];
    untGUI.DataInit.SC8[layer]:=SC8[layer];
    untGUI.DataInit.ACEC[layer]:=ACEC[layer];
    {also assign value in case new layers were added; the values below are constant
     values or dummies over all compartments anyway}
    untGUI.DataInit.MODRET[layer]:=5;   {fit a retention model}
    untGUI.DataInit.RP[layer]:=1;       {standard type}
    if layer<=NoCompartments then
    untProcesses.estimate_KatH(layer,clay[layer],silt[layer],rho[layer],oc[layer],DUM,
                          untGUI.DataInit.CMATCH[layer],
                          untGUI.DataInit.HMATCH[layer]);
    untGUI.DataInit.fc[layer]:=untGUI.DataInit.fc[1]; {Addiscot option; not used}
    untGUI.DataInit.d[layer]:=untGUI.DataInit.d[1]; {Addiscot option; not used}
    {dummy values below}
    untGUI.DataInit.AAEV[layer]:=untGUI.DataInit.AAEV[1];
    untGUI.DataInit.ABCAM[layer]:=untGUI.DataInit.ABCAM[1];
    untGUI.DataInit.AH[layer]:=untGUI.DataInit.AH[1];
  end;
  untGUI.DataInit.PDEPTH:=PDEPTH;
  untGUI.DataInit.DLZ:=DLZ;
  untGUI.DataInit.DIFO:=DIFO;
  untGUI.DataInit.DELM:=DELM;
  untGUI.DataInit.RHOS1:=RHOS1;
  untGUI.DataInit.RHOS2:=RHOS2;
  untGUI.DataInit.RHOS3:=RHOS3;
  for dy:=1 to 365 do
  begin
    untGUI.DataInit.FRAC1[dy]:=FRAC1[dy];
    untGUI.DataInit.IRRIGA[dy]:=IRRIGA[dy];
    untGUI.DataInit.IRATEA[dy]:=IRATEA[dy];
    for chem:=1 to MaxChem do
        untGUI.DataInit.ARSE[chem,dy]:=ARSE[chem,dy];
  end;
  for wk:=1 to 53 do
  begin
    untGUI.DataInit.PAN[wk]:=PAN[wk];
    untGUI.DataInit.WTDEP[wk]:=WTDEP[wk];
    untGUI.DataInit.AVETEM[wk]:=AVETEM[wk];
    untGUI.DataInit.AMP[wk]:=AMP[wk];
  end;
  untGUI.DataInit.NoLayers:=round(PDEPTH/DLZ);
  untGUI.DataYearIn:=untGUI.DataInit;
  untGUI.frmMAIN.WriteData(edtLeachcFileName.Text);
  untGUI.frmMAIN.mmDefaultLeachC.Lines.LoadFromFile(edtLeachcFileName.Text);
  untGUI.frmMAIN.mmDefaultLeachC.Enabled:=true;
  untGUI.frmMAIN.InitChem;
  untGUI.frmMAIN.ReadData(edtLeachcFileName.Text);
  untGUI.FirstLeachCFileName:=edtLeachcFileName.Text;
  DataYearIn.oldCALCITEA:=DataYearIn.CALCITEA;
  DataYearIn.oldGYPA:=DataYearIn.GYPA;

  {update the minerals.txt file}
  {open the weathering data file for writing}
  system.assign(fWE,edtMineralsFileName.text); system.rewrite(fWE);
  {first 3 header lines, then the mineral pools per layer}
  system.writeln(fWE,'[Initial mineral pools (fractions of total mineral mass, per layer)]');
  system.writeln(fWE,'[These initial fractions frX will be converted to mass via frX*RHO*(1-frcalcite-frgypsum-frOM)*T where T=thickness soil layer in mm]');
  system.writeln(fWE,'[Layer Albite K-Feldspar Muscovite Biotite Quartz Chlorite Anorthite Fosterite Augite Kaolinite Illite Montmorillonite Hornblende Fayalite Gibbsite Amorphite Otherite]');
  for i:=1 to NoCompartments do
  begin
   system.write(fWE,i,'  '); {layer number}
   for j:=1 to 14 do
   begin
     system.write(fWE,TMPMinPools[j][i],'  '); ;
   end;
   system.write(fWE,TMPGibbsitepool[i],'   ');
   for j:=15 to 16 do  {amorphite and otherite)}
   begin
     system.write(fWE,TMPMinPools[j][i],'   ');
   end;
   system.writeln(fWE);
  end;
  closefile(fWE);
  {minerals.txt file updated}
  untGUI.frmMAIN.mmWeatheringData.Lines.LoadFromFile(edtMineralsFileName.text);
  untGUI.frmMAIN.ReadDataWE(untGUI.FirstWeatherFileName);
  {update the actual amounts of minerals and include the updated versions of Amorphite+Otherite}
  {DataYearIn:=DataInit;} untGUI.frmMAIN.CopyRecToFrom(DataYearIn,DataInit);
  close;
end;

procedure TfrmEditSoilData.bttnAmorphiteClick(Sender: TObject);
var ftmp:textfile;
  txt:string; number:real; i:integer;
begin
  system.assign(ftmp,'Amorphite.txt'); system.reset(ftmp);
  system.readln(ftmp,txt); lblAmorphite.Caption:=txt;
  strgrdAmorphite.Cells[0,0]:='Now in file'; strgrdAmorphite.Cells[2,0]:='New value';
  for i:=1 to 12 do
  begin
    system.readln(ftmp,number,txt); txt:=trim(txt);
    strgrdAmorphite.Cells[1,i]:=txt;
    str(number:6:3,txt); strgrdAmorphite.Cells[0,i]:=txt; strgrdAmorphite.Cells[2,i]:=txt;
  end;
  closefile(ftmp);
  grpbxAmorphite.Color:=clSilver;
  grpbxAmorphite.visible:=true;
end;

procedure TfrmEditSoilData.bttnNoSaveAmorphite1Click(Sender: TObject);
begin
  grpbxAmorphite.visible:=false;
end;

procedure TfrmEditSoilData.bttnNoSaveOtheriteClick(Sender: TObject);
begin
  grpbxOtherite.visible:=false;
end;

procedure TfrmEditSoilData.bttnOtheriteClick(Sender: TObject);
var ftmp:textfile;
  txt:string; number:real; i:integer;
begin
  system.assign(ftmp,'Otherite.txt'); system.reset(ftmp);
  system.readln(ftmp,txt); lblOtherite.Caption:=txt;
  strgrdOtherite.Cells[0,0]:='Now in file'; strgrdOtherite.Cells[2,0]:='New value';
  for i:=1 to 12 do
  begin
    system.readln(ftmp,number,txt); txt:=trim(txt);
    strgrdOtherite.Cells[1,i]:=txt;
    str(number:6:3,txt); strgrdOtherite.Cells[0,i]:=txt;  strgrdOtherite.Cells[2,i]:=txt;
  end;
  closefile(ftmp);
  grpbxOtherite.Color:=clSilver;
  grpbxOtherite.visible:=true;
end;

procedure TfrmEditSoilData.bttnSaveAmorphiteClick(Sender: TObject);
var ftmp:textfile; i,code:integer; dummy:real; allowed:boolean;
begin
  allowed:=true;
  for i:=1 to 12 do
  begin
    system.val(strgrdAmorphite.Cells[2,i],dummy,code);
    if code<>0 then allowed:=false;
  end;
  if allowed then
  begin
    system.assign(ftmp,'Amorphite.txt'); system.rewrite(ftmp);
    system.writeln(ftmp,lblAmorphite.Caption);
    for i:=1 to 12 do
      system.writeln(ftmp,strgrdAmorphite.Cells[2,i],' ',strgrdAmorphite.Cells[1,i]);
    closefile(ftmp);
  end
  else showmessage('Incomplete or wrong new values. File not updated.');
  grpbxAmorphite.visible:=false;
end;

procedure TfrmEditSoilData.bttnSaveOtheriteClick(Sender: TObject);
var ftmp:textfile; i,code:integer; dummy:real; allowed:boolean;
begin
  allowed:=true;
  for i:=1 to 12 do
  begin
    system.val(strgrdOtherite.Cells[2,i],dummy,code);
    if code<>0 then allowed:=false;
  end;
  if allowed then
  begin
    system.assign(ftmp,'Otherite.txt'); system.rewrite(ftmp);
    system.writeln(ftmp,lblOtherite.Caption);
    for i:=1 to 12 do
      system.writeln(ftmp,strgrdOtherite.Cells[2,i],' ',strgrdOtherite.Cells[1,i]);
    closefile(ftmp);
  end
  else showmessage('Incomplete or wrong new values. File not updated.');
  grpbxOtherite.visible:=false;
end;

procedure TfrmEditSoilData.edtAlxChange(Sender: TObject);
begin
  CheckData
end;

procedure TfrmEditSoilData.edtCaxChange(Sender: TObject);
begin
  CheckData
end;

procedure TfrmEditSoilData.edtCLAYChange(Sender: TObject);
var tclay,tSilt,tSand,tOC,tRHO:real; txt:string;
begin
  CheckData ;
  tClay:=CheckVal(edtCLAY.Text); tSilt:=CheckVal(edtSILT.Text);
  tSand:=CheckVal(edtSAND.Text); tOC:=CheckVal(edtOC.Text);
  tRHO:=untGUI.frmMAIN.HYPRESRHO(100*tClay/(tClay+tSilt+tSand),
                               100*1.7*tOC/(tClay+tSilt+tSand+1.7*tOC));
  str(tRHO:5:3,txt);
  edtRHO.Text:=txt;
end;

procedure TfrmEditSoilData.edtGravelChange(Sender: TObject);
begin
  CheckData
end;

procedure TfrmEditSoilData.edtHxChange(Sender: TObject);
begin
  CheckData
end;

procedure TfrmEditSoilData.edtKxChange(Sender: TObject);
begin
  CheckData
end;

procedure TfrmEditSoilData.edtMgxChange(Sender: TObject);
begin
  CheckData
end;

procedure TfrmEditSoilData.edtNaxChange(Sender: TObject);
begin
  CheckData
end;

procedure TfrmEditSoilData.edtOCChange(Sender: TObject);
var tclay,tSilt,tSand,tOC,tRHO:real; txt:string;
begin
  CheckData ;
  tClay:=CheckVal(edtCLAY.Text); tSilt:=CheckVal(edtSILT.Text);
  tSand:=CheckVal(edtSAND.Text); tOC:=CheckVal(edtOC.Text);
  tRHO:=untGUI.frmMAIN.HYPRESRHO(100*tClay/(tClay+tSilt+tSand),
                               100*1.7*tOC/(tClay+tSilt+tSand+1.7*tOC));
  str(tRHO:5:3,txt);
  edtRHO.Text:=txt;
end;

procedure TfrmEditSoilData.edtSILTChange(Sender: TObject);
begin
  CheckData
end;

procedure TfrmEditSoilData.edtStonesChange(Sender: TObject);
begin
  checkdata
end;

procedure TfrmEditSoilData.spndtDISPChange(Sender: TObject);
begin
  if (spndtDISP.value<(0.5*spndtDLZ.value)) or (spndtDISP.value>(2*spndtDLZ.value))
  then spndtDISP.Color:=clyellow else spndtDISP.Color:=clWindow;
end;

procedure TfrmEditSoilData.spndtDLZChange(Sender: TObject);
begin
  if (spndtDISP.value<(0.5*spndtDLZ.value)) or (spndtDISP.value>(2*spndtDLZ.value))
  then spndtDISP.Color:=clyellow else spndtDISP.Color:=clWindow;
  {profile depth must be integer multitude of segment thickness; stepwise increment = thickness}
  spndtPDEPTH.MaxValue:=spndtDLZ.Value*round(spndtPDEPTH.MaxValue/spndtDLZ.Value+0.5);
  spndtPDEPTH.Increment:=round(spndtDLZ.Value);
  spndtPDEPTH.Value:=spndtDLZ.Value*round(spndtPDEPTH.Value/spndtDLZ.Value);
  nocompartments:=round(spndtPDEPTH.Value/spndtDLZ.Value);
  Actief;
end;

procedure TfrmEditSoilData.ShowLayer(layer:integer);
var txt,txt2:string;
begin
  spndtPDEPTH.Value:=round(PDEPTH);   edtDELM.Text:=val2str(DELM);
  spndtDLZ.value:=round(DLZ);         edtRHOS1.Text:=val2str(Rhos1);
  spndtDISP.Value:=round(DISP);       edtRHOS2.Text:=val2str(Rhos2);
  spndtMDIFF.Value:=round(DIFO);      edtRHOS3.Text:=val2str(Rhos3);
  edtCLAY.Text:=val2str(CLAY[layer]); edtSILT.Text:=val2str(SILT[layer]);
  SAND[layer]:=100-CLAY[layer]-SILT[layer]-1.7*OC[layer];
  edtSAND.Text:=val2str(SAND[layer]); edtOC.Text:=val2str(OC[layer]);
  edtGRAVEL.Text:=val2str(GRAVEL[layer]); edtSTONES.Text:=val2str(STONES[layer]);
  {RHO[layer]:=untLEACHcShell.frmMAIN.HYPRESRHO(100*clay[layer]/(clay[layer]+silt[layer]+sand[layer]),
                                 100*1.7*OC[layer]/(clay[layer]+silt[layer]+sand[layer]+1.7*OC[layer]));  }
  edtRHO.Text:=val2str(RHO[layer]);   edtAW.Text:=val2str(AW[layer]);
  edtAT.Text:=val2str(ATEM[layer]);   edtACALCITE.Text:=val2str(CALCITEA[layer]);
  edtAGYPS.Text:=val2str(GYPA[layer]);edtAPCO2.Text:=val2str(APCO2[layer]);
  edtCas.Text:=val2str(SEA[1,layer]); edtMgs.Text:=val2str(SEA[2,layer]);
  edtNAs.Text:=val2str(SEA[3,layer]); edtKs.Text:=val2str(SEA[4,layer]);
  edtCls.Text:=val2str(SEA[5,layer]); edtSO4s.Text:=val2str(SEA[6,layer]);
  edtAlks.Text:=val2str(SEA[7,layer]);edtAls.Text:=val2str(SEA[12,layer]);
  edtOrgs.Text:=val2str(SEA[13,layer]); edtHOrgs.Text:=val2str(SEA[14,layer]);
  edtH2Orgs.Text:=val2str(SEA[15,layer]); edtDOCs.Text:=val2str(SEA[16,layer]);
  edtCax.Text:=val2str(AXSE[1,layer]);edtMgx.Text:=val2str(AXSE[2,layer]);
  edtNax.Text:=val2str(AXSE[3,layer]);edtKx.Text:=val2str(AXSE[4,layer]);
  edtHx.Text:=val2str(AXSE[10,layer]);edtAlx.Text:=val2str(AXSE[12,layer]);
  edtCEC.Text:=val2str(ACEC[layer]);  edtSC1.Text:=val2str(SC1[layer]);
  edtSC2.Text:=val2str(SC2[layer]);   edtSC3.Text:=val2str(SC3[layer]);
  edtSC7.Text:=val2str(SC7[layer]);   edtSC8.Text:=val2str(SC8[layer]);
  str(compartment,txt);               lblCompartment.Caption:=txt;
  str(((compartment-1)*spndtDLZ.value/10):3:0,txt);
  str((compartment*spndtDLZ.value/10):3:0,txt2);
  lblDepths.Caption:=txt+'-'+txt2+' cm';
  edtfrAlbite.Text:=val2str(TMPMinPools[1][layer]);  
  edtfrKfeldspar.Text:=val2str(TMPMinPools[2][layer]);
  edtfrMuscovite.Text:=val2str(TMPMinPools[3][layer]);  
  edtfrBiotite.Text:=val2str(TMPMinPools[4][layer]);
  edtfrQuartz.Text:=val2str(TMPMinPools[5][layer]);
  edtfrChlorite.Text:=val2str(TMPMinPools[6][layer]);
  edtfrAnorthite.Text:=val2str(TMPMinPools[7][layer]);
  edtfrFosterite.Text:=val2str(TMPMinPools[8][layer]);
  edtfrAugite.Text:=val2str(TMPMinPools[9][layer]); 
  edtfrKaolinite.Text:=val2str(TMPMinPools[10][layer]);
  edtfrIllite.Text:=val2str(TMPMinPools[11][layer]);
  edtfrMontmorillonite.Text:=val2str(TMPMinPools[12][layer]);
  edtfrHornblende.Text:=val2str(TMPMinPools[13][layer]);
  edtfrFayalite.Text:=val2str(TMPMinPools[14][layer]);
  edtfrAmorphite.Text:=val2str(TMPMinPools[15][layer]);
  edtfrOtherite.Text:=val2str(TMPMinPools[16][layer]);
  edtfrGibbsite.Text:=val2str(TMPGibbsitepool[layer]);
end;

procedure TfrmEditSoilData.FormActivate(Sender: TObject);
var chem,layer,dy,wk:integer;
begin
  {read the initial soil data}
   untGUI.frmMAIN.ReadData(untGUI.FirstLeachCFileName);
   {read the initial weathering (mineralogical) data}
   untGUI.frmMAIN.ReadDataWE(untGUI.FirstWeatherFileName);
  {enable or disable editboxes conditional to Typechem}
  if untGUI.TypeChem>=2 then
  begin
     edtAls.enabled:=true;   edtOrgs.enabled:=true;
     edtHOrgs.enabled:=true; edtH2Orgs.enabled:=true;
     edtDOCs.enabled:=true;  edtHX.enabled:=true;
     edtAlX.enabled:=true;   edtSC7.enabled:=true;
     edtSC8.enabled:=true;   edtARSE12.enabled:=true;
  end;
  {copy data from calling form (read from standard inputfile);
   on-the-go assign data from layer 1 to the screeninput boxes}
  for layer:=1 to MaxLayers do
  begin;
    for chem:=1 to MaxChem do
    begin
      SEA[chem][layer]:=untGUI.DataInit.SEA[chem][layer];
      AXSE[chem][layer]:=untGUI.DataInit.AXSE[chem][layer];
    end;
    CLAY[layer]:=untGUI.DataInit.CLAY[layer];
    SILT[layer]:=untGUI.DataInit.SILT[layer] ;
    OC[layer]:=untGUI.DataInit.OC[layer];
    SAND[layer]:=100-(CLAY[layer]+SILT[layer]+1.7*OC[layer]);    {markOC}
    GRAVEL[layer]:=untGUI.DataInit.GRAVEL[layer];
    STONES[layer]:=untGUI.DataInit.STONES[layer];
    {RHO[layer]:=untLEACHcShell.frmMAIN.HYPRESRHO(100*clay[layer]/(clay[layer]+silt[layer]+sand[layer]),
                                 100*1.7*OC[layer]/(clay[layer]+silt[layer]+sand[layer]+1.7*OC[layer])); }
    RHO[layer]:=max(0.01,untGUI.DataInit.RHO[layer]);
    AW[layer]:=untGUI.DataInit.AW[layer];
    ATEM[layer]:=untGUI.DataInit.ATEM[layer];
    lambda[layer]:=untGUI.DataInit.lambda[layer];
    {PF23 convert back to mass fraction, correcting for stones+gravel (no calcite and gypsum in these fractions): mfr=mass/(RHO*DLZ*(1-stonesfr-gravelfr))}
    CALCITEA[layer]:=untGUI.DataInit.CALCITEA[layer]/
                     (RHO[layer]*untGUI.DataInit.DLZ*(1-(STONES[layer]+GRAVEL[layer])/100));   {all calcite a fraction of the fine earth}
    GYPA[layer]:=untGUI.DataInit.GYPA[layer]/
                 (RHO[layer]*untGUI.DataInit.DLZ*(1-(STONES[layer]+GRAVEL[layer])/100));       {all gypsum a fraction of the fine earth}
    APCO2[layer]:=untGUI.DataInit.APCO2[layer];
    SC1[layer]:=untGUI.DataInit.SC1[layer];
    SC2[layer]:=untGUI.DataInit.SC2[layer];
    SC3[layer]:=untGUI.DataInit.SC3[layer];
    SC7[layer]:=untGUI.DataInit.SC7[layer];
    SC8[layer]:=untGUI.DataInit.SC8[layer];
    ACEC[layer]:=untGUI.DataInit.ACEC[layer];
  end;
  PDEPTH:=untGUI.DataInit.PDEPTH;
  DLZ:=untGUI.DataInit.DLZ;
  DISP:=lambda[1];   {assuming dispersivity constant!}
  DIFO:=untGUI.DataInit.DIFO;
  DELM:=untGUI.DataInit.DELM;
  RHOS1:=untGUI.DataInit.RHOS1;
  RHOS2:=untGUI.DataInit.RHOS2;
  RHOS3:=untGUI.DataInit.RHOS3;
  compartment:=1; nocompartments:=round(spndtPDEPTH.Value/spndtDLZ.Value);
  spndtPDEPTH.Increment:=round(spndtDLZ.Value);
  {also read the mineralogical data}
  ReadMinFile;
  ShowLayer(compartment); actief;
  edtLeachcFileName.Text:=untGUI.FirstLeachCFileName;
  edtMineralsFileName.Text:=untGUI.FirstWeatherFileName;
  for dy:=1 to 365 do
  begin
    FRAC1[dy]:=untGUI.DataInit.FRAC1[dy];
    IRRIGA[dy]:=untGUI.DataInit.IRRIGA[dy];
    IRATEA[dy]:=untGUI.DataInit.IRATEA[dy];
    for chem:=1 to MaxChem do
        ARSE[chem,dy]:=untGUI.DataInit.ARSE[chem,dy];
  end;
  for wk:=1 to 53 do
  begin
    PAN[wk]:=untGUI.DataInit.PAN[wk];
    WTDEP[wk]:=untGUI.DataInit.WTDEP[wk];
    AVETEM[wk]:=untGUI.DataInit.AVETEM[wk];
    AMP[wk]:=untGUI.DataInit.AMP[wk];
  end;
  day:=1; week:=1;
  showtime(day,week);
end;

function TfrmEditSoilData.Val2Str(value:real):string;
var txt:string;
begin
  str(value:8:5,txt); val2str:=txt;
end;

procedure TfrmEditSoilData.btbttnBOTTOMClick(Sender: TObject);
begin
  storelayer(compartment);
  nocompartments:=round(spndtPDEPTH.Value/spndtDLZ.Value);
  compartment:=nocompartments;
  ShowLayer(compartment); Actief;
end;

procedure TfrmEditSoilData.btbttnUPClick(Sender: TObject);
begin
  storelayer(compartment);
  nocompartments:=round(spndtPDEPTH.Value/spndtDLZ.Value);
  compartment:=compartment-1;
  if compartment<1 then compartment:=1;
  ShowLayer(compartment); Actief;
end;

procedure TfrmEditSoilData.btbttnDOWNClick(Sender: TObject);
begin
  storelayer(compartment);
  nocompartments:=round(spndtPDEPTH.Value/spndtDLZ.Value);
  compartment:=compartment+1;
  if compartment>nocompartments
  then compartment:=nocompartments;
  ShowLayer(compartment); Actief;
end;

procedure TfrmEditSoilData.Actief;
var txt:string;
begin
   nocompartments:=round(spndtPDEPTH.Value/spndtDLZ.Value);
   btbttnUP.enabled:=true; btbttnDOWN.enabled:=true; btbttnCopy.enabled:=true;
   if compartment=1 then
   begin btbttnCopy.enabled:=false; btbttnUP.enabled:=false end;
   if compartment=nocompartments then btbttnDOWN.enabled:=false;
   str(nocompartments,txt); lblNoCompartments.Caption:=txt;
end;

procedure TfrmEditSoilData.btbttnTOPClick(Sender: TObject);
begin
   storelayer(compartment);
   compartment:=1; ShowLayer(compartment); Actief;
end;

procedure TfrmEditSoilData.spndtPDEPTHExit(Sender: TObject);
begin
  nocompartments:=round(spndtPDEPTH.Value/spndtDLZ.Value);
  PDEPTH:=spndtPDEPTH.Value;
  Actief;
end;

procedure TfrmEditSoilData.spndtDLZExit(Sender: TObject);
begin
  nocompartments:=round(spndtPDEPTH.Value/spndtDLZ.Value);
  DLZ:=spndtDLZ.Value;
  Actief;
end;

procedure TfrmEditSoilData.spndtPDEPTHChange(Sender: TObject);
begin
  nocompartments:=round(spndtPDEPTH.Value/spndtDLZ.Value); Actief;
end;

procedure TfrmEditSoilData.spndtDISPExit(Sender: TObject);
begin
  DISP:=spndtDISP.Value;
end;

procedure TfrmEditSoilData.spndtMDIFFExit(Sender: TObject);
begin
  DIFO:=spndtMDIFF.Value;
end;

procedure TfrmEditSoilData.edtDELMExit(Sender: TObject);
begin
  DELM:=CheckVal(edtDELM.Text);
end;

function TfrmEditSoilData.CheckVal(txt:string):real;
var code:integer; value:real;
begin
  system.Val(txt,value,code);
  if code<>0 then
  begin showmessage('Enter a value'); CheckVal:=-999; end
  else CheckVal:=value;
end;

procedure TfrmEditSoilData.edtRHOS2Exit(Sender: TObject);
begin
  Rhos2:=CheckVal(edtRHOS2.Text);
end;

procedure TfrmEditSoilData.edtRHOS1Exit(Sender: TObject);
begin
  Rhos1:=CheckVal(edtRHOS1.Text);
end;

procedure TfrmEditSoilData.edtRHOS3Exit(Sender: TObject);
begin
  Rhos3:=CheckVal(edtRHOS3.Text);
end;

procedure TfrmEditSoilData.storelayer(layer:integer);
var sum:extended; imin:integer;
begin
  CLAY[layer]:=CheckVal(edtCLAY.Text);   SILT[layer]:=CheckVal(edtSILT.Text); ;
  OC[layer]:=CheckVal(edtOC.Text);       SAND[layer]:=CheckVal(edtSAND.Text);
  GRAVEL[layer]:=CheckVal(edtGRAVEL.Text);STONES[layer]:=CheckVal(edtSTONES.Text);
  RHO[layer]:=CheckVal(edtRHO.Text);     AW[layer]:=CheckVal(edtAW.Text);
  ATEM[layer]:=CheckVal(edtAT.Text);     CALCITEA[layer]:=CheckVal(edtACALCITE.Text);
  GYPA[layer]:=CheckVal(edtAGYPS.Text);  APCO2[layer]:=CheckVal(edtAPCO2.Text);
  SEA[1,layer]:=CheckVal(edtCas.Text);   SEA[2,layer]:=CheckVal(edtMgs.Text);
  SEA[3,layer]:=CheckVal(edtNAs.Text);   SEA[4,layer]:=CheckVal(edtKs.Text);
  SEA[5,layer]:=CheckVal(edtCls.Text);   SEA[6,layer]:=CheckVal(edtSO4s.Text);
  SEA[7,layer]:=CheckVal(edtAlks.Text);  SEA[12,layer]:=CheckVal(edtAls.Text);
  SEA[13,layer]:=CheckVal(edtOrgs.Text); SEA[14,layer]:=CheckVal(edtHOrgs.Text);
  SEA[15,layer]:=CheckVal(edtH2Orgs.Text);SEA[16,layer]:=CheckVal(edtDOCs.Text);
  AXSE[1,layer]:=CheckVal(edtCax.Text);  AXSE[2,layer]:=CheckVal(edtMgx.Text);
  AXSE[3,layer]:=CheckVal(edtNax.Text);  AXSE[4,layer]:=CheckVal(edtKx.Text);
  AXSE[10,layer]:=CheckVal(edtHx.Text);  AXSE[12,layer]:=CheckVal(edtAlx.Text);
  ACEC[layer]:=AXSE[1,layer]+AXSE[2,layer]+AXSE[3,layer]+AXSE[4,layer]+
               AXSE[10,layer]+AXSE[12,layer];
  edtCEC.Text:=val2str(ACEC[layer]);
  SC1[layer]:=CheckVal(edtSC1.Text);
  SC2[layer]:=CheckVal(edtSC2.Text);     SC3[layer]:=CheckVal(edtSC3.Text);
  SC7[layer]:=CheckVal(edtSC7.Text);     SC8[layer]:=CheckVal(edtSC8.Text);

  {check on Gapon coefficients too close to 0 and correct to a low value}
  if (SC1[layer]<0.00000001) then SC1[layer]:=0.00000001;
  if (SC2[layer]<0.00000001) then SC2[layer]:=0.00000001;
  if (SC3[layer]<0.00000001) then SC3[layer]:=0.00000001;
  if (SC7[layer]<0.00000001) then SC7[layer]:=0.00000001;
  if (SC8[layer]<0.00000001) then SC8[layer]:=0.00000001;
  {minerals:}
  sum:=0;
  TMPMinPools[1][layer]:=CheckVal(edtfrAlbite.Text); 
  TMPMinPools[2][layer]:=CheckVal(edtfrKfeldspar.Text);
  TMPMinPools[3][layer]:=CheckVal(edtfrMuscovite.Text);
  TMPMinPools[4][layer]:=CheckVal(edtfrBiotite.Text);
  TMPMinPools[5][layer]:=CheckVal(edtfrQuartz.Text);
  TMPMinPools[6][layer]:=CheckVal(edtfrChlorite.Text);
  TMPMinPools[7][layer]:=CheckVal(edtfrAnorthite.Text);
  TMPMinPools[8][layer]:=CheckVal(edtfrFosterite.Text);
  TMPMinPools[9][layer]:=CheckVal(edtfrAugite.Text);
  TMPMinPools[10][layer]:=CheckVal(edtfrKaolinite.Text);
  TMPMinPools[11][layer]:=CheckVal(edtfrIllite.Text);
  TMPMinPools[12][layer]:=CheckVal(edtfrMontmorillonite.Text);
  TMPMinPools[13][layer]:=CheckVal(edtfrHornblende.Text);
  TMPMinPools[14][layer]:=CheckVal(edtfrFayalite.Text);
  TMPMinPools[15][layer]:=CheckVal(edtfrAmorphite.Text);
  TMPMinPools[16][layer]:=CheckVal(edtfrOtherite.Text);
  TMPGibbsitepool[layer]:=CheckVal(edtfrGibbsite.Text);
  {rescale so that all mass fractions of minerals 1..16 + Gibbsite sum up to 1}
  if sum<>0 then
  begin
    for imin:=1 to nrminerals do
    begin
     {potentially newforming minerals should not start with value=0}
     if (imin in [5,6,10,11,12]) and (TMPMinPools[imin][layer]=0)
     then TMPMinPools[imin][layer]:=untGUI.seedmineral;
       sum:=sum+TMPMinPools[imin][layer];
    end;
    sum:=sum+TMPGibbsitepool[layer];
    for imin:=1 to nrminerals do TMPMinPools[imin][layer]:=TMPMinPools[imin][layer]/sum;
    TMPGibbsitepool[layer]:=TMPGibbsitepool[layer]/sum;
  end;
  {rescale ready}
end;

procedure TfrmEditSoilData.btbttnCopyClick(Sender: TObject);
var imin:integer;
begin
 if compartment>1 then
 begin
  CLAY[compartment]:=CLAY[compartment-1];     SILT[compartment]:=SILT[compartment-1];
  OC[compartment]:=OC[compartment-1];         SAND[compartment]:=SAND[compartment-1];
  GRAVEL[compartment]:=GRAVEL[compartment-1]; STONES[compartment]:=STONES[compartment-1];
  RHO[compartment]:=RHO[compartment-1];       AW[compartment]:=AW[compartment-1];
  ATEM[compartment]:=ATEM[compartment-1];     CALCITEA[compartment]:=CALCITEA[compartment-1];
  GYPA[compartment]:=GYPA[compartment-1];     APCO2[compartment]:=APCO2[compartment-1];
  SEA[1,compartment]:=SEA[1,compartment-1];   SEA[2,compartment]:=SEA[2,compartment-1];
  SEA[3,compartment]:=SEA[3,compartment-1];   SEA[4,compartment]:=SEA[4,compartment-1];
  SEA[5,compartment]:=SEA[5,compartment-1];   SEA[6,compartment]:=SEA[6,compartment-1];
  SEA[7,compartment]:=SEA[7,compartment-1];   SEA[12,compartment]:=SEA[12,compartment-1];
  SEA[13,compartment]:=SEA[13,compartment-1]; SEA[14,compartment]:=SEA[14,compartment-1];
  SEA[15,compartment]:=SEA[15,compartment-1]; SEA[16,compartment]:=SEA[16,compartment-1];
  AXSE[1,compartment]:=AXSE[1,compartment-1];  AXSE[2,compartment]:=AXSE[2,compartment-1];
  AXSE[3,compartment]:=AXSE[3,compartment-1];  AXSE[4,compartment]:=AXSE[4,compartment-1];
  AXSE[10,compartment]:=AXSE[10,compartment-1];AXSE[12,compartment]:=AXSE[12,compartment-1];
  ACEC[compartment]:=ACEC[compartment-1];     SC1[compartment]:=SC1[compartment-1];
  SC2[compartment]:=SC2[compartment-1];       SC3[compartment]:=SC3[compartment-1];
  SC7[compartment]:=SC7[compartment-1];       SC8[compartment]:=SC8[compartment-1];
  for imin:=1 to nrminerals do TMPMinPools[imin][compartment]:=TMPMinPools[imin][compartment-1];
  TMPGibbsitepool[compartment]:=TMPGibbsitepool[compartment-1];
  showlayer(compartment);
 end
 else showmessage('No source compartment');
 if chckbxProceed.checked then btbttnDOWNClick(Sender);
end;

procedure TfrmEditSoilData.chckbxWEEKClick(Sender: TObject);
begin
   if chckbxWEEK.Checked
   then lblWEEKDAY.Caption:='browse by week'
   else lblWEEKDAY.Caption:='browse by day';
end;

procedure TfrmEditSoilData.btbttnFirstClick(Sender: TObject);
begin
   storetime(day,week);
   if chckbxWEEK.Checked
   then begin day:=1; week:=1; end
   else begin day:=1; week:=1; end;
   ShowTime(day,week); ActiefTime;
end;

procedure TfrmEditSoilData.btbttnPREVClick(Sender: TObject);
begin
   storetime(day,week);
   if chckbxWEEK.Checked
   then begin week:=week-1; day:=(week-1)*7+1 end
   else begin day:=day-1; week:=((day-1) div 7)+1; end;
   ShowTime(day,week); ActiefTime;
end;

procedure TfrmEditSoilData.btbttnNEXTClick(Sender: TObject);
begin
   storetime(day,week);
   if chckbxWEEK.Checked
   then begin week:=week+1; day:=(week-1)*7+1 end
   else begin day:=day+1; week:=((day-1) div 7)+1; end;
   ShowTime(day,week); ActiefTime;
end;

procedure TfrmEditSoilData.btbttnLASTClick(Sender: TObject);
begin
   storetime(day,week);
   if chckbxWEEK.Checked
   then begin day:=365; week:=53; end
   else begin day:=365; week:=53; end;
   ShowTime(day,week); ActiefTime;
end;

procedure TfrmEditSoilData.StoreTime(day,week:integer);
begin
  FRAC1[day]:=CheckVal(edtFRAC1.Text);
  IRRIGA[day]:=CheckVal(edtIRRIGA.Text);
  IRATEA[day]:=CheckVal(edtIRATEA.Text);
  ARSE[1,day]:=CheckVal(edtARSE1.Text);
  ARSE[2,day]:=CheckVal(edtARSE2.Text);
  ARSE[3,day]:=CheckVal(edtARSE3.Text);
  ARSE[4,day]:=CheckVal(edtARSE4.Text);
  ARSE[5,day]:=CheckVal(edtARSE5.Text);
  ARSE[6,day]:=CheckVal(edtARSE6.Text);
  ARSE[7,day]:=CheckVal(edtARSE7.Text);
  ARSE[12,day]:=CheckVal(edtARSE12.Text);
  PAN[week]:=CheckVal(edtPAN.Text);
  WTDEP[week]:=CheckVal(edtWTDEP.Text);
  AVETEM[week]:=CheckVal(edtAVETEM.Text);
  AMP[week]:=CheckVal(edtAMP.Text);
end;

procedure TfrmEditSoilData.ShowTime(day,week:integer);
var txt:string;
begin
  str(day,txt);  edtDAY.Text:=txt;
  str(week,txt);   edtWEEK.Text:=txt;
  edtFRAC1.Text:=  val2str(FRAC1[day]) ;
  edtIRRIGA.Text:= val2str(IRRIGA[day]) ;
  edtIRATEA.Text:= val2str(IRATEA[day]) ;
  edtARSE1.Text:=  val2str(ARSE[1,day]) ;
  edtARSE2.Text:=  val2str(ARSE[2,day]) ;
  edtARSE3.Text:=  val2str(ARSE[3,day]) ;
  edtARSE4.Text:=  val2str(ARSE[4,day]) ;
  edtARSE5.Text:=  val2str(ARSE[5,day]) ;
  edtARSE6.Text:=  val2str(ARSE[6,day]) ;
  edtARSE7.Text:=  val2str(ARSE[7,day]) ;
  edtARSE12.Text:= val2str(ARSE[12,day]) ;
  edtPAN.Text:=    val2str(PAN[week]);
  edtWTDEP.Text:=  val2str(WTDEP[week]);
  edtAVETEM.Text:= val2str(AVETEM[week]);
  edtAMP.Text:=    val2str(AMP[week]);
end;

procedure TfrmEditSoilData.ActiefTime;
begin
   nocompartments:=round(spndtPDEPTH.Value/spndtDLZ.Value);
   btbttnPREV.enabled:=true; btbttnNEXT.enabled:=true;
   if chckbxWEEK.Checked
   then
   begin
     if week=1 then btbttnPREV.enabled:=false;
     if week=53 then btbttnNEXT.enabled:=false;
   end
   else
   begin
     if day=1 then btbttnPREV.enabled:=false;
     if day=365 then btbttnNEXT.enabled:=false;
   end;
end;

procedure TfrmEditSoilData.CheckData;
var tCLAY, tSILT, tOC, tSAND, tACEC, tgravel, tstones:real;
  tAXSE:array[1..12]of real;
begin
  tCLAY:=CheckVal(edtCLAY.Text);   tSILT:=CheckVal(edtSILT.Text); ;
  tOC:=CheckVal(edtOC.Text);       tSAND:=CheckVal(edtSAND.Text);
  tgravel:=CheckVal(edtGravel.Text); tstones:=CheckVal(edtStones.Text);
  tAXSE[1]:=CheckVal(edtCax.Text);  tAXSE[2]:=CheckVal(edtMgx.Text);
  tAXSE[3]:=CheckVal(edtNax.Text);  tAXSE[4]:=CheckVal(edtKx.Text);
  tAXSE[10]:=CheckVal(edtHx.Text);  tAXSE[12]:=CheckVal(edtAlx.Text);
  tACEC:=tAXSE[1]+tAXSE[2]+tAXSE[3]+tAXSE[4]+
               tAXSE[10]+tAXSE[12];
  edtCEC.Text:=val2str(tACEC);
end;

procedure TfrmEditSoilData.ReadMinFile;
var fWE:textfile;
    txt:string;
    q,maxq,i,j,idum:integer;
    sum:real;
begin
  {initialize}
  for i:=1 to MaxLayers do
  begin
     for j:=1 to untGUI.nrminerals do TMPMinPools[j][i]:=0;
     TMPMinPools[5][i]:=1; {assuming all Quartz, initially}
     TMPGibbsitepool[i]:=0;
  end;
  {now read the weathering data file}
  system.assign(fWE,untGUI.FirstWeatherFileName);
  q:=0; maxq:=frmMAIN.mmWeatheringData.Lines.count;
  {initial mineral pools per layer}
  system.reset(fWE);
  repeat q:=q+1; system.readln(fWE,txt)
  until (pos('[Layer Albite K-Feldspar Muscovite Biotite Quartz Chlorite Anorthite',txt)>0)or(q=maxq);
  if q=maxq then raise EInOutError.Create('File incomplete');
  for i:=1 to DataInit.NoLayers do
  begin
   system.read(fWE,idum); {layer number}
   sum:=0;
   for j:=1 to 14 do
   begin
     system.read(fWE,TMPMinPools[j][i]);
     {potentially newforming minerals should not start with value=0}
     if (j in [5,6,10,11,12]) and (TMPMinPools[j][i]=0)
     then TMPMinPools[j][i]:=seedmineral;
     sum:=sum+TMPMinPools[j][i];
   end;
   system.read(fWE,TMPGibbsitepool[i]); sum:=sum+TMPGibbsitepool[i];
   for j:=15 to 16 do  {amorphite and otherite)}
   begin
     system.read(fWE,TMPMinPools[j][i]); sum:=sum+TMPMinPools[j][i];
   end;
   system.readln(fWE);
   if sum<>1 then
   for j:=1 to untGUI.nrminerals do {rescale to have fractions sum to 1}
   begin
     TMPMinPools[j][i]:=TMPMinPools[j][i]/sum;
   end;
   TMPGibbsitepool[i]:=TMPGibbsitepool[i]/sum;
   q:=q+1;
   if q>maxq then raise EInOutError.Create('File incomplete');
  end;
  closefile(fWE);
end;

end.
