unit untEvents;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$MODE Delphi}

{$DEBUGINFO ON}
interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin, Math, Buttons, Grids, Menus, FileUtil, lazfileutils,untSoilGenConst;

type
  {real=double;  }

  { TfrmEvents }

  TfrmEvents = class(TForm)
    edtSTONES: TEdit;
    edtfrAmorphite: TEdit;
    edtfrOtherite: TEdit;
    edtfrKfeldspar: TEdit;
    edtfrKaolinite: TEdit;
    edtfrIllite: TEdit;
    edtfrMontmorillonite: TEdit;
    edtfrHornblende: TEdit;
    edtfrFayalite: TEdit;
    edtfrGibbsite: TEdit;
    edtfrAlbite: TEdit;
    edtfrBiotite: TEdit;
    edtfrMuscovite: TEdit;
    edtfrQuartz: TEdit;
    edtfrChlorite: TEdit;
    edtfrAnorthite: TEdit;
    edtfrFosterite: TEdit;
    edtfrAugite: TEdit;
    edtGravel: TEdit;
    edtSlashBurnCO2: TEdit;
    edtSlashBurnIOM: TEdit;
    edtSlashBurnHUM: TEdit;
    grpbxSlashBurn: TGroupBox;
    Label56: TLabel;
    Label57: TLabel;
    Label58: TLabel;
    Label59: TLabel;
    Label60: TLabel;
    Label61: TLabel;
    Label62: TLabel;
    Label63: TLabel;
    Label64: TLabel;
    Label65: TLabel;
    Label66: TLabel;
    Memo3: TMemo;
    spndtEventYearBP: TSpinEdit;
    Label10: TLabel;
    Label11: TLabel;
    spndtEventSimYear: TSpinEdit;
    cmbbxEvent: TComboBox;
    Label12: TLabel;
    Label13: TLabel;
    spndtNoComps: TSpinEdit;
    lblMM: TLabel;
    btbttnConfirm: TBitBtn;
    grpbxDepData: TGroupBox;
    Label9: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    Label28: TLabel;
    Label29: TLabel;
    Label30: TLabel;
    Label31: TLabel;
    Label32: TLabel;
    Label33: TLabel;
    Label34: TLabel;
    Label19: TLabel;
    Label35: TLabel;
    Label36: TLabel;
    Label37: TLabel;
    Label38: TLabel;
    Label39: TLabel;
    Label40: TLabel;
    Label41: TLabel;
    Label42: TLabel;
    Label43: TLabel;
    Label44: TLabel;
    Label45: TLabel;
    Label46: TLabel;
    Label47: TLabel;
    Label48: TLabel;
    Label49: TLabel;
    Label50: TLabel;
    Label51: TLabel;
    edtCLAY: TEdit;
    edtSILT: TEdit;
    edtSAND: TEdit;
    edtOC: TEdit;
    edtRHO: TEdit;
    edtAW: TEdit;
    edtAT: TEdit;
    edtACALCITE: TEdit;
    edtAGYPS: TEdit;
    edtAPCO2: TEdit;
    edtCas: TEdit;
    edtMgs: TEdit;
    edtNAs: TEdit;
    edtKs: TEdit;
    edtCls: TEdit;
    edtSO4s: TEdit;
    edtAlks: TEdit;
    edtAls: TEdit;
    edtOrgs: TEdit;
    edtHOrgs: TEdit;
    edtH2Orgs: TEdit;
    edtDOCs: TEdit;
    edtCax: TEdit;
    edtMgx: TEdit;
    edtNax: TEdit;
    edtKx: TEdit;
    edtHx: TEdit;
    edtAlx: TEdit;
    edtCEC: TEdit;
    edtSC1: TEdit;
    edtSC2: TEdit;
    edtSC3: TEdit;
    edtSC7: TEdit;
    edtSC8: TEdit;
    strgrdEventData: TStringGrid;
    btbttnEventImport: TBitBtn;
    btbttnSaveClose: TBitBtn;
    btbttnClose: TBitBtn;
    pndlgOpen: TOpenDialog;
    Label5: TLabel;
    Label6: TLabel;
    PopupMenu1: TPopupMenu;
    Deleterowundercursor1: TMenuItem;
    grpbxVGN: TGroupBox;
    Memo1: TMemo;
    Label7: TLabel;
    Label8: TLabel;
    Label20: TLabel;
    Labelx: TLabel;
    Label52: TLabel;
    Label53: TLabel;
    edtVGNfilename: TEdit;
    grpbxPlowing: TGroupBox;
    Memo2: TMemo;
    Label54: TLabel;
    edtPlowMixFraction: TEdit;
    Label55: TLabel;
    bttnPlowMixFraction: TButton;
    procedure edtfrAlbiteChange(Sender: TObject);
    procedure edtfrAmorphiteChange(Sender: TObject);
    procedure edtfrAnorthiteChange(Sender: TObject);
    procedure edtfrAugiteChange(Sender: TObject);
    procedure edtfrBiotiteChange(Sender: TObject);
    procedure edtfrChloriteChange(Sender: TObject);
    procedure edtfrFayaliteChange(Sender: TObject);
    procedure edtfrFosteriteChange(Sender: TObject);
    procedure edtfrGibbsiteChange(Sender: TObject);
    procedure edtfrHornblendeChange(Sender: TObject);
    procedure edtfrIlliteChange(Sender: TObject);
    procedure edtfrKaoliniteChange(Sender: TObject);
    procedure edtfrKfeldsparChange(Sender: TObject);
    procedure edtfrMontmorilloniteChange(Sender: TObject);
    procedure edtfrMuscoviteChange(Sender: TObject);
    procedure edtfrOtheriteChange(Sender: TObject);
    procedure edtfrQuartzChange(Sender: TObject);
    procedure edtGravelChange(Sender: TObject);
    procedure edtSlashBurnCO2Change(Sender: TObject);
    procedure edtSlashBurnIOMChange(Sender: TObject);
    procedure edtSTONESChange(Sender: TObject);
    procedure spndtEventYearBPChange(Sender: TObject);
    procedure spndtEventSimYearChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure spndtNoCompsChange(Sender: TObject);
    procedure cmbbxEventChange(Sender: TObject);
    procedure edtCLAYChange(Sender: TObject);
    procedure edtSILTChange(Sender: TObject);
    procedure edtOCChange(Sender: TObject);
    procedure edtCaxChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure btbttnConfirmClick(Sender: TObject);
    procedure btbttnEventImportClick(Sender: TObject);
    procedure btbttnCloseClick(Sender: TObject);
    procedure btbttnSaveCloseClick(Sender: TObject);
    procedure Deleterowundercursor1Click(Sender: TObject);
    procedure bttnPlowMixFractionClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure CheckTextureOM(var fout:boolean);
    procedure calcCEC;
    procedure ShowLayer;
    function Val2Str(value:real):string;
    function Str2Val(txt:string):real;
    procedure sortstrgrd;
    procedure LeesData(fn:string);
    procedure LeesWoord(var f:textfile; var woord:string);
    procedure CheckMinerals(var scalefactor:real);
    function CheckVal(txt:string):real;
    procedure CheckData;
  end;

var
  frmEvents: TfrmEvents;
  fout,confirming:boolean;
  CEC,clay,silt,oc,sand,stones,gravel,rho,aw,atem,lambda,calcitea,gypa,apco2,
  sc1,sc2,sc3,sc7,sc8,scalefactor:real;
  frAlbite,frKfeldspar,frMuscovite,frBiotite,frQuartz,frChlorite,frAnorthite,
  frFosterite,frAugite,frKaolinite,frIllite,frMontmorillonite,frHornblende,
  frFayalite,frGibbsite,frAmorphite,frOtherite:real;
  SEA : array[1..MaxChem] of real;
  AXSE: array[1..MaxChem] of real;

implementation

{$R *.lfm}
uses untGUI;

procedure TfrmEvents.spndtEventYearBPChange(Sender: TObject);
begin
  spndtEventSimYear.value:=untGUI.frmMAIN.spndtMultipleYears.Value-spndtEventYearBP.Value+1;
   if cmbbxEvent.ItemIndex=3 then
   begin
     str(spndtNoComps.Value,txt);
     labelx.Caption:='compartment '+txt;
     str(spndtEventSimYear.Value,txt);
     edtVGNfilename.Text:='VGN'+txt+'.txt';
   end;
end;

procedure TfrmEvents.edtSlashBurnCO2Change(Sender: TObject);
var frCO2,frIOM,frHUM:real; code:integer;   txt:string;
begin
  frCO2:=Str2Val(edtSlashBurnCO2.Text);
  if frCO2=-999 then begin frCO2:=0.7; edtSlashBurnCO2.Text:='0.7'; end
  else
  begin
    frIOM:=Str2Val(edtSlashBurnIOM.Text);
    frHUM:=1-frIOM-frCO2;
    if (frHUM<0)or(frHUM>1)
    then
    begin
      showmessage('Combination of fractions outside [0;1]');
      frCO2:=0.7; frIOM:=0.03; frHUM:=0.27;
      edtSlashBurnCO2.Text:='0.7';
      edtSlashBurnIOM.Text:='0.03';
    end;
    str(frHUM:4:2,txt);
    edtSlashBurnHUM.text:=txt;
  end;
end;

procedure TfrmEvents.edtfrAlbiteChange(Sender: TObject);
begin
  CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrAmorphiteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrAnorthiteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrAugiteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrBiotiteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrChloriteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrFayaliteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrFosteriteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrGibbsiteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrHornblendeChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrIlliteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrKaoliniteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrKfeldsparChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrMontmorilloniteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrMuscoviteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrOtheriteChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtfrQuartzChange(Sender: TObject);
begin
    CheckMinerals(scalefactor);
end;

procedure TfrmEvents.edtGravelChange(Sender: TObject);
begin
  CheckData;
end;

procedure TfrmEvents.edtSlashBurnIOMChange(Sender: TObject);
var frCO2,frIOM,frHUM:real; code:integer;   txt:string;
begin
  frIOM:=Str2Val(edtSlashBurnIOM.Text);
  if frIOM=-999 then begin frIOM:=0.03; edtSlashBurnIOM.Text:='0.03'; end
  else
  begin
    frCO2:=Str2Val(edtSlashBurnCO2.Text);
    frHUM:=1-frIOM-frCO2;
    if (frHUM<0)or(frHUM>1)
    then
    begin
      showmessage('Combination of fractions outside [0;1]');
      frCO2:=0.7; frIOM:=0.03; frHUM:=0.27;
      edtSlashBurnCO2.Text:='0.7';
      edtSlashBurnIOM.Text:='0.03';
    end;
    str(frHUM:4:2,txt);
    edtSlashBurnHUM.text:=txt;
  end;
end;

procedure TfrmEvents.edtSTONESChange(Sender: TObject);
begin 
  CheckData;
end;

procedure TfrmEvents.spndtEventSimYearChange(Sender: TObject);
begin
  spndtEventYearBP.value:=untGUI.frmMAIN.spndtMultipleYears.Value-spndtEventSimYear.Value+1;
   if cmbbxEvent.ItemIndex=3 then
   begin
     str(spndtNoComps.Value,txt);
     labelx.Caption:='compartment '+txt;
     str(spndtEventSimYear.Value,txt);
     edtVGNfilename.Text:='VGN'+txt+'.txt';
   end;
end;

procedure TfrmEvents.FormShow(Sender: TObject);
begin
  spndtEventSimYear.value:=1; spndtEventSimYearChange(Sender);
  spndtNoComps.MinValue:=0;
  spndtNoComps.MaxValue:=max(5,untGUI.dataInit.NoLayers-1);
  spndtNoComps.Value:=0;   spndtNoCompsChange(Sender);
  cmbbxEvent.ItemIndex:=0; cmbbxEventChange(Sender);
end;

procedure TfrmEvents.spndtNoCompsChange(Sender: TObject);
var millimeters:real; txt:string;
begin
  millimeters:=spndtNoComps.Value*untGUI.dataYearIn.DLZ;
  str(millimeters:3:0,txt); lblMM.caption:=txt; lblMM.refresh;
  btbttnConfirm.Enabled:=(spndtNoComps.Value>0);
   if cmbbxEvent.ItemIndex=3 then
   begin
     str(spndtNoComps.Value,txt);
     labelx.Caption:='compartment '+txt;
     str(spndtEventSimYear.Value,txt);
     edtVGNfilename.Text:='VGN'+txt+'.txt';
   end;
end;

procedure TfrmEvents.cmbbxEventChange(Sender: TObject);
var txt:string;
begin
   case cmbbxEvent.ItemIndex of
     0: begin grpbxDepData.visible:=false; grpbxVGN.visible:=false; grpbxPlowing.Visible:=false;
              grpbxSlashBurn.visible:=false;
              spndtNoComps.MaxValue:=max(5,untGUI.dataInit.NoLayers-1);
        end;
     1: begin grpbxDepData.visible:=true;  grpbxVGN.visible:=false; grpbxPlowing.Visible:=false;
              grpbxSlashBurn.visible:=false;
              spndtNoComps.MaxValue:=untGUI.dataInit.NoLayers;
        end;
     2: begin grpbxDepData.visible:=false; grpbxVGN.visible:=false; grpbxPlowing.Visible:=false;
              grpbxSlashBurn.visible:=false;  end;
     3: begin grpbxDepData.visible:=false; grpbxVGN.visible:=true;  grpbxPlowing.Visible:=false;
              grpbxSlashBurn.visible:=false;  end;
     4: begin grpbxDepData.visible:=false; grpbxVGN.visible:=false; grpbxPlowing.Visible:=true;
              grpbxSlashBurn.visible:=false;  end;
     5: begin grpbxDepData.visible:=false; grpbxVGN.visible:=false; grpbxPlowing.Visible:=false;
              grpbxSlashBurn.visible:=true;   end;
   end;
   if cmbbxEvent.ItemIndex=3 then
   begin
     str(spndtNoComps.Value,txt);
     labelx.Caption:='compartment '+txt;
     str(spndtEventSimYear.Value,txt);
     edtVGNfilename.Text:='VGN'+txt+'.txt';
   end;
end;

procedure TfrmEvents.CheckTextureOM(var fout:boolean);
var clayt,siltt,oct:real; code:integer;  txt:string;
begin
   val(edtClay.Text,clayt,code);    val(edtSilt.Text,siltt,code);
   val(edtOC.Text,oct,code);
   fout:=(100-clayt-siltt-1.7*oct<0);
   if not(fout) then
   begin str((100-clayt-siltt-1.7*oct):9:5,txt); edtSand.Text:=txt; end;
end;

procedure TfrmEvents.calcCEC;
var code:integer; txt:string;
begin
   Val(edtCax.Text,AXSE[1],code);  Val(edtMgx.Text,AXSE[2],code);
   Val(edtNax.Text,AXSE[3],code);  Val(edtKx.Text,AXSE[4],code);
   Val(edtHx.Text,AXSE[10],code);  Val(edtAlx.Text,AXSE[12],code);
   CEC:=AXSE[1]+AXSE[2]+AXSE[3]+AXSE[4]+AXSE[10]+AXSE[12];
   str(CEC:9:5,txt); edtCEC.Text:=txt;
end;

procedure TfrmEvents.edtCLAYChange(Sender: TObject);
var tRHO, tClay, tSilt, tSand, tOC:real;  strRHO:string;
begin
  CheckData;
  CheckTextureOM(fout);
  if fout then edtClay.Text:='0'
  else
  begin
    tClay:=CheckVal(edtCLAY.Text); tSilt:=CheckVal(edtSILT.Text);
    tSand:=CheckVal(edtSAND.Text); tOC:=CheckVal(edtOC.Text);
    tRHO:=untGUI.frmMAIN.HYPRESRHO(100*tClay/(tClay+tSilt+tSand),
                                 100*1.7*tOC/(tClay+tSilt+tSand+1.7*tOC));
    str(tRHO:5:3,strRHO);
    edtRHO.Text:=strRHO;
  end;
end;

procedure TfrmEvents.edtSILTChange(Sender: TObject); 
var tRHO, tClay, tSilt, tSand, tOC:real;  strRHO:string;
begin    
  CheckData;
  CheckTextureOM(fout);
  if fout then edtSilt.Text:='0'
  else
  begin
    tClay:=CheckVal(edtCLAY.Text); tSilt:=CheckVal(edtSILT.Text);
    tSand:=CheckVal(edtSAND.Text); tOC:=CheckVal(edtOC.Text);
    tRHO:=untGUI.frmMAIN.HYPRESRHO(100*tClay/(tClay+tSilt+tSand),
                                 100*1.7*tOC/(tClay+tSilt+tSand+1.7*tOC));
    str(tRHO:5:3,strRHO);
    edtRHO.Text:=strRHO;
  end;
end;

procedure TfrmEvents.edtOCChange(Sender: TObject);  
var tRHO, tClay, tSilt, tSand, tOC:real;  strRHO:string;
begin
  CheckTextureOM(fout); 
  CheckData;
  if fout then edtOC.Text:='0'
  else
  begin
    tClay:=CheckVal(edtCLAY.Text); tSilt:=CheckVal(edtSILT.Text);
    tSand:=CheckVal(edtSAND.Text); tOC:=CheckVal(edtOC.Text);
    tRHO:=untGUI.frmMAIN.HYPRESRHO(100*tClay/(tClay+tSilt+tSand),
                                 100*1.7*tOC/(tClay+tSilt+tSand+1.7*tOC));
    str(tRHO:5:3,strRHO);
    edtRHO.Text:=strRHO;
  end;
end;

procedure TfrmEvents.edtCaxChange(Sender: TObject);
begin  
  CheckData;
  calcCEC;
end;

procedure TfrmEvents.FormCreate(Sender: TObject);
begin
  {enable or disable editboxes conditional to Typechem}
  if untGUI.TypeChem>=2 then
  begin
     edtAls.enabled:=true;   edtOrgs.enabled:=true;
     edtHOrgs.enabled:=true; edtH2Orgs.enabled:=true;
     edtDOCs.enabled:=true;  edtHX.enabled:=true;
     edtAlX.enabled:=true;   edtSC7.enabled:=true;
     edtSC8.enabled:=true;
  end;
end;

procedure TfrmEvents.FormActivate(Sender: TObject);
var chem:integer;  totalmass:real;
begin
    pndlgOPEN.InitialDir:=startdir;
    confirming:=true;
    spndtEventYearBP.MinValue:=1;
    spndtEventYearBP.MaxValue:=untGUI.frmMAIN.spndtMultipleYears.Value;
    spndtEventSimYear.MinValue:=1;
    spndtEventSimYear.MaxValue:=untGUI.frmMAIN.spndtMultipleYears.Value;
    spndtEventSimYear.Value:=untGUI.frmMAIN.spndtMultipleYears.Value;
    spndtEventYearBP.Value:=1;
    for chem:=1 to MaxChem do
    begin
      SEA[chem]:=untGUI.DataInit.SEA[chem][1];
      AXSE[chem]:=untGUI.DataInit.AXSE[chem][1];
    end;
    CLAY:=untGUI.DataInit.CLAY[1];
    SILT:=untGUI.DataInit.SILT[1] ;
    OC:=untGUI.DataInit.OC[1];
    SAND:=100-(CLAY+SILT+1.7*OC);
    GRAVEL:=untGUI.DataInit.GRAVEL[1];
    STONES:=untGUI.DataInit.STONES[1];
    RHO:=untGUI.DataInit.RHO[1];
    AW:=untGUI.DataInit.AW[1];
    ATEM:=untGUI.DataInit.ATEM[1];
    lambda:=untGUI.DataInit.lambda[1]; {pf23: 2 lines below: mass from massfraction: mass=RHO*DLZ*mfr*(1-stonefr-gravelfr)}
    CALCITEA:=untGUI.DataInit.CALCITEA[1]*RHO*untGUI.DataInit.DLZ*(1-(STONES+GRAVEL)/100); {PF23: all mass in fine earth fraction, thus corection for stones and gravel when using RHO}
    GYPA:=untGUI.DataInit.GYPA[1]*RHO*untGUI.DataInit.DLZ*(1-(STONES+GRAVEL)/100);         {PF23: all mass in fine earth fraction, thus corection for stones and gravel when using RHO}
    APCO2:=untGUI.pCO2atm;
    SC1:=untGUI.DataInit.SC1[1];
    SC2:=untGUI.DataInit.SC2[1];
    SC3:=untGUI.DataInit.SC3[1];
    SC7:=untGUI.DataInit.SC7[1];
    SC8:=untGUI.DataInit.SC8[1];
    CEC:=untGUI.DataInit.ACEC[1];
    {initialize mineralogy with values from top layer initial profile}
    totalmass:=DataInit.RHO[1]*DataInit.DLZ*(100-STONES-GRAVEL)/100;          {kg dry matter over DLZ mm compartment thickness and 1 m2 area, excluding stones+gravel}
    totalmass:=totalmass-calciteA-gypA-1.7*DataInit.OCm[1];    {but we subtract mass of gypsum, calcite and OC, all assumed part of fine earth}
    totalmass:=totalmass+((STONES+GRAVEL)/100)*DataInit.DLZ *2.65; {..raised by the mass in stones and gravel, with specific weight assumed to be 2.65 kg dm-3}
    {now totalmass is mass associated with the 16 minerals + gibbsite}
    {1 albite Na Al Si3 O8}
    {2 K-feldspar K Al Si3 O8}
    {3 muscovite K Al3 Si3 O10 (OH)2}
    {4 biotite K Mg1.5 Fe1.5 Al Si3 O10 (OH)2}
    {5 Quartz Si O2}
    {6 Chlorite Clinochlore Mg5 Al2 Si3 O10 (OH)8 }
    {7 anorthite Ca Al2 Si2 O8 }
    {8 Forsterite Mg2 Si O4}
    {9 augite orthoferrosilite Fe Si O3 }
    {10 kaolinite Al2 Si2 O5 (OH)4}
    {11 Illite Si3.4 O10 (OH)2 Mg0.02 Al2.38 Ca0.01 K0.44, Godderis et al 2006}
    {12 Montmorillonite Si4 O10 (OH)2 Mg0.33 Al1.67 Ca0.04125 Mg0.04125 K0.0825 Na0.0825 }
    {13 hornblende Ca2 Mg4 Al Si6.7 Al O22.4 from Profile model }
    {14 fayalite Fe2 Si O4}
    {15 Amorphite}
    {16 Otherite}
    frAlbite:=    DataInit.WE.MinPools[1][1]/totalmass;
    frKfeldspar:= DataInit.WE.MinPools[2][1]/totalmass;
    frMuscovite:= DataInit.WE.MinPools[3][1]/totalmass;
    frBiotite:=   DataInit.WE.MinPools[4][1]/totalmass;
    frQuartz:=    DataInit.WE.MinPools[5][1]/totalmass;
    frChlorite:=  DataInit.WE.MinPools[6][1]/totalmass;
    frAnorthite:= DataInit.WE.MinPools[7][1]/totalmass;
    frFosterite:= DataInit.WE.MinPools[8][1]/totalmass;
    frAugite:=    DataInit.WE.MinPools[9][1]/totalmass;
    frKaolinite:= DataInit.WE.MinPools[10][1]/totalmass;
    frIllite:=    DataInit.WE.MinPools[11][1]/totalmass;
    frMontmorillonite:= DataInit.WE.MinPools[12][1]/totalmass;
    frHornblende:=DataInit.WE.MinPools[13][1]/totalmass ;
    frFayalite:=  DataInit.WE.MinPools[14][1]/totalmass;
    frAmorphite:=  DataInit.WE.MinPools[15][1]/totalmass;
    frOtherite:=  DataInit.WE.MinPools[16][1]/totalmass;
    frGibbsite:=  (26.982+3*(16.000+1))*0.001*DataInit.WE.Gibbsitepool[1]/totalmass;
    ShowLayer;
    strgrdEventData.Cells[0,0]:='SimulationYear';
    strgrdEventData.Cells[1,0]:='Event';
    strgrdEventData.Cells[2,0]:='No.Compartments';
    strgrdEventData.Cells[3,0]:='Clay Silt OC BulkDensity MoistureContent Calcite Gypsum '+
                                'pCO2 Ca-sol Mg-sol Na-sol K-sol Cl-sol SO4-sol Alkalinity '+
                                'Al-sol Fe3-sol Fe2-sol H4SiO4-sol DOC Ca-exch Mg-exch '+
                                'Na-exch K-exch H-exch Al-exch CEC Mg/Ca-select '+
                                'Ca/Na-select Ca/K-select Ca/Al-select Ca/H-select '+
                                'Albite K-Feldspar Muscovite Biotite Quartz Chlorite '+
                                'Anorthite Fosterite Augite Kaolinite Illite Montmorillonite '+
                                'Hornblende Fayalite Gibbsite Amorphite Otherite Gravel Stones';
    strgrdEventData.RowCount:=1;
    strgrdEventData.ColWidths[3]:=900;
    confirming:=false;
end;

procedure TfrmEvents.ShowLayer;
begin
  edtCLAY.Text:=val2str(CLAY);
  edtSILT.Text:=val2str(SILT);
  edtSAND.Text:=val2str(SAND);
  edtOC.Text:=val2str(OC);
  edtSTONES.text:=val2str(STONES); 
  edtGRAVEL.text:=val2str(GRAVEL);
  edtRHO.Text:=val2str(RHO);
  edtAW.Text:=val2str(AW);
  edtAT.Text:=val2str(ATEM);
  edtACALCITE.Text:=val2str(CALCITEA);
  edtAGYPS.Text:=val2str(GYPA);
  edtAPCO2.Text:=val2str(APCO2);
  edtCas.Text:=val2str(SEA[1]);
  edtMgs.Text:=val2str(SEA[2]);
  edtNAs.Text:=val2str(SEA[3]);
  edtKs.Text:=val2str(SEA[4]);
  edtCls.Text:=val2str(SEA[5]);
  edtSO4s.Text:=val2str(SEA[6]);
  edtAlks.Text:=val2str(SEA[7]);
  edtAls.Text:=val2str(SEA[12]);
  edtOrgs.Text:=val2str(SEA[13]);
  edtHOrgs.Text:=val2str(SEA[14]);
  edtH2Orgs.Text:=val2str(SEA[15]);
  edtDOCs.Text:=val2str(SEA[16]);
  edtCax.Text:=val2str(AXSE[1]);
  edtMgx.Text:=val2str(AXSE[2]);
  edtNax.Text:=val2str(AXSE[3]);
  edtKx.Text:=val2str(AXSE[4]);
  edtHx.Text:=val2str(AXSE[10]);
  edtAlx.Text:=val2str(AXSE[12]);
  edtCEC.Text:=val2str(CEC);
  edtSC1.Text:=val2str(SC1);
  edtSC2.Text:=val2str(SC2);
  edtSC3.Text:=val2str(SC3);
  edtSC7.Text:=val2str(SC7);
  edtSC8.Text:=val2str(SC8);
  edtfrAlbite.Text:=val2str(frAlbite);
  edtfrKfeldspar.Text:=val2str(frKfeldspar);
  edtfrMuscovite.Text:=val2str(frMuscovite);
  edtfrBiotite.Text:=val2str(frBiotite);
  edtfrQuartz.Text:=val2str(frQuartz);
  edtfrChlorite.Text:=val2str(frChlorite);
  edtfrAnorthite.Text:=val2str(frAnorthite);
  edtfrFosterite.Text:=val2str(frFosterite);
  edtfrAugite.Text:=val2str(frAugite);
  edtfrKaolinite.Text:=val2str(frKaolinite);
  edtfrIllite.Text:=val2str(frIllite);
  edtfrMontmorillonite.Text:=val2str(frMontmorillonite);
  edtfrHornblende.Text:=val2str(frHornblende);
  edtfrFayalite.Text:=val2str(frFayalite);
  edtfrGibbsite.Text:=val2str(frGibbsite);
  edtfrAmorphite.Text:=val2str(frAmorphite);
  edtfrOtherite.Text:=val2str(frOtherite);
  grpbxDepData.Refresh;
end;

function TfrmEvents.Val2Str(value:real):string;
var txt:string;
begin
  str(value:9:5,txt); val2str:=txt;
end;

function TfrmEvents.Str2Val(txt:string):real;
var code:integer; tmp:real;
begin
  val(txt,tmp,code);
  if code=0 then Str2Val:=tmp
  else begin str2Val:=-999; showmessage('Conversion error; check typed values'); end;
end;

procedure TfrmEvents.btbttnConfirmClick(Sender: TObject);
var txt:string; i,replacerow:integer; yearfound:boolean;
begin
  confirming:=true;
  CLAY:=str2val(edtCLAY.Text);             SILT:=str2val(edtSILT.Text);
  SAND:=str2val(edtSAND.text);             OC:=str2val(edtOC.text);
  GRAVEL:=str2val(edtGRAVEL.Text);         STONES:=str2val(edtSTONES.Text);
  RHO:=str2val(edtRHO.text);               AW:=str2val(edtAW.text);
  CALCITEA:=str2val(edtACALCITE.text);     CALCITEA:=CALCITEA*RHO*untGUI.DataInit.DLZ*(100-GRAVEL-STONES)/100;    {mass=RHO*DLZ*mfr*(1-stonefr-gravelfr)}
  GYPA:=str2val(edtAGYPS.text);            GYPA:=GYPA*RHO*untGUI.DataInit.DLZ*(100-GRAVEL-STONES)/100;    {mass=RHO*DLZ*mfr*(1-stonefr-gravelfr)}
  APCO2:=str2val(edtAPCO2.text);           SEA[1]:=str2val(edtCas.text);
  SEA[2]:=str2val(edtMgs.text);            SEA[3]:=str2val(edtNAs.text);
  SEA[4]:=str2val(edtKs.text);             SEA[5]:=str2val(edtCls.text);
  SEA[6]:=str2val(edtSO4s.text);           SEA[7]:=str2val(edtAlks.text);
  SEA[12]:=str2val(edtAls.text);           SEA[13]:=str2val(edtOrgs.text);
  SEA[14]:=str2val(edtHOrgs.text);         SEA[15]:=str2val(edtH2Orgs.text);
  SEA[16]:=str2val(edtDOCs.text);          AXSE[1]:=str2val(edtCax.text);
  AXSE[2]:=str2val(edtMgx.text);           AXSE[3]:=str2val(edtNax.text);
  AXSE[4]:=str2val(edtKx.text);            AXSE[10]:=str2val(edtHx.text);
  AXSE[12]:=str2val(edtAlx.text);          CEC:=str2val(edtCEC.text);
  SC1:=str2val(edtSC1.text);               SC2:=str2val(edtSC2.text);
  SC3:=str2val(edtSC3.text);               SC7:=str2val(edtSC7.text);
  SC8:=str2val(edtSC8.text);
  {check on Gapon coefficients too close to 0 and correct to a low value}
  if (SC1<0.00000001) then SC1:=0.00000001;
  if (SC2<0.00000001) then SC2:=0.00000001;
  if (SC3<0.00000001) then SC3:=0.00000001;
  if (SC7<0.00000001) then SC7:=0.00000001;
  if (SC8<0.00000001) then SC8:=0.00000001;
  CheckMinerals(scalefactor);
  {rescale fractions to sum up to 1}
  frAlbite:=scalefactor*str2val(edtfrAlbite.Text);
  frKfeldspar:=scalefactor*str2val(edtfrKfeldspar.Text);
  frMuscovite:=scalefactor*str2val(edtfrMuscovite.Text);
  frBiotite:=scalefactor*str2val(edtfrBiotite.Text);
  frQuartz:=scalefactor*str2val(edtfrQuartz.Text);
  frChlorite:=scalefactor*str2val(edtfrChlorite.Text);
  frAnorthite:=scalefactor*str2val(edtfrAnorthite.Text);
  frFosterite:=scalefactor*str2val(edtfrFosterite.Text);
  frAugite:=scalefactor*str2val(edtfrAugite.Text);
  frKaolinite:=scalefactor*str2val(edtfrKaolinite.Text);
  frIllite:=scalefactor*str2val(edtfrIllite.Text);
  frMontmorillonite:=scalefactor*str2val(edtfrMontmorillonite.Text);
  frHornblende:=scalefactor*str2val(edtfrHornblende.Text);
  frFayalite:=scalefactor*str2val(edtfrFayalite.Text);
  frAmorphite:=scalefactor*str2val(edtfrAmorphite.Text);
  frOtherite:=scalefactor*str2val(edtfrOtherite.Text);
  frGibbsite:=scalefactor*str2val(edtfrGibbsite.Text);
  {update editboxes after rescaling}
  edtfrAlbite.Text:=val2str(frAlbite);
  edtfrKfeldspar.Text:=val2str(frKfeldspar);
  edtfrMuscovite.Text:=val2str(frMuscovite);
  edtfrBiotite.Text:=val2str(frBiotite);
  edtfrQuartz.Text:=val2str(frQuartz);
  edtfrChlorite.Text:=val2str(frChlorite);
  edtfrAnorthite.Text:=val2str(frAnorthite);
  edtfrFosterite.Text:=val2str(frFosterite);
  edtfrAugite.Text:=val2str(frAugite);
  edtfrKaolinite.Text:=val2str(frKaolinite);
  edtfrIllite.Text:=val2str(frIllite);
  edtfrMontmorillonite.Text:=val2str(frMontmorillonite);
  edtfrHornblende.Text:=val2str(frHornblende);
  edtfrFayalite.Text:=val2str(frFayalite);
  edtfrAmorphite.Text:=val2str(frAmorphite);
  edtfrOtherite.Text:=val2str(frOtherite);
  edtfrGibbsite.Text:=val2str(frGibbsite);
  {check if Simyear is already in the stringgrid; if yes: overwrite;
   else add new line with settings}
  yearfound:=false; replacerow:=0;
  for i:=1 to strgrdEventData.RowCount-1 do
  begin
    if str2val(strgrdEventData.Cells[0,i])=spndtEventSimYear.Value
    then begin yearfound:=true; replacerow:=i; end;
  end;
  if yearfound
  then i:=replacerow
  else
  begin
    strgrdEventData.RowCount:=strgrdEventData.RowCount+1;
    strgrdEventData.FixedRows:=1;
    i:=strgrdEventData.RowCount-1;
  end;
  with strgrdEventData do
  begin
    str(spndtEventSimYear.Value:5,txt);
    Cells[0,i]:=txt;
    Cells[1,i]:=cmbbxEvent.Text;
    str(spndtNoComps.Value:2,txt);
    Cells[2,i]:=txt;
    case cmbbxEvent.ItemIndex of
      0: begin {erosion} Cells[3,i]:='NoData'; end;
      1: begin {deposition} Cells[3,i]:=edtCLAY.Text+'  '+
           edtSILT.Text+'  ' +edtOC.text+'  '      +edtRHO.text+'  '+
           edtAW.text+'  '   +edtACALCITE.text+'  '+edtAGYPS.text+'  '+
           edtAPCO2.text+'  '+edtCas.text+'  '     +edtMgs.text+'  '+
           edtNAs.text+'  '  +edtKs.text+'  '      +edtCls.text+'  '+
           edtSO4s.text+'  ' +edtAlks.text+'  '    +edtAls.text+'  '+
           edtOrgs.text+'  ' +edtHOrgs.text+'  '   +edtH2Orgs.text+'  '+
           edtDOCs.text+'  ' +edtCax.text+'  '     +edtMgx.text+'  '+
           edtNax.text+'  '  +edtKx.text+'  '      +edtHx.text+'  '+
           edtAlx.text+'  '  +edtCEC.text+'  '     +edtSC1.text+'  '+
           edtSC2.text+'  '  +edtSC3.text+'  '     +edtSC7.text+'  '+
           edtSC8.text+'  '+
           edtfrAlbite.text+'  '+edtfrKfeldspar.text+'  '+edtfrMuscovite.text+'  '+
           edtfrBiotite.text+'  '+edtfrQuartz.text+'  '+edtfrChlorite.text+'  '+
           edtfrAnorthite.text+'  '+edtfrFosterite.text+'  '+edtfrAugite.text+'  '+
           edtfrKaolinite.text+'  '+edtfrIllite.text+'  '+edtfrMontmorillonite.text+'  '+
           edtfrHornblende.text+'  '+edtfrFayalite.text+'  '+edtfrGibbsite.text+'  '+
           edtfrAmorphite.text+'  '+edtfrOtherite.text+'  '+
           edtGRAVEL.text+'  '+edtSTONES.text;
           if (scalefactor>1.001)or(scalefactor<0.999)
           then showmessage('Mass fractions of minerals did not sum to 1'+#13#10+
               'All values were rescaled.');
         end;
      2: begin {water table} Cells[3,i]:='NoData'; end;
      3: begin {VGNChange} Cells[3,i]:=edtVGNfilename.Text; end;
      4: begin {Plowing} Cells[3,i]:=edtPlowMixFraction.Text; end;
      5: begin {SlashBurn} Cells[3,i]:=edtSlashBurnCO2.Text+' '+edtSlashBurnIOM.Text+' '+
                                       edtSlashBurnHUM.Text; end;
    end;
  end;
  {sort the grid so that inputlines start with the lowest value of SimYear}
  sortstrgrd;
  confirming:=false;
end;

procedure TfrmEvents.sortstrgrd;
var txt1,txt2,txt3,txt4:string; i:integer; year1,year2:real; ready:boolean;
begin
  {bubble sort}
  ready:=false;
  while not(ready) do
  begin
    ready:=true;
    for i:=1 to strgrdEventData.RowCount-2 do
    begin
       year1:=str2val(strgrdEventData.Cells[0,i]);
       year2:=str2val(strgrdEventData.Cells[0,i+1]);
       if year1>year2 then {must swap}
       begin
         ready:=false;
         with strgrdEventData do
         begin
           txt1:=cells[0,i]; txt2:=cells[1,i]; txt3:=cells[2,i]; txt4:=cells[3,i];
           cells[0,i]:=cells[0,i+1]; cells[1,i]:=cells[1,i+1];
           cells[2,i]:=cells[2,i+1]; cells[3,i]:=cells[3,i+1];
           cells[0,i+1]:=txt1; cells[1,i+1]:=txt2;
           cells[2,i+1]:=txt3; cells[3,i+1]:=txt4;
         end;
       end;
    end;
  end;

end;

procedure TfrmEvents.btbttnEventImportClick(Sender: TObject);
begin
  if pndlgOPEN.Execute
  then LeesData(pndlgOPEN.FileName);
end;

procedure TfrmEvents.LeesData(fn:string);
var f:textfile; txt1,txt2,txt3,txt:string; counter:integer;
begin
  confirming:=false;
  system.assign(f,fn); system.Reset(f);
  leeswoord(f,txt1); leeswoord(f,txt2); leeswoord(f,txt3); system.readln(f,txt);
  if not((txt1='SimulationYear')
          and (txt2='Event')
          and (txt3='No.Compartments'))
  then showmessage('Format of the header line of '+fn+' is not correct; file will not be read.')
  else
  begin
    counter:=0;
    {strgrdEventData.Cells[0,0]:=txt1;
    strgrdEventData.Cells[0,1]:=txt2;
    strgrdEventData.Cells[0,2]:=txt3;
    strgrdEventData.Cells[0,3]:=txt; }
    strgrdEventData.Cells[0,0]:=txt1;
    strgrdEventData.Cells[1,0]:=txt2;
    strgrdEventData.Cells[2,0]:=txt3;
    strgrdEventData.Cells[3,0]:=txt;
    repeat leeswoord(f,txt1); leeswoord(f,txt2); leeswoord(f,txt3); system.readln(f,txt);
      counter:=counter+1; strgrdEventData.RowCount:=strgrdEventData.RowCount+1;
      strgrdEventData.Cells[0,counter]:=txt1;  strgrdEventData.Cells[1,counter]:=txt2;
      strgrdEventData.Cells[2,counter]:=txt3;  strgrdEventData.Cells[3,counter]:=txt;
    until eof(f);
    strgrdEventData.FixedRows:=1;
  end;
  closefile(f);
end;

procedure TfrmEvents.LeesWoord(var f:textfile; var woord:string);
var a:char;
begin
  repeat read(f,a) until a<>' '; woord:=a;
  repeat read(f,a); if a<>' 'then woord:=woord+a until (a=' ')or(eoln(f));
end;

procedure TfrmEvents.btbttnCloseClick(Sender: TObject);
begin
  if FileExistsUTF8('Events.txt') { *Converted from FileExists*  } then DeleteFileUTF8('Events.txt'); { *Converted from DeleteFile*  }
   untGUI.frmMAIN.chckbxEvents.Checked:=false;
   untGUI.frmMAIN.chckbxEvents.Enabled:=false;
   untGUI.frmMAIN.btbttnEventsChange.Enabled:=true;
   close;
end;

procedure TfrmEvents.btbttnSaveCloseClick(Sender: TObject);
var f:textfile; i,j:integer;
begin
  system.assign(f,'Events.txt'); system.Rewrite(f);
  for i:=0 to strgrdEventData.RowCount-1 do
  begin
    for j:=0 to 3 do system.write(f,strgrdEventData.Cells[j,i],' ');
    system.writeln(f);
  end;
  closefile(f);
  untGUI.frmMAIN.chckbxEvents.Checked:=true;
  untGUI.frmMAIN.chckbxEvents.enabled:=true;
  untGUI.frmMAIN.btbttnEventsChange.Enabled:=false;
  close;
end;

procedure TfrmEvents.bttnPlowMixFractionClick(Sender: TObject);
var code:integer;  testfrac:real;
begin
  val(edtPlowMixFraction.Text,testfrac,code);
  if (code<>0)or(testfrac<0)or(testfrac>0.999)then edtPlowMixFraction.Text:='0.5';
end;

procedure TfrmEvents.Deleterowundercursor1Click(Sender: TObject);
var i,j:integer;
begin
  with strgrdEventData do
  begin
    for i:=row to rowcount-2 do for j:=0 to colcount-1 do
      cells[j,i]:=cells[j,i+1];
    rowcount:=rowcount-1;
    refresh;
  end;
end;

procedure TfrmEvents.CheckMinerals(var scalefactor:real);
var code:integer;
    txt:string;
    fout:boolean;
begin
  if not(confirming)then
  begin
   val(edtfrAlbite.Text,frAlbite,code); val(edtfrKfeldspar.Text,frKfeldspar,code);
   val(edtfrBiotite.Text,frBiotite,code); val(edtfrMuscovite.Text,frMuscovite,code);
   val(edtfrQuartz.Text,frQuartz,code); val(edtfrChlorite.Text,frChlorite,code);
   val(edtfrAnorthite.Text,frAnorthite,code); val(edtfrFosterite.Text,frFosterite,code);
   val(edtfrAugite.Text,frAugite,code); val(edtfrKaolinite.Text,frKaolinite,code);
   val(edtfrIllite.Text,frIllite,code); val(edtfrMontmorillonite.Text,frMontmorillonite,code);
   val(edtfrHornblende.Text,frHornblende,code);
   val(edtfrGibbsite.Text,frGibbsite,code);
   val(edtfrAmorphite.Text,frAmorphite,code);
   val(edtfrOtherite.Text,frOtherite,code);

   fout:=(1-frAlbite-frKfeldspar-frMuscovite-frBiotite-frQuartz-frChlorite-frAnorthite-
    frFosterite-frAugite-frKaolinite-frIllite-frMontmorillonite-frHornblende-
    frFayalite-frGibbsite-frAmorphite-frOtherite<>0);
   scalefactor:=1;
   {if (fout)and(not(confirming)) then
   showmessage('Mass fractions of minerals do not sum to 1'+#13#10+
               'If you do not change all values will be rescaled when you press <confirm>.');}
  end
  else scalefactor:=1/(frAlbite+frKfeldspar+frMuscovite+frBiotite+frQuartz+frChlorite+frAnorthite+
    frFosterite+frAugite+frKaolinite+frIllite+frMontmorillonite+frHornblende+
    frFayalite+frGibbsite+frAmorphite+frOtherite);
end;

function TfrmEvents.CheckVal(txt:string):real;
var code:integer; value:real;
begin
  system.Val(txt,value,code);
  if code<>0 then
  begin showmessage('Enter a value'); CheckVal:=-999; end
  else CheckVal:=value;
end;

procedure TfrmEvents.CheckData;
var tCLAY, tSILT, tOC, tSAND, tACEC, tgravel, tstones:real;
  tAXSE:array[1..12]of real;
begin
  tCLAY:=CheckVal(edtCLAY.Text);   tSILT:=CheckVal(edtSILT.Text); ;
  tOC:=CheckVal(edtOC.Text);       tSAND:=CheckVal(edtSAND.Text);
  tgravel:=CheckVal(edtGravel.Text); tstones:=CheckVal(edtStones.Text);
  tAXSE[1]:=CheckVal(edtCax.Text);  tAXSE[2]:=CheckVal(edtMgx.Text);
  tAXSE[3]:=CheckVal(edtNax.Text);  tAXSE[4]:=CheckVal(edtKx.Text);
  tAXSE[10]:=CheckVal(edtHx.Text);  tAXSE[12]:=CheckVal(edtAlx.Text);
  tACEC:=tAXSE[1]+tAXSE[2]+tAXSE[3]+tAXSE[4]+
               tAXSE[10]+tAXSE[12];
  edtCEC.Text:=val2str(tACEC);
end;

end.
