unit untImportClimateData;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$MODE Delphi}

{$DEBUGINFO ON} 
interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, ExtCtrls, Buttons, Menus, ComCtrls, StrUtils, Math;

const
  mistxt='missing';

type

  { TfrmImportClimateData }

  TfrmImportClimateData = class(TForm)
    dlgOPENBUISDATA: TOpenDialog;
    edtLiethFactor: TEdit;
    rdgrpNPP: TRadioGroup;
    strgrdSTAMBUIS: TStringGrid;
    chckbxMultiple: TCheckBox;
    rdgrpSEP: TRadioGroup;
    btbttnOKLEES: TBitBtn;
    lblstrgrdSTAMBUIS: TLabel;
    stsbrSTAMBUIS: TStatusBar;
    ppmnGRID: TPopupMenu;
    Toevoegenrecordmetbuisgegevens1: TMenuItem;
    Omzettenvanelkekommanaardecimalepunt1: TMenuItem;
    grpbxKOLOM: TGroupBox;
    Verwijderenrecordmetbuisgegevens1: TMenuItem;
    cmbbxP: TComboBox;
    cmbbxJanT: TComboBox;
    cmbbxPlantC: TComboBox;
    cmbbxManureC: TComboBox;
    cmbbxJulT: TComboBox;
    cmbbxPE: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    btbttnEXITSTAMBUIS: TBitBtn;
    cmbbxVegetation: TComboBox;
    Label7: TLabel;
    Label8: TLabel;
    cmbbxComments: TComboBox;
    Label9: TLabel;
    cmbbxYearBP: TComboBox;
    procedure FormActivate(Sender: TObject);
    procedure btbttnOKLEESClick(Sender: TObject);
    procedure strgrdSTAMBUISMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure btbttnOKLEESMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure rdgrpSEPClick(Sender: TObject);
    procedure Toevoegenrecordmetbuisgegevens1Click(Sender: TObject);
    procedure Omzettenvanelkekommanaardecimalepunt1Click(Sender: TObject);
    procedure Bewareninbestand1Click(Sender: TObject);
    procedure Verwijderenrecordmetbuisgegevens1Click(Sender: TObject);
    procedure grpbxKOLOMMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure chckbxMultipleMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure Toevoegenkolommetmetingen1Click(Sender: TObject);
    procedure Verwijderenkolomonderdecursor1Click(Sender: TObject);
    procedure strgrdSTAMBUISKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btbttnEXITSTAMBUISClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure bepaalsep(FileName:string);
    procedure vulgrid(FileName:string);
    Procedure leesveld(nr:integer; regel:string;
                       var woord:string);
    procedure insertrow(irow:integer);
    procedure insertcol(icol:integer);
    procedure vulboxen;
  end;

var
  frmImportClimateData: TfrmImportClimateData;
  calling:(bestand,handinvoer);
  sep:char;
  FileName:string;
  startlees,rowins,colins,rowdel,coldel,NoInput:boolean;
  nosep,kopregels,datanr,
  kolP,kolPE,koljanT,kolJulT,kolPlantC,kolManureC,kolVegetation,kolComments,kolYearBP:integer;
  potsep:array[1..20] of char;
  potsepcount:array[1..20] of integer;
  LiethFactor,PlantC,MAT,MAP:real;

implementation

{$R *.lfm}
uses untClimate, untGUI;

procedure TfrmImportClimateData.FormActivate(Sender: TObject);
var i,j:integer;
begin
  {altijd uitvoeren:}
  dlgOPENBUISDATA.InitialDir:=startdir;
  kopregels:=1; datanr:=0;
  NoInput:=false;
  strgrdSTAMBUIS.visible:=false;
  lblstrgrdSTAMBUIS.visible:=false;
  strgrdSTAMBUIS.Enabled:=false;
  strgrdSTAMBUIS.FixedRows:=kopregels;
  STARTLEES:=false;
  rdgrpSEP.ItemIndex:=-1;
  rdgrpSEP.Items.Clear;
  RowIns:=false; RowDel:=false;
  ColIns:=false; ColDel:=false;
  btbttnEXITSTAMBUIS.visible:=true;
  grpbxKOLOM.Visible:=false;
  btbttnOKLEES.Caption:='OK, start reading';
  btbttnOKLEES.Hint:='Press this button if the right delimiter sign is set'+
                     ' and the above option is (or not) selected';

  {afhankelijk van type aanroep:}
  if calling=bestand then
  begin
    btbttnEXITSTAMBUIS.enabled:=false;
    {form wordt aangeroepen met als doel een bestand
    in te lezen en vervolgens de tabel op te maken}
    {1. Een dialoogvenster wordt geopend waarmee
        een bestand met stambuisgegevens
        wordt aangekoppeld.
        Eisen aan het bestand:
        + ASCII
        + gestructureerd, dwz elke regel (record) bevat alle gegevens
          van 1 buis, en er is een constant aantal kolommen (fields)
          gescheiden door spaties, tabs, puntkomma's}
    if dlgOPENBUISDATA.execute then
    begin
    {2. Het aangekoppelde bestand wordt ingelezen en opgemaakt, dwz:
      2init:}
      grpbxKOLOM.visible:=false;
      rdgrpSEP.VISIBLE:=true;
      chckbxMULTIPLE.visible:=true;
      btbttnOKLEES.visible:=true;
      {2a. er wordt afgeleid wat het scheidingsteken tussen de kolommen is}
      bepaalsep(dlgOPENBUISDATA.FileName);
      {overige activiteiten uitgevoerd na indrukken knop <btbttnOKLEES>}
    end
    else
    begin
      NoInput:=true;
      rdgrpSEP.Visible:=false;
      chckbxMULTIPLE.visible:=false;
      btbttnEXITSTAMBUIS.visible:=false;
      btbttnOKLEES.visible:=true;
      btbttnOKLEES.Caption:='No input; abort';
      btbttnOKLEES.Hint:='No input file; '+
                         'press button to start again';
    end;
  end
  else
  if calling=handinvoer then
  begin
    rdgrpSEP.Visible:=false;
    chckbxMULTIPLE.visible:=false;
    btbttnOKLEES.visible:=false;
    btbttnEXITSTAMBUIS.enabled:=true;
    {form wordt aangeroepen met als doel live ingeklopte gegevens
    in te lezen en vervolgens de tabel op te maken}
    {1. Opmaak
       + er wordt gevraagd in hoeveel/welke kolommen
         de meetgegevens staan (format: kolom voor datum1 met meting1;
         kolom voor datum2 met meting2 etc)
       + er wordt gevraagd in welke kolom de GHG, GVG, GLG etc staan
       + er wordt gevraagd in welke kolom
         betrouwbaarheid GHG etc staat (sdGHG) en
         of deze getallen nog gekwadrateerd moeten worden
       + er wordt gevraagd hoeveel buizen (records) er zijn
       + code voor missing data ingeven}
    SEP:=';';
    strgrdSTAMBUIS.RowCount:=4;
    strgrdSTAMBUIS.ColCount:=9;
    strgrdSTAMBUIS.visible:=true;
    strgrdSTAMBUIS.enabled:=true;
    strgrdSTAMBUIS.refresh;
    lblstrgrdSTAMBUIS.Caption:='manual input data';
    lblstrgrdSTAMBUIS.Visible:=true;
    lblstrgrdSTAMBUIS.Refresh;

    with strgrdSTAMBUIS do
    begin
      Cells[0,0]:='YearBP';     Cells[1,0]:='Precipitation';
      Cells[2,0]:='Pot.evapotranspiration';      Cells[3,0]:='JanT';
      Cells[4,0]:='JulT';      Cells[5,0]:='PlantC';
      Cells[6,0]:='ManureC';      Cells[7,0]:='Vegetation';
      Cells[8,0]:='Comments';
      for i:=1 to RowCount-1 do
      for j:=0 to ColCount-1 do
        Cells[j,i]:=mistxt;
    end;
    grpbxKOLOM.visible:=true;
    vulboxen;
    {2. Invoer/editen/wegschrijven
       + getallen invullen
       + evt alle komma's naar decimale punten omzetten
       + mogelijkheid om handmatig een record (regel) met nieuwe buisgegevens
         toe te voegen
       + mogelijkheid om file te bewaren
       + scherm afsluiten wanneer gereed  }
  end;
end;

procedure TfrmImportClimateData.bepaalsep(FileName:string);
var ftemp:textfile; regel,si:string;
    i,j:integer;
    toegelaten:set of char;
begin
  {init}
  for i:=1 to 20 do begin potsep[i]:='x'; potsepcount[i]:=0; end;
  toegelaten:=['1','2','3','4','5','6','7','8','9','0','.',
               'a','b','c','d','e','f','g','h','i','j','k',
               'l','m','n','o','p','q','r','s','t','u','v',
               'w','x','y','z','''','"',
               'A','B','C','D','E','F','G','H','I','J','K',
               'L','M','N','O','P','Q','R','S','T','U','V',
               'W','X','Y','Z'];
  regel:='';
  {inlezen eerste regel uit bestand}
  system.assign(ftemp,FileName); system.reset(ftemp);
  system.readln(ftemp,regel);
  closefile(ftemp);
  {uit ingelezen regel <regel> alle niet-scheidingstekens verwijderen.
   alle niet-scheidingstekens zitten in de set <toegelaten>}
  i:=1;
  repeat
    if regel[i] in toegelaten
    then delete(regel,i,1)
    else i:=i+1;
  until i>length(regel);
  {nu unieke resterende tekens tellen en verwijderen uit <regel>}
  nosep:=0;
  while length(regel)>0 do
  begin
    nosep:=nosep+1;
    potsep[nosep]:=regel[1];
    for j:=1 to length(regel) do
    if regel[j]=regel[1] then potsepcount[nosep]:=potsepcount[nosep]+1;
    {verwijderen scheidingsteken <potsep[nosep]> uit <regel>}
    j:=1;
    repeat
      if regel[j]=potsep[nosep]
      then delete(regel,j,1)
      else j:=j+1;
    until j>length(regel);
  end;
  {potentiële scheidingstekens met max. frequentie zoeken}
  j:=1; sep:=potsep[j]; {rdgrpSEP.ItemIndex:=j-1;}
  {alle gevonden scheidingstekens in radiogroup zetten;
   meest frequent gevonden scheidingsteken "aanzetten".}
  rdgrpSEP.Height:=16+nosep*20;
  for i:=1 to nosep do
  begin
    si:=potsep[i];
    if ord(potsep[i])=32 then si:='space';
    if ord(potsep[i])=9 then si:='tab-sign';
    rdgrpSEP.Items.Add(si);
    if potsepcount[i]>potsepcount[j] then
    begin j:=i; sep:=potsep[i]; end;
  end;
  rdgrpSEP.ItemIndex:=j-1;
end;

procedure TfrmImportClimateData.vulgrid(FileName:string);
var f:textfile; regel,delstr,woord:string;
    i,j,nocols,norows:integer;
begin
  system.assign(f,FileName); system.reset(f);
  {geselecteerd scheidingsteken toepasbaar maken}
  sep:=potsep[rdgrpSEP.ItemIndex+1];
  {aantal kolommen bepalen}
  system.Readln(f,regel);
  {indien ingesteld: verwijder dubbele scheidingstekens}
  if chckbxMultiple.checked then
  begin
    delstr:=sep+sep;
    while pos(delstr,regel)>0 do delete(regel,pos(delstr,regel),1);
  end;
  {remove trailing separators}
  while regel[1]=sep do delete(regel,1,1);
  {now the text line starts with data}
  i:=1;
  repeat
    leesveld(i,regel,woord);
    i:=i+1;
  until woord='';
  nocols:=i-2;
  {aantal regels bepalen}
  i:=1;
  repeat
    readln(f,regel); i:=i+1;
  until (eof(f));
  norows:=i;
  {nu inlezen}
  strgrdSTAMBUIS.RowCount:=norows;
  strgrdSTAMBUIS.ColCount:=nocols;
  system.reset(f);
  for i:=0 to norows-1 do
  begin
    system.readln(f,regel);
    {indien ingesteld: verwijder dubbele scheidingstekens}
    if chckbxMultiple.checked then
    begin
      delstr:=sep+sep;
      while pos(delstr,regel)>0 do delete(regel,pos(delstr,regel),1);
    end;
    {remove trailing separators}
    while regel[1]=sep do delete(regel,1,1);
    {now the text line starts with data}
    for j:=0 to nocols-1 do
    begin
      leesveld(j+1,regel,woord);
      strgrdSTAMBUIS.Cells[j,i]:=woord;
    end;
  end;
  strgrdSTAMBUIS.visible:=true;
  strgrdSTAMBUIS.enabled:=true;
  strgrdSTAMBUIS.refresh;
  closefile(f);
end;

procedure TfrmImportClimateData.btbttnOKLEESClick(Sender: TObject);
var txt:string;
begin
if not(NoInput) then
begin
  StartLees:=true;
  txt:=dlgOPENBUISDATA.Filename;
  if length(txt)>60 then txt:=copy(txt,1,3)+'...'+rightstr(txt,54);
  lblstrgrdSTAMBUIS.Caption:='Read data from '+txt;
  lblstrgrdSTAMBUIS.visible:=true;
  rdgrpSEP.VISIBLE:=FALSE;
  chckbxMULTIPLE.visible:=FALSE;
  btbttnOKLEES.visible:=false;
  vulgrid(dlgOPENBUISDATA.FileName);
  grpbxKOLOM.Visible:=true;
  vulboxen;
  btbttnEXITSTAMBUIS.enabled:=true;
end
else close;
end;

Procedure TfrmImportClimateData.leesveld(nr:integer; regel:string;
                            var woord:string);
var telveld:integer;
{krijgt een regel uit een <sep>-delimited tekst binnen
 en selecteert het woord <woord> voor de <nr>de separator <sep>.
 indien deze <sep> er niet is is <woord> een lege string}
begin
  for telveld:=1 to nr do
  begin
    if pos(sep,regel)>0
    then begin
           woord:=copy(regel,1,pos(sep,regel)-1);
           delete(regel,1,pos(sep,regel));
         end
    else if length(regel)>0
         then begin
                woord:=regel;
                regel:='';
              end
         else woord:='';
    if woord=' ' then woord:='';
  end;
end;

procedure TfrmImportClimateData.strgrdSTAMBUISMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   stsbrSTAMBUIS.SimpleText:=strgrdSTAMBUIS.Hint;
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportClimateData.btbttnOKLEESMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   stsbrSTAMBUIS.SimpleText:=btbttnOKLEES.Hint;
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportClimateData.FormMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
   stsbrSTAMBUIS.SimpleText:='';
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportClimateData.rdgrpSEPClick(Sender: TObject);
begin
   stsbrSTAMBUIS.SimpleText:=rdgrpSEP.Hint;
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportClimateData.insertrow(irow:integer);
var i,j:integer;
begin
  with strgrdSTAMBUIS do
  begin
    rowcount:=rowcount+1;
    for i:=rowcount-2 downto irow do
    begin
      for j:=0 to colcount-1 do
      cells[j,i+1]:=cells[j,i];
    end;
    cells[0,irow]:='extrapiezo';
    for j:=1 to colcount-1 do
      cells[j,irow]:=mistxt;
    if rowcount>1 then fixedrows:=kopregels;
    refresh;
  end;
end;

procedure TfrmImportClimateData.Toevoegenrecordmetbuisgegevens1Click(
  Sender: TObject);
begin
  insertrow(kopregels);
  Omzettenvanelkekommanaardecimalepunt1.Checked:=false;
  RowIns:=true;
end;

procedure TfrmImportClimateData.Omzettenvanelkekommanaardecimalepunt1Click(
  Sender: TObject);
var i,j:integer;
   astr:string;
begin
  with strgrdSTAMBUIS do
  begin
    for i:=0 to colcount-1 do
    for j:=0 to rowcount-1 do
    while pos(',',cells[i,j])>0 do
    begin
      astr:=cells[i,j];
      astr[pos(',',astr)]:='.';
      cells[i,j]:=astr;
    end;
  end;
  Omzettenvanelkekommanaardecimalepunt1.Checked:=true;
end;

procedure TfrmImportClimateData.Bewareninbestand1Click(Sender: TObject);
var ftemp:textfile; i,j:integer;
begin
  system.assign(ftemp,'datagrid.csv');
  system.rewrite(ftemp);
  with strgrdSTAMBUIS do
  begin
    for i:=0 to rowcount-1 do
    begin
      for j:=0 to colcount-1 do system.write(ftemp,cells[j,i],sep);
      system.writeln(ftemp);
    end;
  end;
  closefile(ftemp);
  showmessage('Current data set written to <datagrid.csv>');
end;

procedure TfrmImportClimateData.Verwijderenrecordmetbuisgegevens1Click(
  Sender: TObject);
var i,j:integer;
begin
  with strgrdSTAMBUIS do
  begin
    for i:=row to rowcount-2 do for j:=0 to colcount-1 do
      cells[j,i]:=cells[j,i+1];
    rowcount:=rowcount-1;
    refresh;
  end;
  Omzettenvanelkekommanaardecimalepunt1.Checked:=false;
  RowDel:=true;
end;

procedure TfrmImportClimateData.grpbxKOLOMMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   stsbrSTAMBUIS.SimpleText:=grpbxKOLOM.Hint;
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportClimateData.chckbxMultipleMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   stsbrSTAMBUIS.SimpleText:=chckbxMultiple.Hint;
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportClimateData.insertcol(icol:integer);
var i,j:integer; datstr:string;
begin
  with strgrdSTAMBUIS do
  begin
    colcount:=colcount+1;
    for i:=colcount-2 downto icol do
    begin
      for j:=0 to rowcount-1 do
      cells[i+1,j]:=cells[i,j];
    end;
    datanr:=datanr+1;
    str(datanr,datstr);
    cells[icol,0]:='data'+datstr;
    for j:=1 to rowcount-1 do
      cells[icol,j]:=mistxt;
    refresh;
  end;
end;

procedure TfrmImportClimateData.Toevoegenkolommetmetingen1Click(Sender: TObject);
var ii:integer;
begin
  ii:=strgrdSTAMBUIS.ColCount;
  insertcol(ii);
  Omzettenvanelkekommanaardecimalepunt1.Checked:=false;
  vulboxen;
  ColIns:=true;
end;

procedure TfrmImportClimateData.Verwijderenkolomonderdecursor1Click(
  Sender: TObject);
var i,j:integer;
begin
  with strgrdSTAMBUIS do
  begin
    for i:=col to colcount-2 do for j:=0 to rowcount-1 do
      cells[i,j]:=cells[i+1,j];
    colcount:=colcount-1;
    refresh;
  end;
  Omzettenvanelkekommanaardecimalepunt1.Checked:=false;
  vulboxen;
  ColDel:=true;
end;

procedure TfrmImportClimateData.strgrdSTAMBUISKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  Omzettenvanelkekommanaardecimalepunt1.Checked:=false;
  vulboxen;
end;

procedure TfrmImportClimateData.vulboxen;
var i:integer; 
begin
    cmbbxP.Clear;       cmbbxJanT.Clear;      cmbbxPlantC.Clear;
    cmbbxManureC.Clear; cmbbxJulT.Clear;      cmbbxPE.Clear;
    cmbbxVegetation.Clear;  cmbbxComments.Clear; cmbbxYearBP.Clear;
  {vullen alle comboboxen en listbox TITELS met beschikbare kolomtitels}
  for i:=0 to strgrdSTAMBUIS.ColCount-1 do
  begin
    cmbbxP.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxPE.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxJanT.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxJulT.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxPlantC.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxManureC.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxVegetation.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxComments.Items.Add(strgrdSTAMBUIS.Cells[i,0]);   
    cmbbxYearBP.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
  end;

  {Vul in het Text veld de titel in die het meest voor de hand ligt}
  for i:=strgrdSTAMBUIS.ColCount-1 downto 0 do
  begin
    if ((pos('Prec',strgrdSTAMBUIS.Cells[i,0])>0)or
       (pos('P',strgrdSTAMBUIS.Cells[i,0])>0))and
       not((pos('BP',strgrdSTAMBUIS.Cells[i,0])>0)or
           (pos('PE',strgrdSTAMBUIS.Cells[i,0])>0)or
           (pos('EP',strgrdSTAMBUIS.Cells[i,0])>0))
    then cmbbxP.Text:=strgrdSTAMBUIS.Cells[i,0];
    if (pos('Pot',strgrdSTAMBUIS.Cells[i,0])>0)or
       (pos('PE',strgrdSTAMBUIS.Cells[i,0])>0)or
       (pos('EP',strgrdSTAMBUIS.Cells[i,0])>0)
    then cmbbxPE.Text:=strgrdSTAMBUIS.Cells[i,0];
    if pos('Jan',strgrdSTAMBUIS.Cells[i,0])>0
    then cmbbxJanT.Text:=strgrdSTAMBUIS.Cells[i,0];
    if pos('Jul',strgrdSTAMBUIS.Cells[i,0])>0
    then cmbbxJulT.Text:=strgrdSTAMBUIS.Cells[i,0];
    if pos('Plant',strgrdSTAMBUIS.Cells[i,0])>0
    then cmbbxPlantC.Text:=strgrdSTAMBUIS.Cells[i,0];
    if pos('Manure',strgrdSTAMBUIS.Cells[i,0])>0
    then cmbbxManureC.Text:=strgrdSTAMBUIS.Cells[i,0];
    if (pos('Veg',strgrdSTAMBUIS.Cells[i,0])>0)
    then cmbbxVegetation.Text:=strgrdSTAMBUIS.Cells[i,0];
    if (pos('Period',strgrdSTAMBUIS.Cells[i,0])>0)
    then cmbbxComments.Text:=strgrdSTAMBUIS.Cells[i,0];  
    if (pos('Year',strgrdSTAMBUIS.Cells[i,0])>0)
    then cmbbxYearBP.Text:=strgrdSTAMBUIS.Cells[i,0];
  end;
end;

procedure TfrmImportClimateData.btbttnEXITSTAMBUISClick(Sender: TObject);
var ftemp,fdrab:textfile;
    i,j,code,yearBP:integer;
    tjan,tjul,MAP,MAT,NPPbaseline,fTmax,fTmin,gPsummer,gPwinter,Psummer,Pwinter,ppmCO2:real;
begin
  {initialiseren}
  kolP:=-1; kolPE:=-1; koljanT:=-1; kolJulT:=-1; kolPlantC:=-1; kolManureC:=-1;
  kolVegetation:=-1; kolComments:=-1;
  val(edtliethFactor.Text,LiethFactor,code);
  if code<>0 then LiethFactor:=1;
  {wegschrijven naar bestand <temp.txt> van de kolommen en
   titels van de relevante variabelen}
  system.assign(ftemp,'tempClimate.txt'); system.rewrite(ftemp);
  with strgrdSTAMBUIS do
  begin      
    for j:=0 to ColCount-1 do
    begin
      if cmbbxP.Text=Cells[j,0]
      then kolP:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxPE.Text=Cells[j,0]
      then kolPE:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxJanT.Text=Cells[j,0]
      then kolJanT:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxJulT.Text=Cells[j,0]
      then kolJulT:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxPlantC.Text=Cells[j,0]
      then kolPlantC:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxManureC.Text=Cells[j,0]
      then kolManureC:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxVegetation.Text=Cells[j,0]
      then kolVegetation:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxComments.Text=Cells[j,0]
      then kolComments:=j;
    end;     
    for j:=0 to ColCount-1 do
    begin
      if cmbbxYearBP.Text=Cells[j,0]
      then kolYearBP:=j;
    end;
    {read pCO2 inputfile first, if the Checkbox chckbxSaballa is Checked}
    if rdgrpNPP.ItemIndex=2 then
    begin
      frmMain.pndlgOpen.title:='Open file with CO2-concentrations';
      frmMain.btbttnCO2Click(Sender);
      frmMain.pndlgOpen.title:='Open existing file';
    end;
    for i:=0 to rowcount-1 do
    begin
      if kolYearBP<>-1 then system.write(ftemp,cells[kolYearBP,i],' ') else system.write(ftemp,'-1;');
      if kolP<>-1 then system.write(ftemp,cells[kolP,i],' ') else system.write(ftemp,'-1;');
      if kolPE<>-1 then system.write(ftemp,cells[kolPE,i],' ') else system.write(ftemp,'-1;');
      if kolJanT<>-1 then system.write(ftemp,cells[kolJanT,i],' ') else system.write(ftemp,'-1;');
      if kolJulT<>-1 then system.write(ftemp,cells[kolJulT,i],' ') else system.write(ftemp,'-1;');
      if kolPlantC<>-1 then
      begin
        if (rdgrpNPP.ItemIndex=1)and(i<>0)
        then
        begin
          val(cells[kolP,i],MAP,code);
          val(cells[kolJanT,i],tjan,code); val(cells[kolJulT,i],tjul,code); MAT:=tjan+(tjul-tjan)/2;
          val(cells[kolYearBP,i],yearBP,code);
          PlantC:=Liethfactor*30*MIN(1/(1+exp(1.315-0.119*MAT)) , 1-exp(-0.00064*MAP));
        end
        else if (rdgrpNPP.ItemIndex=2)and(i<>0)
        then
        begin
          val(cells[kolP,i],MAP,code);
          val(cells[kolJanT,i],tjan,code); val(cells[kolJulT,i],tjul,code);
          val(cells[kolYearBP,i],yearBP,code);
          ppmCO2:=1000000*SeriespCO2[yearBP];  {value pCO2 as ppm read from inputfile}
          if untGUI.frmMain.spndtLatitude.Value<0 then  {southern hemisphere: swap summer and winter}
          begin
            Psummer:=untGUI.Pjfmond*MAP/(untGUI.Pjfmond+untGUI.Pamjjas);
            Pwinter:=untGUI.Pamjjas*MAP/(untGUI.Pjfmond+untGUI.Pamjjas);
            {from article Saballa et al 2015, but coefficients given there do no reproduce their Fig.3! Trial and error refit.
            fTmax:=9.78*power(tjan*0.2,1.65)*exp(-(tjan*0.2)/2)/(power(2,2.65*1.485193));
            fTmin:=18.72*power((tjul+40)*1.31,6.3)*exp(-1.31*(tjul+40)/2)/power(2,7.3*1271.424);}
            fTmax:=9.78*POWER(tjan*0.2,5.3/2-1)*EXP(-tjan*0.2/2)/POWER(2,(5.3/2)*1.485193);
            fTmin:=12.0*POWER((tjul+40)*0.34,(14.6/2-1))*EXP(-0.34*(tjul+40)/2)/(POWER(2,(14.6/2))*1271.424)
          end
          else  {northern hemisphere}
          begin
            Psummer:=untGUI.Pamjjas*MAP/(untGUI.Pjfmond+untGUI.Pamjjas);
            Pwinter:=untGUI.Pjfmond*MAP/(untGUI.Pjfmond+untGUI.Pamjjas);
            {from article Salabba et al 2015, but coefficients given there do no reproduce their Fig.3! Trial and error refit,
            with a vertical shift added. Salabba stated:
            fTmax:=9.78*power(tjul*1.92,1.65)*exp(-(tjul*1.92)/2)/(power(2,2.65*1.485193));
            fTmin:=18.72*power((tjan+40)*1.31,6.3)*exp(1.31*(tjan+40)/2)/power(2,7.3*1271.424);
            This is the trial and error refit.}
            fTmax:=-0.66+17.5*POWER(tjul*0.2,5.3/2-1)*EXP(-tjul*0.2/2)/POWER(2,(5.3/2)*1.485193);
            fTmin:=-0.5+18.72*POWER((tjan+40)*0.35,(14.6/2-1))*EXP(-0.35*(tjan+40)/2)/(POWER(2,(14.6/2))*1271.424)
          end;
          gPsummer:=1.04-10.29/power(Psummer,0.63);  {scaled precip summer season}
          gPwinter:=0.92-69.21/power(Pwinter,1.34);  {scaled precip winter season}
          NPPbaseline:=0.96*(SQRT(fTmax*fTmin*gPsummer*gPwinter)*7)-0.2; {units: is Mg C ha-1 year−1 }
          PlantC:=Liethfactor*NPPbaseline*(-0.67*(350/ppmCO2)+1.67);
        end
        else
        begin
          val(cells[kolPlantC,i],PlantC,code);
        end;
        if i<>0 then system.write(ftemp,PlantC:6:2,' ')
                else system.write(ftemp,cells[kolPlantC,i],' ');
      end
      else system.write(ftemp,'-1;');
      if kolManureC<>-1 then system.write(ftemp,cells[kolManureC,i],' ') else system.write(ftemp,'-1;');
      if kolVegetation<>-1 then system.write(ftemp,cells[kolVegetation,i],' ') else system.write(ftemp,'-1;');
      if kolComments<>-1 then system.write(ftemp,cells[kolComments,i],' ') else system.write(ftemp,'-1;');
      system.writeln(ftemp);
    end;
    closefile(ftemp);
  end;
  untClimate.frmClimate.LeesData('tempClimate.txt');
  Close;
end;

end.
