unit untImportFertilizerData;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$MODE Delphi}

{$DEBUGINFO ON} 
{}
interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, ExtCtrls, Buttons, Menus, ComCtrls, StrUtils;

const
  mistxt='missing';

type
  TfrmImportFertilizerData = class(TForm)
    dlgOPENBUISDATA: TOpenDialog;
    strgrdSTAMBUIS: TStringGrid;
    chckbxMultiple: TCheckBox;
    rdgrpSEP: TRadioGroup;
    btbttnOKLEES: TBitBtn;
    lblstrgrdSTAMBUIS: TLabel;
    stsbrSTAMBUIS: TStatusBar;
    ppmnGRID: TPopupMenu;
    Toevoegenrecordmetbuisgegevens1: TMenuItem;
    grpbxKOLOM: TGroupBox;
    Verwijderenrecordmetbuisgegevens1: TMenuItem;
    cmbbxCa: TComboBox;
    cmbbxNa: TComboBox;
    cmbbxCl: TComboBox;
    cmbbxSO4: TComboBox;
    cmbbxK: TComboBox;
    cmbbxMg: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    btbttnEXITSTAMBUIS: TBitBtn;
    cmbbxHCO3: TComboBox;
    Label7: TLabel;
    Label8: TLabel;
    cmbbxCO3: TComboBox;
    Label9: TLabel;
    cmbbxYearBP: TComboBox;
    btbttnNewData: TBitBtn;
    btbttnEditOldData: TBitBtn;
    btbttnEXITNOCHANGE: TBitBtn;
    Label17: TLabel;
    lblUnits: TLabel;
    procedure btbttnOKLEESClick(Sender: TObject);
    procedure strgrdSTAMBUISMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure btbttnOKLEESMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure rdgrpSEPClick(Sender: TObject);
    procedure Toevoegenrecordmetbuisgegevens1Click(Sender: TObject);
    procedure Omzettenvanelkekommanaardecimalepunt1Click(Sender: TObject);
    procedure Bewareninbestand1Click(Sender: TObject);
    procedure Verwijderenrecordmetbuisgegevens1Click(Sender: TObject);
    procedure grpbxKOLOMMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure chckbxMultipleMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure Toevoegenkolommetmetingen1Click(Sender: TObject);
    procedure Verwijderenkolomonderdecursor1Click(Sender: TObject);
    procedure strgrdSTAMBUISKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btbttnEXITSTAMBUISClick(Sender: TObject);
    procedure btbttnNewDataClick(Sender: TObject);
    procedure btbttnEditOldDataClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure btbttnEXITNOCHANGEClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure bepaalsep(FileName:string);
    procedure vulgrid(FileName:string);
    Procedure leesveld(nr:integer; regel:string;
                       var woord:string);
    procedure insertrow(irow:integer);
    procedure insertcol(icol:integer);
    procedure vulboxen;
    procedure proceed;
    procedure backtoinit;
    procedure sortstrgrd;
  end;

var
  frmImportFertilizerData: TfrmImportFertilizerData;
  calling:(bestand,handinvoer);
  sep:char;
  FileName:string;
  startlees,rowins,colins,rowdel,coldel,NoInput:boolean;
  nosep,kopregels,datanr,
  kolP,kolPE,koljanT,kolJulT,kolPlantC,kolManureC,kolVegetation,kolComments,kolYearBP:integer;
  potsep:array[1..20] of char;
  potsepcount:array[1..20] of integer;

implementation

{$R *.lfm}
uses untGUI;

procedure TfrmImportFertilizerData.bepaalsep(FileName:string);
var ftemp:textfile; regel,si:string;
    i,j:integer;
    toegelaten:set of char;
begin
  {init}
  for i:=1 to 20 do begin potsep[i]:='x'; potsepcount[i]:=0; end;
  toegelaten:=['1','2','3','4','5','6','7','8','9','0','.',
               'a','b','c','d','e','f','g','h','i','j','k',
               'l','m','n','o','p','q','r','s','t','u','v',
               'w','x','y','z','''','"',
               'A','B','C','D','E','F','G','H','I','J','K',
               'L','M','N','O','P','Q','R','S','T','U','V',
               'W','X','Y','Z'];
  regel:='';
  {inlezen eerste regel uit bestand}
  system.assign(ftemp,FileName); system.reset(ftemp);
  system.readln(ftemp,regel);
  closefile(ftemp);
  {uit ingelezen regel <regel> alle niet-scheidingstekens verwijderen.
   alle niet-scheidingstekens zitten in de set <toegelaten>}
  i:=1;
  repeat
    if regel[i] in toegelaten
    then delete(regel,i,1)
    else i:=i+1;
  until i>length(regel);
  {nu unieke resterende tekens tellen en verwijderen uit <regel>}
  nosep:=0;
  while length(regel)>0 do
  begin
    nosep:=nosep+1;
    potsep[nosep]:=regel[1];
    for j:=1 to length(regel) do
    if regel[j]=regel[1] then potsepcount[nosep]:=potsepcount[nosep]+1;
    {verwijderen scheidingsteken <potsep[nosep]> uit <regel>}
    j:=1;
    repeat
      if regel[j]=potsep[nosep]
      then delete(regel,j,1)
      else j:=j+1;
    until j>length(regel);
  end;
  {potentiële scheidingstekens met max. frequentie zoeken}
  j:=1; sep:=potsep[j]; {rdgrpSEP.ItemIndex:=j-1;}
  {alle gevonden scheidingstekens in radiogroup zetten;
   meest frequent gevonden scheidingsteken "aanzetten".}
  rdgrpSEP.Height:=16+nosep*20;
  for i:=1 to nosep do
  begin
    si:=potsep[i];
    if ord(potsep[i])=32 then si:='space';
    if ord(potsep[i])=9 then si:='tab-sign';
    rdgrpSEP.Items.Add(si);
    if potsepcount[i]>potsepcount[j] then
    begin j:=i; sep:=potsep[i]; end;
  end;
  rdgrpSEP.ItemIndex:=j-1;
end;

procedure TfrmImportFertilizerData.vulgrid(FileName:string);
var f:textfile; regel,delstr,woord:string;
    i,j,nocols,norows:integer;
begin
  system.assign(f,FileName); system.reset(f);
  {geselecteerd scheidingsteken toepasbaar maken}
  sep:=potsep[rdgrpSEP.ItemIndex+1];
  {aantal kolommen bepalen}
  system.Readln(f,regel);
  {indien ingesteld: verwijder dubbele scheidingstekens}
  if chckbxMultiple.checked then
  begin
    delstr:=sep+sep;
    while pos(delstr,regel)>0 do delete(regel,pos(delstr,regel),1);
  end;
  {remove trailing separators}
  while regel[1]=sep do delete(regel,1,1);
  {now the text line starts with data}
  i:=1; woord:='';
  repeat
    leesveld(i,regel,woord);
    i:=i+1;
  until woord='';
  nocols:=i-2;
  {aantal regels bepalen}
  i:=1;
  repeat
    readln(f,regel); i:=i+1;
  until (eof(f));
  norows:=i;
  {nu inlezen}
  strgrdSTAMBUIS.RowCount:=norows;
  strgrdSTAMBUIS.ColCount:=nocols;
  system.reset(f);
  for i:=0 to norows-1 do
  begin
    system.readln(f,regel);
    {indien ingesteld: verwijder dubbele scheidingstekens}
    if chckbxMultiple.checked then
    begin
      delstr:=sep+sep;
      while pos(delstr,regel)>0 do delete(regel,pos(delstr,regel),1);
    end;
    {remove trailing separators}
    while regel[1]=sep do delete(regel,1,1);
    {now the text line starts with data}
    for j:=0 to nocols-1 do
    begin
      leesveld(j+1,regel,woord);
      strgrdSTAMBUIS.Cells[j,i]:=woord;
    end;
  end;
  strgrdSTAMBUIS.visible:=true;
  strgrdSTAMBUIS.enabled:=true;
  strgrdSTAMBUIS.refresh;
  closefile(f);
end;

procedure TfrmImportFertilizerData.btbttnOKLEESClick(Sender: TObject);
var txt:string;
begin
if not(NoInput) then
begin
  StartLees:=true;
  txt:=dlgOPENBUISDATA.Filename;
  if length(txt)>60 then txt:=copy(txt,1,3)+'...'+rightstr(txt,54);
  lblstrgrdSTAMBUIS.Caption:='Read data from '+txt;
  lblstrgrdSTAMBUIS.visible:=true;
  rdgrpSEP.VISIBLE:=FALSE;
  chckbxMULTIPLE.visible:=FALSE;
  btbttnOKLEES.visible:=false;
  vulgrid(dlgOPENBUISDATA.FileName);
  grpbxKOLOM.Visible:=true;
  vulboxen;
  btbttnEXITSTAMBUIS.enabled:=true;
end
else close;
end;

Procedure TfrmImportFertilizerData.leesveld(nr:integer; regel:string;
                            var woord:string);
var telveld:integer;
{krijgt een regel uit een <sep>-delimited tekst binnen
 en selecteert het woord <woord> voor de <nr>de separator <sep>.
 indien deze <sep> er niet is is <woord> een lege string}
begin
  for telveld:=1 to nr do
  begin
    if pos(sep,regel)>0
    then begin
           woord:=copy(regel,1,pos(sep,regel)-1);
           delete(regel,1,pos(sep,regel));
         end
    else if length(regel)>0
         then begin
                woord:=regel;
                regel:='';
              end
         else woord:='';
    if woord=' ' then woord:='';
  end;
end;

procedure TfrmImportFertilizerData.strgrdSTAMBUISMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   stsbrSTAMBUIS.SimpleText:=strgrdSTAMBUIS.Hint;
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportFertilizerData.btbttnOKLEESMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   stsbrSTAMBUIS.SimpleText:=btbttnOKLEES.Hint;
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportFertilizerData.FormMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
   stsbrSTAMBUIS.SimpleText:='';
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportFertilizerData.rdgrpSEPClick(Sender: TObject);
begin
   stsbrSTAMBUIS.SimpleText:=rdgrpSEP.Hint;
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportFertilizerData.insertrow(irow:integer);
var i,j:integer;
begin
  with strgrdSTAMBUIS do
  begin
    rowcount:=rowcount+1;
    for i:=rowcount-2 downto irow do
    begin
      for j:=0 to colcount-1 do
      cells[j,i+1]:=cells[j,i];
    end;
    cells[0,irow]:='extrayearBP';
    for j:=1 to colcount-1 do
      cells[j,irow]:=mistxt;
    if rowcount>1 then fixedrows:=kopregels;
    refresh;
  end;
end;

procedure TfrmImportFertilizerData.Toevoegenrecordmetbuisgegevens1Click(
  Sender: TObject);
begin
  insertrow(strgrdSTAMBUIS.row);
  RowIns:=true;
end;

procedure TfrmImportFertilizerData.Omzettenvanelkekommanaardecimalepunt1Click(
  Sender: TObject);
var i,j,k:integer;
   astr:string;
begin
  with strgrdSTAMBUIS do
  begin
    for i:=0 to colcount-1 do
    for j:=0 to rowcount-1 do
    while pos(',',cells[i,j])>0 do
    begin
      astr:=cells[i,j];
      k:=pos(',',astr); if k>0 then astr[k]:='.';
      cells[i,j]:=astr;
    end;
  end;
end;

procedure TfrmImportFertilizerData.Bewareninbestand1Click(Sender: TObject);
var ftemp:textfile; i,j:integer;
begin
  system.assign(ftemp,'Fertilization.txt');
  system.rewrite(ftemp);
  sortstrgrd;
  with strgrdSTAMBUIS do
  begin
    for i:=0 to rowcount-1 do
    begin
      for j:=0 to colcount-1 do system.write(ftemp,cells[j,i],sep);
      system.writeln(ftemp);
    end;
  end;
  closefile(ftemp);
  showmessage('Current data set written to <Fertilization.txt>');
end;

procedure TfrmImportFertilizerData.Verwijderenrecordmetbuisgegevens1Click(
  Sender: TObject);
var i,j:integer;
begin
  with strgrdSTAMBUIS do
  begin
    for i:=row to rowcount-2 do for j:=0 to colcount-1 do
      cells[j,i]:=cells[j,i+1];
    rowcount:=rowcount-1;
    refresh;
  end;
  RowDel:=true;
end;

procedure TfrmImportFertilizerData.grpbxKOLOMMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   stsbrSTAMBUIS.SimpleText:=grpbxKOLOM.Hint;
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportFertilizerData.chckbxMultipleMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   stsbrSTAMBUIS.SimpleText:=chckbxMultiple.Hint;
   stsbrSTAMBUIS.Refresh;
end;

procedure TfrmImportFertilizerData.insertcol(icol:integer);
var i,j:integer; datstr:string;
begin
  with strgrdSTAMBUIS do
  begin
    colcount:=colcount+1;
    for i:=colcount-2 downto icol do
    begin
      for j:=0 to rowcount-1 do
      cells[i+1,j]:=cells[i,j];
    end;
    datanr:=datanr+1;
    str(datanr,datstr);
    cells[icol,0]:='data'+datstr;
    for j:=1 to rowcount-1 do
      cells[icol,j]:=mistxt;
    refresh;
  end;
end;

procedure TfrmImportFertilizerData.Toevoegenkolommetmetingen1Click(Sender: TObject);
var ii:integer;
begin
  ii:=strgrdSTAMBUIS.ColCount;
  insertcol(ii);
  vulboxen;
  ColIns:=true;
end;

procedure TfrmImportFertilizerData.Verwijderenkolomonderdecursor1Click(
  Sender: TObject);
var i,j:integer;
begin
  with strgrdSTAMBUIS do
  begin
    for i:=col to colcount-2 do for j:=0 to rowcount-1 do
      cells[i,j]:=cells[i+1,j];
    colcount:=colcount-1;
    refresh;
  end;
  vulboxen;
  ColDel:=true;
end;

procedure TfrmImportFertilizerData.strgrdSTAMBUISKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  vulboxen;
end;

procedure TfrmImportFertilizerData.vulboxen;
var i:integer; 
begin
    cmbbxCa.Clear;       cmbbxMg.Clear;      cmbbxNa.Clear;
    cmbbxK.Clear; cmbbxCl.Clear;      cmbbxSO4.Clear;
    cmbbxHCO3.Clear;  cmbbxCO3.Clear; cmbbxYearBP.Clear;
  {vullen alle comboboxen en listbox TITELS met beschikbare kolomtitels}
  for i:=0 to strgrdSTAMBUIS.ColCount-1 do
  begin
    cmbbxCa.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxMg.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxK.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxNa.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxCl.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxSO4.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxHCO3.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxCO3.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
    cmbbxYearBP.Items.Add(strgrdSTAMBUIS.Cells[i,0]);
  end;

  {Vul in het Text veld de titel in die het meest voor de hand ligt}
  for i:=strgrdSTAMBUIS.ColCount-1 downto 0 do
  begin
    if (pos('Ca',strgrdSTAMBUIS.Cells[i,0])>0)
    then cmbbxCa.Text:=strgrdSTAMBUIS.Cells[i,0];
    if (pos('Mg',strgrdSTAMBUIS.Cells[i,0])>0)
    then cmbbxMg.Text:=strgrdSTAMBUIS.Cells[i,0];
    if pos('Na',strgrdSTAMBUIS.Cells[i,0])>0
    then cmbbxNa.Text:=strgrdSTAMBUIS.Cells[i,0];
    if pos('K',strgrdSTAMBUIS.Cells[i,0])>0
    then cmbbxK.Text:=strgrdSTAMBUIS.Cells[i,0];
    if pos('Cl',strgrdSTAMBUIS.Cells[i,0])>0
    then cmbbxCl.Text:=strgrdSTAMBUIS.Cells[i,0];
    if pos('SO4',strgrdSTAMBUIS.Cells[i,0])>0
    then cmbbxSO4.Text:=strgrdSTAMBUIS.Cells[i,0];
    if (pos('HCO3',strgrdSTAMBUIS.Cells[i,0])>0)
    then cmbbxHCO3.Text:=strgrdSTAMBUIS.Cells[i,0];
    if (pos('CO3',strgrdSTAMBUIS.Cells[i,0])=1)
    then cmbbxCO3.Text:=strgrdSTAMBUIS.Cells[i,0];
    if (pos('Year',strgrdSTAMBUIS.Cells[i,0])>0)or
       (pos('year',strgrdSTAMBUIS.Cells[i,0])>0)
    then cmbbxYearBP.Text:=strgrdSTAMBUIS.Cells[i,0];
  end;
end;

procedure TfrmImportFertilizerData.btbttnEXITSTAMBUISClick(Sender: TObject);
var ftemp:textfile;
    i,j:integer;
begin
  {initialiseren}
  kolP:=-1; kolPE:=-1; koljanT:=-1; kolJulT:=-1; kolPlantC:=-1; kolManureC:=-1;
  kolVegetation:=-1; kolComments:=-1;
  {wegschrijven naar bestand <temp***.txt> van de kolommen en
   titels van de relevante variabelen}
  sortstrgrd;
  system.assign(ftemp,'Fertilization.txt'); system.rewrite(ftemp);
  with strgrdSTAMBUIS do
  begin
    for j:=0 to ColCount-1 do
    begin
      if cmbbxCa.Text=Cells[j,0]
      then kolP:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxMg.Text=Cells[j,0]
      then kolPE:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxNa.Text=Cells[j,0]
      then kolJanT:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxK.Text=Cells[j,0]
      then kolJulT:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxCl.Text=Cells[j,0]
      then kolPlantC:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxSO4.Text=Cells[j,0]
      then kolManureC:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxHCO3.Text=Cells[j,0]
      then kolVegetation:=j;
    end;
    for j:=0 to ColCount-1 do
    begin
      if cmbbxCO3.Text=Cells[j,0]
      then kolComments:=j;
    end;     
    for j:=0 to ColCount-1 do
    begin
      if cmbbxYearBP.Text=Cells[j,0]
      then kolYearBP:=j;
    end;
    for i:=0 to rowcount-1 do
    begin
      if kolYearBP<>-1 then system.write(ftemp,cells[kolYearBP,i],' ') else system.write(ftemp,'-1;');
      if kolP<>-1 then system.write(ftemp,cells[kolP,i],' ') else system.write(ftemp,'-1;');
      if kolPE<>-1 then system.write(ftemp,cells[kolPE,i],' ') else system.write(ftemp,'-1;');
      if kolJanT<>-1 then system.write(ftemp,cells[kolJanT,i],' ') else system.write(ftemp,'-1;');
      if kolJulT<>-1 then system.write(ftemp,cells[kolJulT,i],' ') else system.write(ftemp,'-1;');
      if kolPlantC<>-1 then system.write(ftemp,cells[kolPlantC,i],' ') else system.write(ftemp,'-1;');
      if kolManureC<>-1 then system.write(ftemp,cells[kolManureC,i],' ') else system.write(ftemp,'-1;');
      if kolVegetation<>-1 then system.write(ftemp,cells[kolVegetation,i],' ') else system.write(ftemp,'-1;');
      if kolComments<>-1 then system.write(ftemp,cells[kolComments,i],' ') else system.write(ftemp,'-1;');
      system.writeln(ftemp);
    end;
    closefile(ftemp);
  end;
  {untClimate.frmClimate.LeesData('tempClimate.txt');}
  showmessage('Current data set written to <Fertilization.txt>');
  backtoinit;
  Close;
end;

procedure TfrmImportFertilizerData.Proceed;
var i,j:integer;
begin
  {altijd uitvoeren:}
  kopregels:=1; datanr:=0;
  NoInput:=false;
  strgrdSTAMBUIS.visible:=false;
  lblstrgrdSTAMBUIS.visible:=false;
  strgrdSTAMBUIS.Enabled:=false;
  strgrdSTAMBUIS.FixedRows:=kopregels;
  STARTLEES:=false;
  rdgrpSEP.ItemIndex:=-1;
  rdgrpSEP.Items.Clear;
  RowIns:=false; RowDel:=false;
  ColIns:=false; ColDel:=false;
  btbttnEXITSTAMBUIS.visible:=true;
  btbttnExitnoChange.Visible:=true;
  grpbxKOLOM.Visible:=false;
  btbttnOKLEES.Caption:='OK, start reading';
  btbttnOKLEES.Hint:='Press this button if the right delimiter sign is set'+
                     ' and the above option is (or not) selected';

  {afhankelijk van type aanroep:}
  if calling=bestand then
  begin
    btbttnEXITSTAMBUIS.enabled:=false;
    {form wordt aangeroepen met als doel een bestand
    in te lezen en vervolgens de tabel op te maken}
    {1. Een dialoogvenster wordt geopend waarmee
        een bestand met stambuisgegevens
        wordt aangekoppeld.
        Eisen aan het bestand:
        + ASCII
        + gestructureerd, dwz elke regel (record) bevat alle gegevens
          van 1 buis, en er is een constant aantal kolommen (fields)
          gescheiden door spaties, tabs, puntkomma's}
    if dlgOPENBUISDATA.execute then
    begin
    {2. Het aangekoppelde bestand wordt ingelezen en opgemaakt, dwz:
      2init:}
      grpbxKOLOM.visible:=false;
      rdgrpSEP.VISIBLE:=true;
      chckbxMULTIPLE.visible:=true;
      btbttnOKLEES.visible:=true;
      {2a. er wordt afgeleid wat het scheidingsteken tussen de kolommen is}
      bepaalsep(dlgOPENBUISDATA.FileName);
      btbttnOKlees.Visible:=true;
      {overige activiteiten uitgevoerd na indrukken knop <btbttnOKLEES>}
    end
    else
    begin
      NoInput:=true;
      rdgrpSEP.Visible:=false;
      chckbxMULTIPLE.visible:=false;
      btbttnEXITSTAMBUIS.visible:=false;
      btbttnOKLEES.visible:=true;
      btbttnOKLEES.Caption:='No input; abort';
      btbttnOKLEES.Hint:='No input file; '+
                         'press button to start again';
    end;
  end
  else
  if calling=handinvoer then
  begin
    rdgrpSEP.Visible:=false;
    chckbxMULTIPLE.visible:=false;
    btbttnOKLEES.visible:=false;
    btbttnEXITSTAMBUIS.enabled:=true;
    {form wordt aangeroepen met als doel live ingeklopte gegevens
    in te lezen en vervolgens de tabel op te maken}
    {1. Opmaak
       + er wordt gevraagd in hoeveel/welke kolommen
         de meetgegevens staan (format: kolom voor datum1 met meting1;
         kolom voor datum2 met meting2 etc)
       + er wordt gevraagd in welke kolom de GHG, GVG, GLG etc staan
       + er wordt gevraagd in welke kolom
         betrouwbaarheid GHG etc staat (sdGHG) en
         of deze getallen nog gekwadrateerd moeten worden
       + er wordt gevraagd hoeveel buizen (records) er zijn
       + code voor missing data ingeven}
    SEP:=';';
    strgrdSTAMBUIS.RowCount:=4;
    strgrdSTAMBUIS.ColCount:=9;
    strgrdSTAMBUIS.visible:=true;
    strgrdSTAMBUIS.enabled:=true;
    strgrdSTAMBUIS.refresh;
    lblstrgrdSTAMBUIS.Caption:='manual input data';
    lblstrgrdSTAMBUIS.Visible:=true;
    lblstrgrdSTAMBUIS.Refresh;

    with strgrdSTAMBUIS do
    begin
      Cells[0,0]:='YearBP';     Cells[1,0]:='Ca';
      Cells[2,0]:='Mg';      Cells[3,0]:='Na';
      Cells[4,0]:='K';      Cells[5,0]:='Cl';
      Cells[6,0]:='SO4';      Cells[7,0]:='HCO3';
      Cells[8,0]:='CO3';
      for i:=1 to RowCount-1 do
      for j:=0 to ColCount-1 do
        Cells[j,i]:=mistxt;
    end;
    grpbxKOLOM.visible:=true;
    vulboxen;
    {2. Invoer/editen/wegschrijven
       + getallen invullen
       + evt alle komma's naar decimale punten omzetten
       + mogelijkheid om handmatig een record (regel) met nieuwe buisgegevens
         toe te voegen
       + mogelijkheid om file te bewaren
       + scherm afsluiten wanneer gereed  }
  end;
  lblUnits.Visible:=true;
end;

procedure TfrmImportFertilizerData.btbttnNewDataClick(Sender: TObject);
begin
   calling:=handinvoer;
   btbttnEditOldData.Visible:=false;  btbttnNewData.Visible:=false;
   grpbxKolom.Visible:=false;
   Proceed;
end;

procedure TfrmImportFertilizerData.btbttnEditOldDataClick(Sender: TObject);
begin
  calling:=bestand; lblUnits.Visible:=false;
  btbttnEditOldData.Visible:=false;  btbttnNewData.Visible:=false;
  Proceed;
end;

procedure TfrmImportFertilizerData.backtoinit;
begin
  btbttnEditOldData.Visible:=true;
  btbttnNewData.Visible:=true;
  grpbxKolom.Visible:=false;
  rdgrpSEP.Visible:=false;
  chckbxMultiple.Visible:=false;
  btbttnOKLees.Visible:=false;
  lblstrgrdstambuis.Visible:=false;
  strgrdStambuis.Visible:=false;
  btbttnexitstambuis.Visible:=false;
  btbttnexitnochange.Visible:=false;
end;

procedure TfrmImportFertilizerData.FormActivate(Sender: TObject);
begin
  dlgOPENBUISDATA.InitialDir:=startdir;
  backtoinit;
end;

procedure TfrmImportFertilizerData.btbttnEXITNOCHANGEClick(Sender: TObject);
begin
  backtoinit;
  Close;
end;

procedure TfrmImportFertilizerData.sortstrgrd;
var ready:boolean; year1,year2,code,i,j:integer;
    txt:array[0..8] of string;
begin
  repeat
      ready:=true;
      for i:=1 to strgrdstambuis.RowCount-2 do
      begin
        val(strgrdstambuis.Cells[kolYearBP,i],year1,code);
        val(strgrdstambuis.Cells[kolYearBP,i+1],year2,code);
        if year1<year2 then  {swap because order is wrong}
        begin
          for j:=0 to 8 do
          begin
            txt[j]:=strgrdstambuis.Cells[j,i];
            strgrdstambuis.Cells[j,i]:=strgrdstambuis.Cells[j,i+1];
            strgrdstambuis.Cells[j,i+1]:=txt[j];
          end;
          ready:=false;
        end;
      end;
  until ready

end;

end.
