unit untMemDump;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$MODE Delphi}

interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Buttons, Spin, Grids, FileUtil, lazfileutils;

type
  TfrmMemDump = class(TForm)
    rdgrpMem: TRadioGroup;
    btbttnExit: TBitBtn;
    GroupBox2: TGroupBox;
    grid: TStringGrid;
    mmNames: TMemo;
    btbttnExitDelDump: TBitBtn;
    procedure btbttnExitClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure rdgrpMemClick(Sender: TObject);
    procedure btbttnExitDelDumpClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure displayit;
  end;

var
  frmMemDump: TfrmMemDump;

implementation

{$R *.lfm}
uses untGUI;


procedure TfrmMemDump.btbttnExitClick(Sender: TObject);
begin
   frmMemDump.close;
end;

procedure TfrmMemDump.FormActivate(Sender: TObject);
begin

  grid.ColWidths[0]:=240;
  displayit;
end;

procedure TfrmMemDump.rdgrpMemClick(Sender: TObject);
begin
  displayit;
end;

procedure TfrmMemDump.displayit;
var kol,rij,i:integer; txt:string;
begin
  untGUI.frmMAIN.readdump;
  grid.colcount:=DataInit.NoLayers+1;  grid.RowCount:=173+1;
  for kol:=1 to grid.ColCount-1 do begin str(kol,txt); grid.Cells[kol,0]:=txt; end;
  for rij:=1 to grid.RowCount-1 do grid.Cells[0,rij]:=mmNames.Lines[rij-1];
  case rdgrpMem.ItemIndex of 0: ondisplay:=DataInit;
                             1: ondisplay:=DataYearIn;
                             2: ondisplay:=DataYearOut;
  end;
  grid.Refresh;
  with ondisplay do
  for i:=1 to Nolayers do
  begin rij:=1;
    str(CLAY[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SILT[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(parden[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(porosity[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ClayDispInd[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(OC[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(AW[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(W[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(AH[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(RDF[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ATEM[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(TEM[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(TEMPGEM[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(AAEV[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ABCAM[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(RHO[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(CMATCH[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(HMATCH[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(LAMBDA[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(APH[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(PH[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(APCO2[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(CALCITEA[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(GYPA[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ACEC[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(alkalinity[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(BS[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(EC[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SAR[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ESP[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ET[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(FLUX[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(DOC[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[1][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[2][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[3][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[4][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[5][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[6][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[7][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(HCO3[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[8][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(CO3[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[9][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[10][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[11][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[12][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[13][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[14][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[15][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[16][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[17][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[18][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[19][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[20][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[21][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[22][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[23][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(AXSE[1][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(AXSE[2][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(AXSE[3][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(AXSE[4][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[10][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(SEA[12][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(RothC.DPMpool[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(RothC.RPMpool[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(RothC.BIOpool[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(RothC.HUMpool[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(RothC.IOMpool[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(RothC.CO2pool[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(RothC.accTSMD[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(RothC.pCO2[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[1][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[1][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[1][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[1][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[1][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[1][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[2][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[2][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[2][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[2][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[2][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[2][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[3][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[3][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[3][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[3][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[3][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[3][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[4][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[4][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[4][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[4][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[4][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[4][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[5][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[5][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[5][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[5][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[5][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[5][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[6][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[6][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[6][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[6][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[6][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[6][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[7][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[7][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[7][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[7][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[7][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[7][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[8][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[8][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[8][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[8][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[8][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[8][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[9][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[9][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[9][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[9][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[9][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[9][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[10][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[10][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[10][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[10][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[10][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[10][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[11][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[11][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[11][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[11][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[11][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[11][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[12][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[12][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[12][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[12][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[12][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[12][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[13][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[13][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[13][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[13][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[13][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[13][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[14][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[14][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[14][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[14][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[14][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[14][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[15][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[15][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[15][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[15][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[15][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1; 
    str(ChemCO2pool[15][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemDPMpool[16][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemRPMpool[16][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemBIOpool[16][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemHUMpool[16][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemIOMpool[16][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(ChemCO2pool[16][i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(WE.Gibbsitepool[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(WE.WE_Capool[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(WE.WE_Mgpool[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(WE.WE_Kpool[i]:10:7,txt); grid.cells[i,rij]:=txt; rij:=rij+1;
    str(WE.WE_Napool[i]:10:7,txt); grid.cells[i,rij]:=txt;
    grid.Refresh;
  end;
end;

procedure TfrmMemDump.btbttnExitDelDumpClick(Sender: TObject);
begin
  if FileExistsUTF8('dump.txt') { *Converted from FileExists*  } then
  begin
    DeleteFileUTF8('dump.txt'); { *Converted from DeleteFile*  }
    untGUI.frmMAIN.Showmemorydump1.Visible:=false;
  end;
  frmMemDump.close;
end;

end.
