unit untSoftConstants;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$MODE Delphi}

{$DEBUGINFO ON} 
interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, Buttons, Grids, untProcesses, FileUtil,math, lazfileutils, untSoilGenConst;

type

  { TfrmSoftConstants }

  TfrmSoftConstants = class(TForm)
    edtAgriculture_relAl: TEdit;
    edtAgriculture_relCa: TEdit;
    edtAgriculture_relK: TEdit;
    edtAgriculture_relMg: TEdit;
    edtAgriculture_relNa: TEdit;
    edtBIO2HUMratio: TEdit;
    edtclay_ectobd: TEdit;
    edtclay_ectothick: TEdit;
    edtclay_fref: TEdit;
    edtClay_hthetamacro: TEdit;
    edtclay_Kd: TEdit;
    edtClay_Kr: TEdit;
    edtclay_n: TEdit;
    edtclay_vref: TEdit;
    edtConiferouswood_relAl: TEdit;
    edtConiferouswood_relCa: TEdit;
    edtConiferouswood_relK: TEdit;
    edtConiferouswood_relMg: TEdit;
    edtConiferouswood_relNa: TEdit;
    edtDeciduouswood_relAl: TEdit;
    edtDeciduouswood_relCa: TEdit;
    edtDeciduouswood_relK: TEdit;
    edtDeciduouswood_relMg: TEdit;
    edtDeciduouswood_relNa: TEdit;
    edtDPM2RPM_agriculture: TEdit;
    edtDPM2RPM_coniferouswood: TEdit;
    edtDPM2RPM_deciduouswood: TEdit;
    edtDPM2RPM_grassscrub: TEdit;
    edtfacscaleCO2BIOHUM: TEdit;
    edtfrEctolitter_agriculture: TEdit;
    edtfrEctoLitter_coniferouswood: TEdit;
    edtfrEctolitter_deciduouswood: TEdit;
    edtfrEctoLitter_grassscrub: TEdit;
    edtfrendoLitter_agriculture: TEdit;
    edtfrEndoLitter_coniferouswood: TEdit;
    edtfrEndoLitter_deciduouswood: TEdit;
    edtfrEndoLitter_Grassscrub: TEdit;
    edtfrPINT_agriculture: TEdit;
    edtfrPINT_coniferouswood: TEdit;
    edtfrPINT_deciduouswood: TEdit;
    edtfrPINT_grassscrub: TEdit;
    edtGrassscrub_relAl: TEdit;
    edtGrassscrub_relCa: TEdit;
    edtGrassscrub_relK: TEdit;
    edtGrassscrub_relMg: TEdit;
    edtGrassscrub_relNa: TEdit;
    edtk_BIO: TEdit;
    edtk_DPM: TEdit;
    edtk_HUM: TEdit;
    edtk_RPM: TEdit;
    edtPW_B: TEdit;
    edtStoneSplit: TEdit;
    edtPW_Psmax: TEdit;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    GroupBox4: TGroupBox;
    GroupBox5: TGroupBox;
    GroupBox6: TGroupBox;
    GroupBox7: TGroupBox;
    grpbxCationUptakeTargetValues: TGroupBox;
    grpbxCcycleParameters: TGroupBox;
    grpbxPhysicoChemicalParameters: TGroupBox;
    grpbxMineralDissolutionParameters: TGroupBox;
    Label1: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label2: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    Label28: TLabel;
    Label29: TLabel;
    Label3: TLabel;
    Label30: TLabel;
    Label31: TLabel;
    Label32: TLabel;
    Label33: TLabel;
    Label34: TLabel;
    Label35: TLabel;
    Label36: TLabel;
    Label37: TLabel;
    Label38: TLabel;
    Label39: TLabel;
    Label4: TLabel;
    Label40: TLabel;
    Label41: TLabel;
    Label42: TLabel;
    Label43: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    strgrdMinDisPar: TStringGrid;
    tbcntrlGroups: TTabControl;
    btbttnSoftConstantSave: TBitBtn;
    btbttnSoftConstantsCancel: TBitBtn;
    procedure FormActivate(Sender: TObject);
    procedure btbttnSoftConstantSaveClick(Sender: TObject);
    procedure btbttnSoftConstantsCancelClick(Sender: TObject);
    procedure tbcntrlGroupsChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmSoftConstants: TfrmSoftConstants;

implementation
{$R *.lfm}
uses untGUI;

procedure TfrmSoftConstants.FormActivate(Sender: TObject);
begin
  tbcntrlGroups.tabindex:=0;
  strgrdMinDisPar.Colcount:=nrparams+1;
  strgrdMinDisPar.RowCount:=nrMinerals+1;
  strgrdMinDisPar.ColWidths[0]:=90;
  strgrdMinDisPar.ColWidths[1]:=80;
  strgrdMinDisPar.ColWidths[2]:=70;
  strgrdMinDisPar.ColWidths[3]:=80;
  strgrdMinDisPar.ColWidths[4]:=80;
  strgrdMinDisPar.ColWidths[5]:=70;
  strgrdMinDisPar.ColWidths[6]:=80;
  strgrdMinDisPar.Cells[0,0]:='Mineral';
  strgrdMinDisPar.Cells[1,0]:='log10 kH25 (mol/m2/s)';
  strgrdMinDisPar.Cells[2,0]:='kEaH (KJ/mol/K)';
  strgrdMinDisPar.Cells[3,0]:='reaction order n';
  strgrdMinDisPar.Cells[4,0]:='log10 kOH25 (mol/m2/s)';
  strgrdMinDisPar.Cells[5,0]:='kEaOH (KJ/mol/K)';
  strgrdMinDisPar.Cells[6,0]:='reaction order m';
  {strgrdMinDisPar.Cells[7,0]:='log10 Keq';
  strgrdMinDisPar.Cells[8,0]:='deltaHr (KJ/mol/K)';
  strgrdMinDisPar.Cells[9,0]:='SurfaceArea (m2/g)';}
  strgrdMinDisPar.Cells[0,7]:='Anorthite';
  strgrdMinDisPar.Cells[0,1]:='Albite';
  strgrdMinDisPar.Cells[0,6]:='Chlorite';
  strgrdMinDisPar.Cells[0,2]:='K-Feldspar';
  strgrdMinDisPar.Cells[0,3]:='Muscovite';
  strgrdMinDisPar.Cells[0,13]:='Hornblende';
  strgrdMinDisPar.Cells[0,4]:='Biotite';
  strgrdMinDisPar.Cells[0,5]:='Quartz';
  strgrdMinDisPar.Cells[0,8]:='Forsterite';
  strgrdMinDisPar.Cells[0,9]:='Augite';
  strgrdMinDisPar.Cells[0,14]:='Fayalite';
  strgrdMinDisPar.Cells[0,10]:='Kaolinite';
  strgrdMinDisPar.Cells[0,11]:='Illite';
  strgrdMinDisPar.Cells[0,12]:='Montmorillonite';
  strgrdMinDisPar.Cells[0,15]:='Amorphite';
  strgrdMinDisPar.Cells[0,16]:='Otherite';
  {
  strgrdMinDisPar.Cells[0,17]:='Gibbsite';
  strgrdMinDisPar.Cells[0,18]:='Siderite';
  strgrdMinDisPar.Cells[0,19]:='Hematite';
  strgrdMinDisPar.Cells[0,20]:='Goethite';
  strgrdMinDisPar.Cells[0,21]:='Imogolite';
  strgrdMinDisPar.Cells[0,22]:='Gypsum';
  strgrdMinDisPar.Cells[0,23]:='Dolomite';
  strgrdMinDisPar.Cells[0,24]:='Calcite';
  strgrdMinDisPar.Cells[0,25]:='Phytoliths';
  }
  tbcntrlGroupsChange(Sender);
  frmSoftConstants.refresh;
end;

procedure TfrmSoftConstants.btbttnSoftConstantSaveClick(Sender: TObject);
var sb1,dum1,dum2:real; col,row,code:integer; foutje,scaled:boolean; fsc:textfile; txt:string;
begin
  foutje:=false;
  system.val(edtk_DPM.Text,untProcesses.k_DPM,code);
  if code<>0 then begin edtk_DPM.Text:='number!'; foutje:=true; end;
  system.val(edtk_RPM.text,untProcesses.k_RPM,code);
  if code<>0 then begin edtk_RPM.Text:='number!'; foutje:=true; end; 
  system.val(edtk_BIO.text,untProcesses.k_BIO,code);
  if code<>0 then begin edtk_BIO.Text:='number!'; foutje:=true; end; 
  system.val(edtk_HUM.text,untProcesses.k_HUM,code);
  if code<>0 then begin edtk_HUM.Text:='number!'; foutje:=true; end; 
  system.val(edtDPM2RPM_agriculture.text,untProcesses.DPM2RPM_agriculture,code);
  if code<>0 then begin edtDPM2RPM_agriculture.Text:='number!'; foutje:=true; end;
  system.val(edtfrPINT_agriculture.text,untProcesses.frPINT_agriculture,code);
  if code<>0 then begin edtfrPINT_agriculture.Text:='number!'; foutje:=true; end;
  system.val(edtDPM2RPM_grassscrub.text,untProcesses.DPM2RPM_grassscrub,code);
  if code<>0 then begin edtDPM2RPM_grassscrub.Text:='number!'; foutje:=true; end;
  system.val(edtfrPINT_grassscrub.text,untProcesses.frPINT_grassscrub,code);
  if code<>0 then begin edtfrPINT_grassscrub.Text:='number!'; foutje:=true; end;
  system.val(edtDPM2RPM_deciduouswood.text,untProcesses.DPM2RPM_deciduouswood,code);
  if code<>0 then begin edtDPM2RPM_deciduouswood.Text:='number!'; foutje:=true; end;
  system.val(edtfrPINT_deciduouswood.text,untProcesses.frPINT_deciduouswood,code);
  if code<>0 then begin edtfrPINT_deciduouswood.Text:='number!'; foutje:=true; end;
  system.val(edtDPM2RPM_coniferouswood.text,untProcesses.DPM2RPM_coniferouswood,code);
  if code<>0 then begin edtDPM2RPM_coniferouswood.Text:='number!'; foutje:=true; end; 
  system.val(edtfrPINT_coniferouswood.text,untProcesses.frPINT_coniferouswood,code);
  if code<>0 then begin edtfrPINT_coniferouswood.Text:='number!'; foutje:=true; end;
  system.val(edtfrEctoLitter_grassscrub.text,untProcesses.frEctoLitter_grassscrub,code);
  if code<>0 then begin edtfrEctoLitter_grassscrub.Text:='number!'; foutje:=true; end;
  system.val(edtfrEndoLitter_grassscrub.text,untProcesses.frEndoLitter_grassscrub,code);
  if code<>0 then begin edtfrEndoLitter_grassscrub.Text:='number!'; foutje:=true; end;
  system.val(edtfrEctoLitter_deciduouswood.text,untProcesses.frEctoLitter_deciduouswood,code);
  if code<>0 then begin edtk_DPM.Text:='number!'; foutje:=true; end;
  system.val(edtfrEndoLitter_deciduouswood.text,untProcesses.frEndoLitter_deciduouswood,code);
  if code<>0 then begin edtfrEndoLitter_deciduouswood.Text:='number!'; foutje:=true; end;
  system.val(edtfrEctoLitter_coniferouswood.text,untProcesses.frEctoLitter_coniferouswood,code);
  if code<>0 then begin edtfrEctoLitter_coniferouswood.Text:='number!'; foutje:=true; end;
  system.val(edtfrEndoLitter_coniferouswood.text,untProcesses.frEndoLitter_coniferouswood,code);
  if code<>0 then begin edtfrEndoLitter_coniferouswood.Text:='number!'; foutje:=true; end;
  system.val(edtfrEctoLitter_agriculture.text,untProcesses.frEctoLitter_agriculture,code);
  if code<>0 then begin edtfrEctoLitter_agriculture.Text:='number!'; foutje:=true; end;
  system.val(edtfrEndoLitter_agriculture.text,untProcesses.frEndoLitter_agriculture,code);
  if code<>0 then begin edtfrEndoLitter_agriculture.Text:='number!'; foutje:=true; end;
  system.val(edtfacscaleCO2BIOHUM.text,untProcesses.facscaleCO2BIOHUM,code);
  if code<>0 then begin edtfacscaleCO2BIOHUM.Text:='number!'; foutje:=true; end;
  system.val(edtBIO2HUMratio.text,untProcesses.BIO2HUMratio,code);
  if code<>0 then begin edtBIO2HUMratio.Text:='number!'; foutje:=true; end;
  system.val(edtclay_kd.text,untProcesses.clay_kd,code);
  if code<>0 then begin edtclay_kd.text:='number!'; foutje:=true; end;
  system.val(edtclay_kr.text,untProcesses.clay_kr,code);
  if code<>0 then begin edtclay_kr.text:='number!'; foutje:=true; end;  
  system.val(edtclay_hthetamacro.text,untProcesses.clay_hthetamacro,code);
  if code<>0 then begin edtclay_hthetamacro.text:='number!'; foutje:=true; end
             else begin
                    untGUI.MacroMicroMM:=untProcesses.clay_hthetamacro;
                  end;
  system.val(edtclay_n.text,untProcesses.CM_n,code);
  if code<>0 then begin edtclay_n.text:='number!'; foutje:=true; end;
  system.val(edtclay_fref.text,untProcesses.CM_fref,code);
  if code<>0 then begin edtclay_fref.text:='number!'; foutje:=true; end;
  system.val(edtclay_vref.text,untProcesses.CM_vref,code);
  if code<>0 then begin edtclay_vref.text:='number!'; foutje:=true; end;
  system.val(edtclay_ectobd.text,untProcesses.clay_ectobd,code);
  if code<>0 then begin edtclay_ectobd.text:='number!'; foutje:=true; end;
  system.val(edtclay_ectothick.text,untProcesses.clay_ectothick,code);
  if code<>0 then begin edtclay_ectothick.text:='number!'; foutje:=true; end;
  system.val(edtStoneSplit.Text,untProcesses.frStoneSplit,code);
  if code<>0 then begin edtStoneSplit.Text:='number!'; foutje:=true; end;
  system.val(edtPW_Psmax.text,untGUI.PW_Pmax,code);
  if code<>0 then begin edtPW_Psmax.text:='number!'; foutje:=true; end;
  system.val(edtPW_B.text,untGUI.PW_dTdtPmax,code);
  if code<>0 then begin edtPW_B.text:='number!'; foutje:=true; end;
  system.val(edtAgriculture_relCa.text,untProcesses.relCa_Agriculture,code);
  if code<>0 then begin edtAgriculture_relCa.text:='number!'; foutje:=true; end;
  system.val(edtAgriculture_relNa.text,untProcesses.relNa_Agriculture,code);
  if code<>0 then begin edtAgriculture_relNa.text:='number!'; foutje:=true; end;
  system.val(edtAgriculture_relK.text,untProcesses.relK_Agriculture,code);
  if code<>0 then begin edtAgriculture_relK.text:='number!'; foutje:=true; end;
  system.val(edtAgriculture_relMg.text,untProcesses.relMg_Agriculture,code);
  if code<>0 then begin edtAgriculture_relMg.text:='number!'; foutje:=true; end;
  system.val(edtAgriculture_relAl.text,untProcesses.relAl_Agriculture,code);
  if code<>0 then begin edtAgriculture_relAl.text:='number!'; foutje:=true; end;
  system.val(edtGrassscrub_relCa.text,untProcesses.relCa_Grassscrub,code);
  if code<>0 then begin edtGrassscrub_relCa.text:='number!'; foutje:=true; end;
  system.val(edtGrassscrub_relNa.text,untProcesses.relNa_Grassscrub,code);
  if code<>0 then begin edtGrassscrub_relNa.text:='number!'; foutje:=true; end;
  system.val(edtGrassscrub_relK.text,untProcesses.relK_Grassscrub,code);
  if code<>0 then begin edtGrassscrub_relK.text:='number!'; foutje:=true; end;
  system.val(edtGrassscrub_relMg.text,untProcesses.relMg_Grassscrub,code);
  if code<>0 then begin edtGrassscrub_relMg.text:='number!'; foutje:=true; end;
  system.val(edtGrassscrub_relAl.text,untProcesses.relAl_Grassscrub,code);
  if code<>0 then begin edtGrassscrub_relAl.text:='number!'; foutje:=true; end;
  system.val(edtConiferouswood_relCa.text,untProcesses.relCa_Coniferouswood,code);
  if code<>0 then begin edtConiferouswood_relCa.text:='number!'; foutje:=true; end;
  system.val(edtConiferouswood_relNa.text,untProcesses.relNa_Coniferouswood,code);
  if code<>0 then begin edtConiferouswood_relNa.text:='number!'; foutje:=true; end;
  system.val(edtConiferouswood_relK.text,untProcesses.relK_Coniferouswood,code);
  if code<>0 then begin edtConiferouswood_relK.text:='number!'; foutje:=true; end;
  system.val(edtConiferouswood_relMg.text,untProcesses.relMg_Coniferouswood,code);
  if code<>0 then begin edtConiferouswood_relMg.text:='number!'; foutje:=true; end;
  system.val(edtConiferouswood_relAl.text,untProcesses.relAl_Coniferouswood,code);
  if code<>0 then begin edtConiferouswood_relAl.text:='number!'; foutje:=true; end;
  system.val(edtDeciduouswood_relCa.text,untProcesses.relCa_Deciduouswood,code);
  if code<>0 then begin edtDeciduouswood_relCa.text:='number!'; foutje:=true; end;
  system.val(edtDeciduouswood_relNa.text,untProcesses.relNa_Deciduouswood,code);
  if code<>0 then begin edtDeciduouswood_relNa.text:='number!'; foutje:=true; end;
  system.val(edtDeciduouswood_relK.text,untProcesses.relK_Deciduouswood,code);
  if code<>0 then begin edtDeciduouswood_relK.text:='number!'; foutje:=true; end;
  system.val(edtDeciduouswood_relMg.text,untProcesses.relMg_Deciduouswood,code);
  if code<>0 then begin edtDeciduouswood_relMg.text:='number!'; foutje:=true; end;
  system.val(edtDeciduouswood_relAl.text,untProcesses.relAl_Deciduouswood,code);
  if code<>0 then begin edtDeciduouswood_relAl.text:='number!'; foutje:=true; end;
  for col:=1 to strgrdMinDisPar.ColCount-1 do for row:=1 to strgrdMinDisPar.ColCount-1 do
  begin
      system.Val(strgrdMinDisPar.Cells[col,row],untProcesses.MinDisPar[col,row],code);
      if code<>0 then strgrdMinDisPar.Cells[col,row]:='number!';
  end;
  {hiero}
  untSoftConstants.frmSoftConstants.Refresh;
  if not(foutje) then
  begin
    scaled:=false;
    sb1:=untProcesses.frEctoLitter_grassscrub+untProcesses.frEndoLitter_grassscrub;
    if sb1<>1 then
    begin
      scaled:=true;
      untProcesses.frEctoLitter_grassscrub:=untProcesses.frEctoLitter_grassscrub/sb1;
      untProcesses.frEndoLitter_grassscrub:=untProcesses.frEndoLitter_grassscrub/sb1;
    end;
    sb1:=untProcesses.frEctoLitter_deciduouswood+untProcesses.frEndoLitter_deciduouswood;
    if sb1<>1 then
    begin
      scaled:=true;
      untProcesses.frEctoLitter_deciduouswood:=untProcesses.frEctoLitter_deciduouswood/sb1;
      untProcesses.frEndoLitter_deciduouswood:=untProcesses.frEndoLitter_deciduouswood/sb1;
    end;
    sb1:=untProcesses.frEctoLitter_coniferouswood+untProcesses.frEndoLitter_coniferouswood;
    if sb1<>1 then
    begin
      scaled:=true;
      untProcesses.frEctoLitter_coniferouswood:=untProcesses.frEctoLitter_coniferouswood/sb1;
      untProcesses.frEndoLitter_coniferouswood:=untProcesses.frEndoLitter_coniferouswood/sb1;
    end;
    sb1:=untProcesses.frEctoLitter_agriculture+untProcesses.frEndoLitter_agriculture;
    if sb1<>1 then
    begin
      scaled:=true;
      untProcesses.frEctoLitter_agriculture:=untProcesses.frEctoLitter_agriculture/sb1;
      untProcesses.frEndoLitter_agriculture:=untProcesses.frEndoLitter_agriculture/sb1;
    end;
    if scaled
    then showmessage('fractions ecto-organic and endo-organic litter were rescaled to sum to 1');
    {save old version softconstants.txt}
    if fileexists('SoftConstants.txt')
    then copyfile(PChar(AppendPathDelim(getcurrentdir)+'SoftConstants.txt'),
                  PChar(AppendPathDelim(getcurrentdir)+'OldSoftConstants.txt'),false);
    {write new file SoftConstants.txt}
    system.assign(fsc,AppendPathDelim(getcurrentdir)+''+'SoftConstants.txt');
    system.rewrite(fsc);
    system.writeln(fsc,'[Only change numbers, not text. Line order not critical.]');
    {C-cycle parameters}
    system.writeln(fsc,'[C-cycle parameters]');
    str(untProcesses.DPM2RPM_grassscrub,txt);
    txt:=txt+' grassscrub decomposable / resistant plant material ratio';
    system.writeln(fsc,txt);
    str(untProcesses.frEctoLitter_grassscrub,txt);
    txt:=txt+' grassscrub fraction of litter that is ecto-organic';
    system.writeln(fsc,txt);
    str(untProcesses.frEndoLitter_grassscrub,txt);
    txt:=txt+' grassscrub fraction of litter that is endo-organic';
    system.writeln(fsc,txt);
    str(untProcesses.frPINT_grassscrub,txt);
    txt:=txt+' grassscrub: fraction of Precipitation lost by Interception Evaporation';
    system.writeln(fsc,txt);
    str(untProcesses.DPM2RPM_deciduouswood,txt);
    txt:=txt+' deciduouswood decomposable / resistant plant material ratio';
    system.writeln(fsc,txt);
    str(untProcesses.frEctoLitter_deciduouswood,txt);
    txt:=txt+' deciduouswood fraction of litter that is ecto-organic';
    system.writeln(fsc,txt);
    str(untProcesses.frEndoLitter_deciduouswood,txt);
    txt:=txt+' deciduouswood fraction of litter that is endo-organic';
    system.writeln(fsc,txt);
    str(untProcesses.frPINT_deciduouswood,txt);
    txt:=txt+' deciduouswood: fraction of Precipitation lost by Interception Evaporation'; 
    system.writeln(fsc,txt);
    str(untProcesses.DPM2RPM_coniferouswood,txt);
    txt:=txt+' coniferouswood decomposable / resistant plant material ratio';
    system.writeln(fsc,txt);
    str(untProcesses.frEctoLitter_coniferouswood,txt);
    txt:=txt+' coniferouswood fraction of litter that is ecto-organic';
    system.writeln(fsc,txt);
    str(untProcesses.frEndoLitter_coniferouswood,txt);
    txt:=txt+' coniferouswood fraction of litter that is endo-organic';
    system.writeln(fsc,txt);
    str(untProcesses.frPINT_coniferouswood,txt);
    txt:=txt+' coniferouswood: fraction of Precipitation lost by Interception Evaporation'; 
    system.writeln(fsc,txt);
    str(untProcesses.DPM2RPM_agriculture,txt);
    txt:=txt+' agriculture decomposable / resistant plant material ratio';
    system.writeln(fsc,txt);
    str(untProcesses.frEctoLitter_agriculture,txt);
    txt:=txt+' agriculture fraction of litter that is ecto-organic';
    system.writeln(fsc,txt);
    str(untProcesses.frEndoLitter_agriculture,txt);
    txt:=txt+' agriculture fraction of litter that is endo-organic';
    system.writeln(fsc,txt); 
    str(untProcesses.frPINT_agriculture,txt);
    txt:=txt+' agriculture: fraction of Precipitation lost by Interception Evaporation';
    system.writeln(fsc,txt); 
    str(untProcesses.k_DPM,txt);
    txt:=txt+' rate for decomposable plant material pool';
    system.writeln(fsc,txt); 
    str(untProcesses.k_RPM,txt);
    txt:=txt+' rate for resistant plant material pool';
    system.writeln(fsc,txt);
    str(untProcesses.k_BIO,txt);
    txt:=txt+' rate for biomass pool';
    system.writeln(fsc,txt);
    str(untProcesses.k_HUM,txt);
    txt:=txt+' rate for humus pool';
    system.writeln(fsc,txt);
    str(untProcesses.facscaleCO2BIOHUM,txt);
    txt:=txt+' scaling factor for CO2/(BIO+HUM) ratio';
    system.writeln(fsc,txt);
    str(untProcesses.BIO2HUMratio,txt);
    txt:=txt+' BIO/HUM ratio';
    system.writeln(fsc,txt);
    system.writeln(fsc,'[clay migration parameters]');
    str(untProcesses.clay_kd,txt);
    txt:=txt+' kd: soil detachability coefficient (g.J-1)';
    system.writeln(fsc,txt);
    str(untProcesses.clay_kr,txt);
    txt:=txt+' kr: replenishment rate coefficient (g.m-2.h-1)';
    system.writeln(fsc,txt);
    str(untProcesses.clay_hthetamacro,txt);
    txt:=txt+' h-theta(macro): pressure head (hPa) at which macropores empty';
    system.writeln(fsc,txt);
    str(untProcesses.CM_n,txt);
    txt:=txt+' n: filtering coefficient (-)';
    system.writeln(fsc,txt);
    str(untProcesses.CM_fref,txt);
    txt:=txt+' fref: reference filter coefficient (m-1)';
    system.writeln(fsc,txt);
    str(untProcesses.CM_vref,txt);
    txt:=txt+' vref: pore water velocity (m.h-1) at which fref is measured';
    system.writeln(fsc,txt);
    str(untProcesses.clay_ectobd,txt);
    txt:=txt+' bulk density (kg.dm-3) of ectorganic layers';
    system.writeln(fsc,txt);
    str(untProcesses.clay_ectothick,txt);
    txt:=txt+' thickness (mm) of ectorganic layer at which no more splash occurs';
    system.writeln(fsc,txt);
    system.writeln(fsc,'[parameters of physical weathering]');
    str(untGUI.PW_Pmax,txt);
    txt:=txt+' Max. split probability';
    system.writeln(fsc,txt);
    str(untGUI.PW_dTdtPmax,txt);
    txt:=txt+' Temperature gradient (oC/h) with max. split probability';
    system.writeln(fsc,txt);
    str(untProcesses.frStoneSplit,txt); untGUI.PW_frStoneSplit:=untProcesses.frStoneSplit;
    txt:=txt+' Fraction of coarse material entering the gravel fraction (frost spalling or root cracking events)';
    system.writeln(fsc,txt);
    system.writeln(fsc,'[Cation uptake forcing function (target concentrations)]');
    str(untProcesses.relAl_Agriculture,txt);
    txt:=txt+' Relative Al contribution Agriculture';
    system.writeln(fsc,txt);
    str(untProcesses.relCa_Agriculture,txt);
    txt:=txt+' Relative Ca contribution Agriculture';
    system.writeln(fsc,txt);
    str(untProcesses.relNa_Agriculture,txt);
    txt:=txt+' Relative Na contribution Agriculture';
    system.writeln(fsc,txt);
    str(untProcesses.relMg_Agriculture,txt);
    txt:=txt+' Relative Mg contribution Agriculture';
    system.writeln(fsc,txt);
    str(untProcesses.relK_Agriculture,txt);
    txt:=txt+' Relative K contribution Agriculture';
    system.writeln(fsc,txt);
    str(untProcesses.relAl_Deciduouswood,txt);
    txt:=txt+' Relative Al contribution Deciduouswood';
    system.writeln(fsc,txt);
    str(untProcesses.relCa_Deciduouswood,txt);
    txt:=txt+' Relative Ca contribution Deciduouswood';
    system.writeln(fsc,txt);
    str(untProcesses.relNa_Deciduouswood,txt);
    txt:=txt+' Relative Na contribution Deciduouswood';
    system.writeln(fsc,txt);
    str(untProcesses.relMg_Deciduouswood,txt);
    txt:=txt+' Relative Mg contribution Deciduouswood';
    system.writeln(fsc,txt);
    str(untProcesses.relK_Deciduouswood,txt);
    txt:=txt+' Relative K contribution Deciduouswood';
    system.writeln(fsc,txt);    
    str(untProcesses.relAl_Coniferouswood,txt);
    txt:=txt+' Relative Al contribution Coniferouswood';
    system.writeln(fsc,txt);
    str(untProcesses.relCa_Coniferouswood,txt);
    txt:=txt+' Relative Ca contribution Coniferouswood';
    system.writeln(fsc,txt);
    str(untProcesses.relNa_Coniferouswood,txt);
    txt:=txt+' Relative Na contribution Coniferouswood';
    system.writeln(fsc,txt);
    str(untProcesses.relMg_Coniferouswood,txt);
    txt:=txt+' Relative Mg contribution Coniferouswood';
    system.writeln(fsc,txt);
    str(untProcesses.relK_Coniferouswood,txt);
    txt:=txt+' Relative K contribution Coniferouswood';
    system.writeln(fsc,txt);  
    str(untProcesses.relAl_Grassscrub,txt);
    txt:=txt+' Relative Al contribution Grassscrub';
    system.writeln(fsc,txt);
    str(untProcesses.relCa_Grassscrub,txt);
    txt:=txt+' Relative Ca contribution Grassscrub';
    system.writeln(fsc,txt);
    str(untProcesses.relNa_Grassscrub,txt);
    txt:=txt+' Relative Na contribution Grassscrub';
    system.writeln(fsc,txt);
    str(untProcesses.relMg_Grassscrub,txt);
    txt:=txt+' Relative Mg contribution Grassscrub';
    system.writeln(fsc,txt);
    str(untProcesses.relK_Grassscrub,txt);
    txt:=txt+' Relative K contribution Grassscrub';
    system.writeln(fsc,txt);
    system.writeln(fsc,'[Mineral dissolution parameters]');
    for row:=1 to strgrdMinDisPar.RowCount-1 do
    begin
      for col:=1 to strgrdMinDisPar.ColCount-1 do
      begin
        system.val(strgrdMinDisPar.Cells[col,row],dum1,code);
        system.writeln(fsc,dum1,'  ',
                           strgrdMinDispar.Cells[0,row],': ',
                           strgrdMinDispar.Cells[col,0]);
      end;
    end;
    {hiero}
    closefile(fsc);
    showmessage('New version of SoftConstants.txt created.'+#13#10+'Old version renamed to OldSoftConstants.txt');
    {untSoftConstants.frmSoftConstants.}Close;
  end
  else showmessage('Only numbers allowed!'+#13#10+'Re-enter values or press other button.');
end;

procedure TfrmSoftConstants.btbttnSoftConstantsCancelClick(
  Sender: TObject);
begin
  {untSoftConstants.frmSoftConstants.}Close;
end;

procedure TfrmSoftConstants.tbcntrlGroupsChange(Sender: TObject);
begin
   case tbcntrlGroups.tabindex of
     0:begin
         grpbxCcycleParameters.Visible:=true;
         grpbxCationUptakeTargetValues.Visible:=false;
         grpbxPhysicoChemicalParameters.Visible:=false;
         grpbxMineralDissolutionParameters.Visible:=false;
       end;
     1:begin
         grpbxCcycleParameters.Visible:=false;
         grpbxCationUptakeTargetValues.Visible:=true;
         grpbxPhysicoChemicalParameters.Visible:=false;
         grpbxMineralDissolutionParameters.Visible:=false;
       end;
     2:begin
         grpbxCcycleParameters.Visible:=false;
         grpbxCationUptakeTargetValues.Visible:=false;
         grpbxPhysicoChemicalParameters.Visible:=true;
         grpbxMineralDissolutionParameters.Visible:=false;
       end;
     3:begin
         grpbxCcycleParameters.Visible:=false;
         grpbxCationUptakeTargetValues.Visible:=false;
         grpbxPhysicoChemicalParameters.Visible:=false;
         grpbxMineralDissolutionParameters.Visible:=true;
       end;
   end;
end;

end.
