unit untEditChemConstants;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$mode Delphi}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, Grids, Buttons,
  StdCtrls;

type

  { TfrmEditChemConstants }

  TfrmEditChemConstants = class(TForm)
    btbttnQuitNoChange: TBitBtn;
    btbttnQuitWithChanges: TBitBtn;
    EQout: TEdit;
    Label53: TLabel;
    Label54: TLabel;
    strgrdChemConstants: TStringGrid;
    procedure btbttnQuitNoChangeClick(Sender: TObject);
    procedure btbttnQuitWithChangesClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure strgrdChemConstantsValidateEntry(Sender: TObject; aCol,
      aRow: Integer; const OldValue: string; var NewValue: String);
  private

  public

  end;

var
  frmEditChemConstants: TfrmEditChemConstants;

implementation

{$R *.lfm}
uses untGUI;
var fin,fout:textfile;
  linecounter:integer;

{ TfrmEditChemConstants }

procedure TfrmEditChemConstants.FormActivate(Sender: TObject);
var dum1,dum2:real; txt,txt1,txt2:string;
begin
  system.assign(fin,untGUI.EQFileName); system.reset(fin);
  strgrdChemConstants.Cells[0,0]:='Equilibrium';
  strgrdChemConstants.Cells[1,0]:='Value of constant';
  strgrdChemConstants.Cells[2,0]:='Standard reaction enthalpy ΔHR (kJ mol-1)';
  linecounter:=1;
  repeat
    system.readln(fin,dum1,dum2,txt);
    str(dum1,txt1); str(dum2,txt2);
    txt:=trim(txt);
    if NOT((pos('EQUILIBRIUM CONSTANT',txt)>0)or(pos('DO NOT CHANGE TEXT',txt)>0)or(pos('END OF FILE',txt)>0))
    then
    begin
      strgrdChemConstants.Cells[0,linecounter]:=txt;
      strgrdChemConstants.Cells[1,linecounter]:=txt1;
      strgrdChemConstants.Cells[2,linecounter]:=txt2;
      linecounter:=linecounter+1;
      strgrdChemConstants.RowCount:=linecounter+1;
    end;
  until txt='END OF FILE';
  closefile(fin);
  strgrdChemConstants.RowCount:=linecounter;
  strgrdChemConstants.AutoSizeColumns;
  strgrdChemConstants.Refresh;
  EQout.Text:=untGUI.EQFileName; EQout.Refresh;
end;

procedure TfrmEditChemConstants.strgrdChemConstantsValidateEntry(
  Sender: TObject; aCol, aRow: Integer; const OldValue: string;
  var NewValue: String);
var code:integer; value:real;
begin
   val(strgrdChemConstants.Cells[aCol,aRow],Value,code);
   if code<>0 then
   begin
     strgrdChemConstants.Cells[aCol,aRow]:=OldValue;
     showmessage('You did not enter a number!');
   end
   else strgrdChemConstants.Cells[aCol,aRow]:=NewValue;  ;
end;

procedure TfrmEditChemConstants.btbttnQuitNoChangeClick(Sender: TObject);
begin
  close;
end;

procedure TfrmEditChemConstants.btbttnQuitWithChangesClick(Sender: TObject);
var i:integer;
begin
  system.assign(fout,EQout.Text); system.rewrite(fout);
  system.writeln(fout,'0   0   EQUILIBRIUM CONSTANT + dH (kJ.mol-1) FILE FOR SOILGEN (all values for 25oC; dH used for temperature corrections)');
  system.writeln(fout,'0   0   DO NOT CHANGE TEXT, ONLY NUMBERS. LINE ORDER IS NOT CRITICAL.');
  for i:=1 to linecounter-1 do system.writeln(fout,strgrdChemConstants.Cells[1,i],' ',
                                                 strgrdChemConstants.Cells[2,i],' ',
                                                 strgrdChemConstants.Cells[0,i]);
  system.writeln(fout,'0   0   END OF FILE');
  closefile(fout);
  untGUI.frmMAIN.ReadDataEQ(EQout.Text);
  untGUI.frmMAIN.mmEquilibrium.Lines.LoadFromFile(EQout.Text);
  untGUI.frmMAIN.mmEquilibrium.Enabled:=true;
  close;
end;

end.

