unit untEditCinputs;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$mode Delphi}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, Buttons,
  Grids;

type

  { TfrmEditCinputs }

  TfrmEditCinputs = class(TForm)
    btbttnQuitNoChange: TBitBtn;
    btbttnQuitWithChanges: TBitBtn;
    COut: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label53: TLabel;
    strgrdCPools: TStringGrid;
    strgrdVegetationData: TStringGrid;
    procedure btbttnQuitNoChangeClick(Sender: TObject);
    procedure btbttnQuitWithChangesClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure strgrdCPoolsValidateEntry(Sender: TObject; aCol, aRow: Integer;
      const OldValue: string; var NewValue: String);
    procedure strgrdVegetationDataValidateEntry(Sender: TObject; aCol,
      aRow: Integer; const OldValue: string; var NewValue: String);
  private

  public

  end;

var
  frmEditCinputs: TfrmEditCinputs;
  fin,fout:textfile;
  linecounter:integer;

implementation

{$R *.lfm}
uses untGUI;

{ TfrmEditCinputs }

procedure TfrmEditCinputs.FormActivate(Sender: TObject);
var i:integer; txt:string;
begin
  strgrdCPools.Cells[0,0]:='DPM';  strgrdCPools.Cells[1,0]:='RPM';  
  strgrdCPools.Cells[2,0]:='BIO';  strgrdCPools.Cells[3,0]:='HUM';
  strgrdCPools.Cells[4,0]:='IOM';
  for i:=1 to 5 do
  begin
    str(DataInit.RothCGen.GENPoolPercs[i]:6:2,txt); strgrdCPools.Cells[i-1,1]:=txt;
  end;
  strgrdVegetationData.Cells[0,0]:='';              strgrdVegetationData.Cells[0,1]:='';
  strgrdVegetationData.Cells[1,0]:='Grassscrub';    strgrdVegetationData.Cells[2,0]:='Grassscrub';
  strgrdVegetationData.Cells[3,0]:='Deciduouswood'; strgrdVegetationData.Cells[4,0]:='Deciduouswood';
  strgrdVegetationData.Cells[5,0]:='Coniferouswood';strgrdVegetationData.Cells[6,0]:='Coniferouswood';
  strgrdVegetationData.Cells[7,0]:='Agriculture';   strgrdVegetationData.Cells[8,0]:='Agriculture';        
  strgrdVegetationData.Cells[1,1]:='litter';        strgrdVegetationData.Cells[2,1]:='manure';
  strgrdVegetationData.Cells[3,1]:='litter';        strgrdVegetationData.Cells[4,1]:='manure';
  strgrdVegetationData.Cells[5,1]:='litter';        strgrdVegetationData.Cells[6,1]:='manure';
  strgrdVegetationData.Cells[7,1]:='crop residue';  strgrdVegetationData.Cells[8,1]:='manure';
  for i:=1 to 12 do
  begin
    str(i,txt); txt:='Month '+txt;
    strgrdVegetationData.Cells[0,i+1]:=txt;
  end;  
  strgrdVegetationData.Cells[0,14]:= 'Maximum rootable depth (mm)';

  for i:=1 to 12 do
  begin
    str(DataInit.RothCGen.GENgrasPlant[i]:6:2,txt); strgrdVegetationData.Cells[1,i+1]:=txt;
    str(DataInit.RothCGen.GENgrasMan[i]:6:2,txt);   strgrdVegetationData.Cells[2,i+1]:=txt;
  end;
  str(DataInit.RothCGen.GENgrasRDEPTH:6:2,txt);     strgrdVegetationData.Cells[1,14]:=txt;
  for i:=1 to 12 do
  begin
    str(DataInit.RothCGen.GENdeciPlant[i]:6:2,txt); strgrdVegetationData.Cells[3,i+1]:=txt;
    str(DataInit.RothCGen.GENdeciMan[i]:6:2,txt);   strgrdVegetationData.Cells[4,i+1]:=txt;
  end;
  str(DataInit.RothCGen.GENdeciRDEPTH:6:2,txt);     strgrdVegetationData.Cells[3,14]:=txt;  
  for i:=1 to 12 do
  begin
    str(DataInit.RothCGen.GENconiPlant[i]:6:2,txt); strgrdVegetationData.Cells[5,i+1]:=txt;
    str(DataInit.RothCGen.GENconiMan[i]:6:2,txt);   strgrdVegetationData.Cells[6,i+1]:=txt;
  end;
  str(DataInit.RothCGen.GENconiRDEPTH:6:2,txt);     strgrdVegetationData.Cells[5,14]:=txt;
  for i:=1 to 12 do
  begin
    str(DataInit.RothCGen.GENagriPlant[i]:6:2,txt); strgrdVegetationData.Cells[7,i+1]:=txt;
    str(DataInit.RothCGen.GENagriMan[i]:6:2,txt);   strgrdVegetationData.Cells[8,i+1]:=txt;
  end;
  str(DataInit.RothCGen.GENagriRDEPTH:6:2,txt);     strgrdVegetationData.Cells[7,14]:=txt;

  strgrdVegetationData.AutoSizeColumns;
  strgrdVegetationData.Refresh;
  COut.Text:=untGUI.FirstRothCFileName;
end;

procedure TfrmEditCinputs.strgrdCPoolsValidateEntry(Sender: TObject; aCol,
  aRow: Integer; const OldValue: string; var NewValue: String);
  var code:integer; value:real;
begin
   val(strgrdCPools.Cells[aCol,aRow],Value,code);
   if code<>0 then
   begin
     strgrdCPools.Cells[aCol,aRow]:=OldValue;
     showmessage('You did not enter a number!');
   end
   else strgrdCPools.Cells[aCol,aRow]:=NewValue;  ;
end;

procedure TfrmEditCinputs.strgrdVegetationDataValidateEntry(Sender: TObject;
  aCol, aRow: Integer; const OldValue: string; var NewValue: String);
  var code:integer; value:real;
begin
   val(strgrdVegetationData.Cells[aCol,aRow],Value,code);
   if code<>0 then
   begin
     strgrdVegetationData.Cells[aCol,aRow]:=OldValue;
     showmessage('You did not enter a number!');
   end
   else strgrdVegetationData.Cells[aCol,aRow]:=NewValue;  ;
end;

procedure TfrmEditCinputs.btbttnQuitNoChangeClick(Sender: TObject);
begin
  close;
end;

procedure TfrmEditCinputs.btbttnQuitWithChangesClick(Sender: TObject);
var i:integer;
begin
  system.assign(fout,COut.Text); system.rewrite(fout);
  system.writeln(fout,'Data file with C-turnover data for SoilGen');
  system.writeln(fout,'C-turnover modelled using the approach of Roth-C26.3 model. Refs:  ');
  system.writeln(fout,'Jenkinson, D.S., Coleman, K., 1994. Calculating the annual input of organic');
  system.writeln(fout,'                                    matter to soil from measurements of total organic carbon  ');
  system.writeln(fout,'                                    and radiocarbon. European Journal of Soil Science 45, 167-174 ');
  system.writeln(fout,'Coleman, K., Jenkinson, D.S., 2005. RothC-26.3: a model for the turnover of carbon in soil.');
  system.writeln(fout,'                                    Model Description and Users Guide. November 1999 Issue ');
  system.writeln(fout,'                                    (Modified April 2005). ');
  system.writeln(fout,'                                    http://www.rothamsted.bbsrc.ac.uk/aen/carbon/download.htm ');
  system.writeln(fout,'DO NOT CHANGE TEXT NOR LINE ORDER, ONLY CHANGE NUMBERS  ');
  system.writeln(fout,'[initial profile data: distribution C over various pools (%) (all soil compartments)]  ');
  system.writeln(fout,'[DPMpool  RPMpool  BIOpool  HUMpool  IOMpool] ');
  for i:=0 to 4 do system.write(fout,strgrdCpools.Cells[i,1],' ');
  system.writeln(fout);
  system.writeln(fout,'[monthly inputs: distribution of inputs (%) of PlantResidues and Manure for all vegetations per month]');
  system.writeln(fout,'[month PlantResiduesIN ManureIN for grassscrub] ');
  for i:=1 to 12 do
  begin
    str(i:2,txt);
    system.writeln(fout,txt,' ',strgrdVegetationData.Cells[1,i+1],' ',strgrdVegetationData.Cells[2,i+1]);
  end;
  system.writeln(fout,strgrdVegetationData.Cells[1,14],' [mm maximum rootable depth]');
  system.writeln(fout,'[month PlantResiduesIN ManureIN for deciduouswood] ');
  for i:=1 to 12 do
  begin
    str(i:2,txt);
    system.writeln(fout,txt,' ',strgrdVegetationData.Cells[3,i+1],' ',strgrdVegetationData.Cells[4,i+1]);
  end;
  system.writeln(fout,strgrdVegetationData.Cells[3,14],' [mm maximum rootable depth]');
  system.writeln(fout,'[month PlantResiduesIN ManureIN for coniferouswood] ');
  for i:=1 to 12 do
  begin
    str(i:2,txt);
    system.writeln(fout,txt,' ',strgrdVegetationData.Cells[5,i+1],' ',strgrdVegetationData.Cells[6,i+1]);
  end;
  system.writeln(fout,strgrdVegetationData.Cells[5,14],' [mm maximum rootable depth]');
  system.writeln(fout,'[month PlantResiduesIN ManureIN for agriculture] ');
  for i:=1 to 12 do
  begin
    str(i:2,txt);
    system.writeln(fout,txt,' ',strgrdVegetationData.Cells[7,i+1],' ',strgrdVegetationData.Cells[8,i+1]);
  end;
  system.writeln(fout,strgrdVegetationData.Cells[7,14],' [mm maximum rootable depth]');
  closefile(fout);
  untGUI.frmMAIN.ReadDataC(COut.Text);
  untGUI.frmMAIN.mmDefaultCModel.Lines.LoadFromFile(COut.Text);
  untGUI.frmMAIN.mmDefaultCModel.Enabled:=true;
  close;
end;

end.

