unit untSoilGenConst;
{**************************************************************************************}
{**************************** This file is part of SoilGen ****************************}
{****************************SoilGen is free software:*********************************}
{you can redistribute it and/or modify it under the terms******************************}
{of the GNU General Public License as published by the Free Software Foundation,*******}
{either version 3 of the License, or (at your option) any later version.***************}
{  It would be polite to refer to the SoilGen model by mentioning the documentation:  }
{  Peter Finke. 2024. Modelling soil development under global change.                 }
{  SpringerBriefs in Earth System Sciences 10032. ISBN: 978-3-031-55582-4.            }
{  https://doi.org/10.1007/978-3-031-55583-1                                          }
{**************************************************************************************}
{SoilGen is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;**}
{without even the implied warranty of MERCHANTABILITY or ******************************}
{FITNESS FOR A PARTICULAR PURPOSE.*****************************************************}
{See the GNU General Public License for more details.**********************************}
{**************************************************************************************}
{You should have received a copy of the GNU General Public License along with SoilGen.*}
{If not, see <https://www.gnu.org/licenses/>.******************************************}
{$mode Delphi}

interface

uses
  Classes, SysUtils;

CONST
  {these constants are used in more than 1 other units.
   Constants locally used in 1 unit are declared in these units.}
  MaxLayers=202;   {maximum numer of model nodes (eqv compartments) plus the top and bottom node}
  MaxChem=25;      {maximum number of chemicals, 23 of these are actually used:
                    1=Ca++; 2=Mg++; 3=Na+; 4=K+; 5=Cl-; 6:=SO4--; 7=HCO3-; 8=CO3--;
                    9=balion+/-; 10=H+; 11=OH-; 12=Al+++; 13=Fe+++; 14=Fe++; 15=H4SiO4s;
                    16=DOC-0;
                    17-23 is dispersible matter (part of clay mass and exchange-species):
                    17=Clay; XCa-=18; XMg=19; XNa:=20; XK=21; XH:=22; XAl:=23}
  MaxNoIrrig=366;  {maximum number of irrigation events in a standard (leap) year}
  MaxNoChemApp=366;{maximum number of chemical applications (fertilization) in a standard (leap) year}
  MaxNoWeeks=53;   {maximum number of weeks in a standard year}
  MaxNoYears=70000;{maximum number of years in a simulation run. More than 30000 is not advised.}
  MaxNoCrops=5;    {maximum number of crops in a standard (agriculture) year}
  MaxNoCultiv=10;  {maximum number of cultivation events in a standard year. Usually there is 1 such event)}
  NrMinerals=17;   {number of minerals:
                     1 Albite Na Al Si3 O8
                     2 K-feldspar K Al Si3 O8
                     3 Muscovite K Al3 Si3 O10 (OH)2
                     4 Biotite K Mg1.5 Fe1.5 Al Si3 O10 (OH)2
                     5 Quartz Si O2
                     6 Clorite Clinochlore Mg5 Al2 Si3 O10 (OH)8
                     7 Anorthite Ca Al2 Si2 O8
                     8 Forsterite Mg2 Si O4
                     9 Augite orthoferrosilite Fe Si O3
                    10 Kaolinite Al2 Si2 O5 (OH)4
                    11 Illite Si3.4 O10 (OH)2 Mg0.02 Al2.38 Ca0.01 K0.44, Godderis et al 2006
                    12 Montmorillonite Si4 O10 (OH)2 Mg0.33 Al1.67 Ca0.04125 Mg0.04125 K0.0825 Na0.0825
                    13 Hornblende Ca2 Mg4 Al Si6.7 Al O22.4 from Profile model
                    14 Fayalite Fe2 Si O4
                    15 amorphite
                    16 otherite
                    17 Gibbsite Al (OH)3
                    Calcite CaCO3 and Gypsum CaSO4 are handled separately}
  NrParams=6;      {number of parameters to describe the weathering of these minerals:
                    param1=log KH25;  param2=kEaH (activ.Energ.acid);     param3=reaction order acid (n);
                    param4=log KOH25; param5=kEaOH (activ.Energ.alkalic); param6=reaction order alkalic (m)}


implementation

end.

